/*---------------------------------------------------------------------+
| Module-name:                                                         |
|        caller1.c                                                     |
|                                                                      |
| Function:                                                            |
|        Invokes the series of ADSM APIs necessary to backup an object.|
+---------------------------------------------------------------------*/
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "dsmrc.h"
#include "dsmapitd.h"
#include "dsmapifp.h"

main()
{
int                rc;
int                i ;
uint32             handle; 
char               options[100] ;
char               configfile[10] ;
mcBindKey          mcBindKey;
ObjAttr            objAttrArea;
DataBlk            dataBlkArea, qDataBlkArea, getBlk ;
qryBackupData      queryBuffer;
qryRespBackupData  qbDataArea;
uint16             reason ;
dsmQueryType       queryType;
dsmGetList         dsmObjList ;
dsmObjName         objName ;
regFSData          regFilespace ;
dsmApiVersion      dsmApiVer ;
dsmApiVersion      apiLibVer ;
char               *rcMsg ;

rcMsg = (char *)malloc(DSM_MAX_RC_MSG_LENGTH) ;

/************ Build File Space ****************************************/
regFilespace.fsName = (char *)malloc(100) ;
regFilespace.fsType = (char *)malloc(100) ;

strcpy(regFilespace.fsName,"/testfs") ;
strcpy(regFilespace.fsType,"CALLER1_FS") ;
regFilespace.capacity.lo = 0;
regFilespace.capacity.hi = 0;
regFilespace.occupancy.lo = 0;
regFilespace.occupancy.hi = 0;
regFilespace.stVersion = regFSDataVersion ;
regFilespace.fsAttr.unixFSAttr.fsInfoLength = 6 ;
strcpy(regFilespace.fsAttr.unixFSAttr.fsInfo,"fsinfo") ;

/***********************************************************************/
dsmObjList.stVersion = dsmGetListVersion ;
dsmObjList.numObjId = 1 ;
dsmObjList.objId = (ObjID *)malloc(sizeof(ObjID) * dsmObjList.numObjId) ;
 
/*********************************************************************/
getBlk.bufferPtr = (char *)malloc(100) ;
getBlk.bufferLen = 100 ;
getBlk.stVersion = DataBlkVersion ;

/*==========================================================================*/
strcpy(objName.fs,"/testfs") ;
strcpy(objName.hl,"/testhl") ;
strcpy(objName.ll,"/testll") ;
objName.objType = DSM_OBJ_FILE ;

/*==========================================================================*/
strcpy(objAttrArea.owner,"logon_id");       
objAttrArea.sizeEstimate.hi = 0;       /* size estimate of object */
objAttrArea.sizeEstimate.lo = 20;       /* size estimate of object */
objAttrArea.objCompressed = bFalse ;  
objAttrArea.objInfoLength = 10;          /* length of objInfo */
objAttrArea.objInfo = (char *)malloc(objAttrArea.objInfoLength + 1) ;
strcpy(objAttrArea.objInfo,"abcdefghij");      /* object-dependent info */
objAttrArea.stVersion = ObjAttrVersion ;

/*==========================================================================*/
dataBlkArea.bufferPtr = (char *)malloc(21) ;
dataBlkArea.stVersion = DataBlkVersion ;
dataBlkArea.bufferLen = 20;              /* buffer length in bytes */
strcpy(dataBlkArea.bufferPtr,"12345678901234567890"); /* datastream to backup */

/* The following 2 can be updated optionally */
options[0] = '\0';                          /* options string */
configfile[0]= '\0';                        /* API config file for dsInit */

/*==========================================================================*/
/* check for compatibility problems                                         */

memset(&apiLibVer,0x00,sizeof(dsmApiVersion));
dsmQueryApiVersion(&apiLibVer);

if (apiLibVer.version < DSM_API_VERSION)
{ printf("\n***************************************************************\n");
  printf("The ADSM API library is at a lower version than the application.\n");
  printf("Install the current version.\n");
  printf("***************************************************************\n");
  return;
}
if ((apiLibVer.version == DSM_API_VERSION) &&
    (apiLibVer.release < DSM_API_RELEASE))
{ printf("\n***************************************************************\n");
  printf("The ADSM API library is at a lower release than the application.\n");
  printf("Proceed with caution.\n");
  printf("***************************************************************\n");
  /* Make sure you know what changes you have incorporated from the  */
  /* later release to ensure compatibility.                          */
}

/*==========================================================================*/
dsmApiVer.version = DSM_API_VERSION ;
dsmApiVer.release = DSM_API_RELEASE ;
dsmApiVer.level   = DSM_API_LEVEL   ;

/* Initialize the API session */
if ((rc = dsmInit(&handle,&dsmApiVer,NULL,NULL,NULL,NULL,configfile, options)) != 0)
{
  dsmRCMsg(handle,rc,rcMsg) ;  
  printf("error in dsmInit. rcMsg=%s\n",rcMsg);
  free(rcMsg);
  dsmTerminate(handle);
  return ;
}

/* Register the file space */
rc = dsmRegisterFS(handle, &regFilespace) ;
if ((rc != 0) && (rc != DSM_RC_FS_ALREADY_REGED))
{
  dsmRCMsg(handle,rc,rcMsg) ;  
  printf("error in dsmRegisterFS. rcMsg=%s\n", rcMsg);
  free(rcMsg);
  dsmTerminate(handle);
  return ;
}

if ((rc = dsmBeginTxn(handle)) != DSM_RC_OK)
{
  dsmRCMsg(handle,rc,rcMsg) ;  
  printf("error in dsmBeginTxn. rcMsg=%s\n", rcMsg) ;
  free(rcMsg);
  dsmTerminate(handle);
  return ;
}

mcBindKey.stVersion = mcBindKeyVersion ;
if ((rc = dsmBindMC(handle, &objName, stBackup, &mcBindKey)) != DSM_RC_OK)
{
  dsmRCMsg(handle,rc,rcMsg) ;  
  printf("error in dsmBindMC. rcMsg=%s\n", rcMsg);
  free(rcMsg);
  dsmTerminate(handle);
  return ;
}

/* Backup the data  */
if ((rc = dsmSendObj(handle, stBackup, NULL,
                   &objName,
                   &objAttrArea,
                   NULL)) != DSM_RC_OK)
{
  dsmRCMsg(handle,rc,rcMsg) ;  
  printf("error in dsmBackup. rcMsg=%s\n",rcMsg);
  free(rcMsg);
  dsmTerminate(handle);
  return ;
}

for (i=0 ; i<10 ; i++)
    if ((rc = dsmSendData(handle,&dataBlkArea)) != DSM_RC_OK)
    {
         dsmRCMsg(handle,rc,rcMsg) ;  
         printf("error in dsmSendData. rcMsg=%s\n",rcMsg);
         free(rcMsg);
         dsmTerminate(handle);
         return ;
    }

dsmEndSendObj(handle) ;


if ((rc = dsmEndTxn(handle,DSM_VOTE_COMMIT, &reason)) != DSM_RC_OK)
{
  dsmRCMsg(handle,rc,rcMsg) ;  
  printf("error in dsmEndTxn. rcMsg=%s\n", rcMsg) ;
  free(rcMsg);
  dsmTerminate(handle);
  return ;
}

queryType = qtBackup ;

qbDataArea.stVersion = qryRespBackupDataVersion ;
qDataBlkArea.stVersion = DataBlkVersion ;
qDataBlkArea.bufferPtr = (char *)&qbDataArea;
qDataBlkArea.bufferLen = sizeof(qryRespBackupData);

queryBuffer.objName = (dsmObjName *)malloc(sizeof(dsmObjName)) ;
queryBuffer.owner = (char *)malloc(100) ;

strcpy(queryBuffer.objName->fs, objName.fs);
strcpy(queryBuffer.objName->hl, objName.hl);
strcpy(queryBuffer.objName->ll, objName.ll);
queryBuffer.objName->objType = objName.objType;
/*queryBuffer.copyGroup = DSM_COPYGROUP_ANY_MATCH;*/
/*queryBuffer.mgmtClass = DSM_MGMTCLASS_ANY_MATCH;*/
strcpy(queryBuffer.owner, "logon_id");
queryBuffer.objState = DSM_ACTIVE ;
queryBuffer.stVersion = qryBackupDataVersion ;

if ((rc=dsmBeginQuery(handle,queryType, (void *)&queryBuffer )) != DSM_RC_OK)
{
  dsmRCMsg(handle,rc,rcMsg) ;  
  printf("error in dsmBeginBackupQuery. rcMsg=%s\n",rcMsg);
  free(rcMsg);
  dsmTerminate(handle);
  return ;
} 

while ((rc = dsmGetNextQObj(handle, &qDataBlkArea)) == DSM_RC_MORE_DATA)
     ;

if ((rc != DSM_RC_FINISHED) && (rc != DSM_RC_MORE_DATA))
{
  dsmRCMsg(handle,rc,rcMsg) ;  
  printf("error from dsmGetNextQObj. rcMsg=%s\n", rcMsg) ;
  free(rcMsg);
  dsmTerminate(handle);
  return ;
}

if ((rc = dsmEndQuery(handle)) != DSM_RC_OK)
{
  dsmRCMsg(handle,rc,rcMsg) ;  
  printf("error from dsmEndQuery. rcMsg=%s\n", rcMsg) ;
  free(rcMsg);
  dsmTerminate(handle);
  return ;
}

dsmObjList.objId[0].hi = qbDataArea.objId.hi ;
dsmObjList.objId[0].lo = qbDataArea.objId.lo ;

if ((rc = dsmBeginGetData(handle,bTrue,gtBackup,&dsmObjList)) != DSM_RC_OK)
{
  dsmRCMsg(handle,rc,rcMsg) ;  
  printf("error from dsmBeginGetData. rcMsg=%s\n", rcMsg) ;
  free(rcMsg);
  dsmTerminate(handle);
  return ;
}

rc = dsmGetObj(handle, &(dsmObjList.objId[0]), &getBlk) ;

if (rc == DSM_RC_FINISHED)
{
  if ((rc = dsmEndGetObj(handle)) != DSM_RC_OK)
  {
     dsmRCMsg(handle,rc,rcMsg) ;  
     printf("error from dsmEndGetObj. rcMsg=%s\n", rcMsg) ;
     free(rcMsg);
     dsmTerminate(handle);
     return ;
  }

}
else if (rc == DSM_RC_MORE_DATA)
{
  while ((rc = dsmGetData(handle, &getBlk)) == DSM_RC_MORE_DATA) 
           ;

  if (rc == DSM_RC_FINISHED)
  {
      if ((rc = dsmEndGetObj(handle)) != DSM_RC_OK)
      {
        dsmRCMsg(handle,rc,rcMsg) ;  
        printf("error from dsmEndGetObj. rcMsg=%s\n", rcMsg) ;
        free(rcMsg);
        dsmTerminate(handle);
        return ;
      }

  }
  else    
  {
     dsmRCMsg(handle,rc,rcMsg) ;  
     printf("error from dsmGetData. rcMsg=%s\n", rcMsg) ;
     free(rcMsg);
     dsmTerminate(handle);
     return ;
  }

}
else
{
  dsmRCMsg(handle,rc,rcMsg) ;  
  printf("error from dsmGetObj. rcMsg=%s\n", rcMsg) ;
  free(rcMsg);
  dsmTerminate(handle);
  return ;
}


if ((rc = dsmEndGetData(handle)) != DSM_RC_OK)
{
  dsmRCMsg(handle,rc,rcMsg) ;  
  printf("error from dsmEndGetData. rcMsg=%s\n", rcMsg) ;
  free(rcMsg);
  dsmTerminate(handle);
  return ;
}

dsmTerminate(handle);

free(regFilespace.fsName) ;
free(regFilespace.fsType) ;
free(objAttrArea.objInfo) ;
free(dsmObjList.objId) ;
free(getBlk.bufferPtr) ;
free(dataBlkArea.bufferPtr) ;
free(queryBuffer.objName) ;
free(queryBuffer.owner) ;
free(rcMsg);
printf("caller1.c is done successfully\n") ;
 
}
