/*---------------------------------------------------------------------+
| Module-name:                                                         |
|        caller2.c                                                     |
|                                                                      |
| Function:                                                            |
|        Invokes the series of ADSM APIs necessary to backup an object.|
+---------------------------------------------------------------------*/
#include <stdio.h>

#include "dsmrc.h"
#include "dsmapitd.h"
#include "dsmapifp.h"

main()
{
int                rc;
uint32             handle;
unsigned char      fstype ;
mcBindKey          mcBindKeyvar;
ObjAttr            objAttrArea;
DataBlk            dataBlkArea ;
uint16             reason ;
uint64             fsCapacity, fsOccupancy;
dsmObjName         objName ;
regFSData          regFilespace ;
dsmApiVersion      dsmApiVer ;
dsmApiVersion      apiLibVer ;
char               *rcMsg ;

rcMsg = (char *)malloc(DSM_MAX_RC_MSG_LENGTH) ;

/*=================== Build File Space Structure ===================*/
regFilespace.fsName = (char *)malloc(100) ;
regFilespace.fsType = (char *)malloc(100) ;

strcpy(regFilespace.fsName,"/testfs") ;
strcpy(regFilespace.fsType,"CALLER2_FS") ;
regFilespace.capacity.lo = 0;
regFilespace.capacity.hi = 0;
regFilespace.occupancy.lo = 0;
regFilespace.occupancy.hi = 0;
regFilespace.stVersion = regFSDataVersion ;
regFilespace.fsAttr.unixFSAttr.fsInfoLength = 6 ;
strcpy(regFilespace.fsAttr.unixFSAttr.fsInfo,"fsinfo") ;


/*================= Build Object Name Structure ===================*/
strcpy(objName.fs,"/testfs") ;
strcpy(objName.hl,"/testhl") ;
strcpy(objName.ll,"/testll") ;
objName.objType = DSM_OBJ_FILE ;


/*================= Build Object Attribute Structure ==============*/
objAttrArea.stVersion = ObjAttrVersion ;
strcpy(objAttrArea.owner,"logon_id");
objAttrArea.sizeEstimate.hi = 0;
objAttrArea.sizeEstimate.lo = 20;
objAttrArea.objCompressed = bFalse ;
objAttrArea.objInfoLength = 10;
objAttrArea.objInfo = (char *)malloc(objAttrArea.objInfoLength + 1) ;
strcpy(objAttrArea.objInfo,"abcdefghij");


/*================= Build Data Block Structure ====================*/
dataBlkArea.bufferPtr = (char *)malloc(21) ;
dataBlkArea.stVersion = DataBlkVersion ;
dataBlkArea.bufferLen = 20;
strcpy(dataBlkArea.bufferPtr,"12345678901234567890");

/*==========================================================================*/
/* check for compatibility problems                                         */

memset(&apiLibVer,0x00,sizeof(dsmApiVersion));
dsmQueryApiVersion(&apiLibVer);

if (apiLibVer.version < DSM_API_VERSION)
{ printf("\n***************************************************************\n");
  printf("The ADSM API library is at a lower version than the application.\n");
  printf("Install the current version.\n");
  printf("***************************************************************\n");
  return;
}
if ((apiLibVer.version == DSM_API_VERSION) &&
    (apiLibVer.release < DSM_API_RELEASE))
{ printf("\n***************************************************************\n");
  printf("The ADSM API library is at a lower release than the application.\n");
  printf("Proceed with caution.\n");
  printf("***************************************************************\n");
  /* Make sure you know what changes you have incorporated from the  */
  /* later release to ensure compatibility.                          */
}

/*================= Beginning of API calls for Backup =============*/

dsmApiVer.version = DSM_API_VERSION ;
dsmApiVer.release = DSM_API_RELEASE ;
dsmApiVer.level   = DSM_API_LEVEL   ;

/* Initialize the API session */
if ((rc = dsmInit(&handle,&dsmApiVer,NULL,NULL,NULL,NULL,NULL,NULL)) !=
          DSM_RC_OK )
{
  dsmRCMsg(handle,rc,rcMsg) ;
  printf("error in dsmInit. rcMsg=%s\n",rcMsg);
  free(rcMsg);
  dsmTerminate(handle);
  return ;
}


/* Register the file space */
rc = dsmRegisterFS(handle, &regFilespace) ;
if ((rc != DSM_RC_OK) && (rc != DSM_RC_FS_ALREADY_REGED))
{
  dsmRCMsg(handle,rc,rcMsg) ;
  printf("error in dsmRegisterFS. rcMsg=%s\n",rcMsg);
  free(rcMsg);
  dsmTerminate(handle);
  return ;
}

if ((rc = dsmBeginTxn(handle)) != DSM_RC_OK)
{
  dsmRCMsg(handle,rc,rcMsg) ;
  printf("error in dsmBeginTxn. rcMsg=%s\n",rcMsg);
  free(rcMsg);
  dsmTerminate(handle);
  return ;
}

mcBindKeyvar.stVersion = mcBindKeyVersion ;
if ((rc = dsmBindMC(handle, &objName, stBackup, &mcBindKeyvar)) != DSM_RC_OK)
{
  dsmRCMsg(handle,rc,rcMsg) ;
  printf("error in dsmBindMC. rcMsg=%s\n",rcMsg);
  free(rcMsg);
  dsmTerminate(handle);
  return ;
}

/* Backup the object  */
if ((rc = dsmSendObj(handle, stBackup, NULL,
                   &objName,
                   &objAttrArea,
                   NULL)) != DSM_RC_OK)
{
  dsmRCMsg(handle,rc,rcMsg) ;
  printf("error in dsmSendObj. rcMsg=%s\n",rcMsg);
  free(rcMsg);
  dsmTerminate(handle);
  return ;
}

if ((rc = dsmSendData(handle,&dataBlkArea)) != DSM_RC_OK)
{
  dsmRCMsg(handle,rc,rcMsg) ;
  printf("error in dsmSendData. rcMsg=%s\n",rcMsg);
  free(rcMsg);
  dsmTerminate(handle);
  return ;
}

if ((rc = dsmEndSendObj(handle)) != DSM_RC_OK)
{
  dsmRCMsg(handle,rc,rcMsg) ;
  printf("error in dsmEndSendObj. rcMsg=%s\n",rcMsg);
  free(rcMsg);
  dsmTerminate(handle);
  return ;
}

if ((rc = dsmEndTxn(handle,DSM_VOTE_COMMIT, &reason)) != DSM_RC_OK)
{
  dsmRCMsg(handle,rc,rcMsg) ;
  printf("error in dsmEndTxn. rcMsg=%s\n",rcMsg);
  free(rcMsg);
  dsmTerminate(handle);
  return ;
}

if ((rc = dsmTerminate(handle)) != DSM_RC_OK)
{
  dsmRCMsg(handle,rc,rcMsg) ;
  printf("error in dsmTerminate. rcMsg=%s\n",rcMsg);
  free(rcMsg);
  return ;
}

printf("ADSM API Backup was done successfully.\n") ;

free(regFilespace.fsName) ;
free(regFilespace.fsType) ;
free(dataBlkArea.bufferPtr) ;
free(objAttrArea.objInfo) ;

}

