/***********************************************************************
* Name:
*        dapiinit.c
*
* Function:
*        Perform a signon to the ADSM server via the API functions.
*
* Environment:
*        This is a PLATFORM-INDEPENDENT source file. As such it may
*        contain no dependencies on any specific operating system
*        environment or hardware platform.
*
* Description:
*        Call the dsmInit function to perform a signon to the ADSM
*        server.
***********************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef min
#define min(a,b)  (((a) < (b)) ? (a) : (b))
#endif

#include "dsmapitd.h"
#include "dsmapifp.h"
#include "dsmrc.h"
#include "dapitype.h"        /* Sample API data types.  */

#ifdef _NO_PROTO_
extern void rcApiOut();
#else
extern void rcApiOut(int16 rc);
#endif
/*----------------------------------------------------------------------+
| Public routines
.----------------------------------------------------------------------*/

/*----------------------------------------------------------------------+
| Name:    perform_signon()
|
| Action:  Do a signon.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int16 perform_signon(sel_dialog, parm1)
   dialog *sel_dialog;
   uint16 parm1;
#else
int16 perform_signon(dialog *sel_dialog,uint16 parm1)
#endif

{
char   *node;
char   *owner;
char   *pw;
char   *confFile;
char   *options;
dialog *dlg;
uint32  i;
int16   rc = 0;
char    t[100];

/* First pull all values out of the passed dialog for our use.  */
i = 0;
dlg = &sel_dialog[i];
while (dlg->item_type != DSMAPI_END)
  {
  switch (dlg->item_type)
    {
    case DSMAPI_NODENAME :
      node = dlg->item_buff;
      break;
    case DSMAPI_OWNER :
      owner = dlg->item_buff;
      break;
    case DSMAPI_PASSWORD :
      pw = dlg->item_buff;
      break;
    case DSMAPI_CONFIG   :
      confFile = dlg->item_buff;
      break;
    case DSMAPI_OPTIONS  :
      options = dlg->item_buff;
      break;
    default :
      printf("*** Signon dialog does not match code in perform_signon! ***\n");
    }
  i++;
  dlg = &sel_dialog[i];
  }

memset(&apiApplVer,0x00,sizeof(dsmApiVersion));
apiApplVer.version = DSM_API_VERSION;  /* Set the applications compile */
apiApplVer.release = DSM_API_RELEASE;  /* time version.                */
apiApplVer.level   = DSM_API_LEVEL;

printf("Doing signon for node %s, owner %s, with password %s\n",
       node,owner,pw);

rc = dsmInit(&dsmHandle,    /* Will contain session handle on retrun. */
             &apiApplVer,   /* Version of the API we are using.       */
             node,owner,pw, /* Node, owner, & password                */
             "Sample-API",  /* Name of our node type.                 */
             confFile,      /* API config file name                   */
             options);      /* option string                          */

if (rc == DSM_RC_REJECT_VERIFIER_EXPIRED)
{
   printf("*** Password expired. Select Change Password.\n");
   return(rc);
}
else
if (rc)
{
   printf("*** Init failed: ");
   rcApiOut(rc);
   dsmTerminate(dsmHandle);       /* clean up memory blocks */
   return(rc);
}

printf("Handle on return = %lu\n",dsmHandle);

/* Put code in here for expired pw, etc... */

/*----------------------------------------------------------------------+
| Once signed on get our session info block
.----------------------------------------------------------------------*/
memset(&dsmSessInfo,0x00,sizeof(ApiSessInfo));  /* Zero out block.     */
dsmSessInfo.stVersion = ApiSessInfoVersion;     /* Init struct version */
rc = dsmQuerySessInfo(dsmHandle,                /* Our session handle  */
                      &dsmSessInfo);            /* Output structure.   */

return 0;
}
