/***********************************************************************
* Name:
*        dapiproc.c
*
* Function:
*        Functions invoked via menu selections
*
* Environment:
*        This is a PLATFORM-INDEPENDENT source file. As such it may
*        contain no dependencies on any specific operating system
*        environment or hardware platform.
*
* Description:
*        The file contains the functions that are invoked from the
*        menu processing code and tables.
***********************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "dsmapitd.h"      /* ADSM API type definitions                */
#include "dsmapifp.h"      /* ADSM API function prototypes.            */
#include "dsmrc.h"         /* ADSM API rc definitions.                 */
#include "dapitype.h"      /* ADSM Sample API type definitions.        */
#include "dapiutil.h"

#ifdef _NO_PROTO_
typedef int16 (*func_proto)();
#else
typedef int16 (*func_proto)(dialog *, uint16 );
#endif

#ifdef _NO_PROTO_
extern int16 perform_send     ();
extern int16 perform_signon   ();
extern int16 perform_query_fs ();
extern int16 perform_query_mc ();
extern int16 perform_query_get();
#else
/*----------------------------------------------------------------------+
| External procedures referenced from this module
.----------------------------------------------------------------------*/
extern int16 perform_send     (dialog *dlg,uint16 parm1);
extern int16 perform_signon   (dialog *dlg,uint16 parm1);
extern int16 perform_query_fs (dialog *dlg,uint16 parm1);
extern int16 perform_query_mc (dialog *dlg,uint16 parm1);
extern int16 perform_query_get(dialog *dlg,uint16 parm1);
#endif

/*----------------------------------------------------------------------+
| Private routines
.----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static int16 get_input();
static int16 process_dialog();
static void output_sessinfo();
#else
static int16 get_input(dialog *line);
static int16 process_dialog(func_proto func, dialog *sel_dialog,uint16 parm1);
static void output_sessinfo();
#endif

/*----------------------------------------------------------------------+
| Public routines
.----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
| Name:    smpapi_exit()
|
| Action:  Performs any cleanup necessary before exiting the sample api
|          main menu processing.
|
| Input:   sel_dialog  - Pointer to dialog table if any.
|
| Returns: RC_OK            - Successful
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int16 smpapi_exit(sel_dialog)
   dialog *sel_dialog;
#else
int16 smpapi_exit(dialog *sel_dialog)
#endif

{
int16 rc;
rc = dsmTerminate(dsmHandle);
dsmHandle = 0;
memset(&dsmSessInfo,0x00,sizeof(ApiSessInfo));  /* Zero out block.     */
return 0;
}

/*----------------------------------------------------------------------+
| Name:    smpapi_backup()
|
| Action:  Prompts user for backup objects to create and then sends those
|          objects to the DSM server.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|          others           - Any other valid ADSM api rc.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int16 smpapi_backup (sel_dialog)
   dialog *sel_dialog;
#else
int16 smpapi_backup (dialog *sel_dialog)
#endif
{
uint16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the backup dialog! ***\n");
  }
else rc = process_dialog((func_proto)perform_send,sel_dialog,Backup_Send);

return rc;
}

/*----------------------------------------------------------------------+
| Name:    smpapi_archive()
|
| Action:  Prompts user for archive objects to create and then sends those
|          objects to the DSM server.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|          others           - Any other valid ADSM api rc.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int16 smpapi_archive (sel_dialog)
   dialog *sel_dialog;
#else
int16 smpapi_archive (dialog *sel_dialog)
#endif
{
uint16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the archive dialog! ***\n");
  }
else rc = process_dialog((func_proto)perform_send,sel_dialog,Archive_Send);

return rc;
}

/*----------------------------------------------------------------------+
| Name:    smpapi_restore(dialog *sel_dialog)
|
| Action:  Prompts user for items to restore and then does so.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|          others           - Any other valid ADSM api rc.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int16 smpapi_restore(sel_dialog)
   dialog *sel_dialog;
#else
int16 smpapi_restore(dialog *sel_dialog)
#endif
{
uint16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the restore dialog! ***\n");
  }
else rc = process_dialog((func_proto)perform_query_get,sel_dialog,Restore_Get);

return rc;
}

/*----------------------------------------------------------------------+
| Name:    smpapi_retrieve(dialog *sel_dialog)
|
| Action:  Prompts user for items to retrieve and then does so.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|          others           - Any other valid ADSM api rc.
|          RC_NO_MEMORY     - Memory allocation call failed.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int16 smpapi_retrieve(sel_dialog)
   dialog *sel_dialog;
#else
int16 smpapi_retrieve(dialog *sel_dialog)
#endif
{
uint16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the restore dialog! ***\n");
  }
else rc = process_dialog((func_proto)perform_query_get,sel_dialog,Retrieve_Get);

return rc;
}

/*----------------------------------------------------------------------+
| Name:    smpapi_signon()
|
| Action:  Prompts user for backup objects to create and then sends those
|          objects to the DSM server.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|          others           - Any other valid ADSM api rc.
|          RC_NO_MEMORY     - Memory allocation call failed.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int16 smpapi_signon (sel_dialog)
   dialog *sel_dialog;
#else
int16 smpapi_signon (dialog *sel_dialog)
#endif
{
uint16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the signon dialog! ***\n");
  }
else rc = process_dialog(perform_signon,sel_dialog,0);

return rc;
}

/*----------------------------------------------------------------------+
| Name:    smpapi_qry_archive(dialog *sel_dialog)
|
| Action:  Prompts user for filespace(s) to query and then calls
|          perform_query_fs to do the actual query.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|          others           - Any other valid ADSM api rc.
|          RC_NO_MEMORY     - Memory allocation call failed.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int16 smpapi_qry_archive (sel_dialog)
   dialog *sel_dialog;
#else
int16 smpapi_qry_archive (dialog *sel_dialog)
#endif
{
uint16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the query archive dialog! ***\n");
  }
else rc = process_dialog(perform_query_get,sel_dialog,Archive_Get);

return rc;
}

/*----------------------------------------------------------------------+
| Name:    smpapi_qry_backup(dialog *sel_dialog)
|
| Action:  Prompts user for filespace(s) to query and then calls
|          perform_query_fs to do the actual query.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|          others           - Any other valid ADSM api rc.
|          RC_NO_MEMORY     - Memory allocation call failed.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int16 smpapi_qry_backup (sel_dialog)
   dialog *sel_dialog;
#else
int16 smpapi_qry_backup (dialog *sel_dialog)
#endif
{
uint16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the query backup dialog! ***\n");
  }
else rc = process_dialog(perform_query_get,sel_dialog,Backup_Get);

return rc;
}

/*----------------------------------------------------------------------+
| Name:    smpapi_del_backup(dialog *sel_dialog)
|
| Action:  Prompts user for filespace(s) to query and then calls
|          perform_query_fs to do the actual query.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|          others           - Any other valid ADSM api rc.
|          RC_NO_MEMORY     - Memory allocation call failed.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int16 smpapi_del_backup (sel_dialog)
   dialog *sel_dialog;
#else
int16 smpapi_del_backup (dialog *sel_dialog)
#endif
{
uint16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the query backup dialog! ***\n");
  }
else rc = process_dialog(perform_query_get,sel_dialog,Backup_Delete);

return rc;
}

/*----------------------------------------------------------------------+
| Name:    smpapi_del_archive(dialog *sel_dialog)
|
| Action:  Prompts user for filespace(s) to query and then calls
|          perform_query_fs to do the actual query.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|          others           - Any other valid ADSM api rc.
|          RC_NO_MEMORY     - Memory allocation call failed.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int16 smpapi_del_archive (sel_dialog)
   dialog *sel_dialog;
#else
int16 smpapi_del_archive (dialog *sel_dialog)
#endif
{
uint16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the query backup dialog! ***\n");
  }
else rc = process_dialog(perform_query_get,sel_dialog,Archive_Delete);

return rc;
}

/*----------------------------------------------------------------------+
| Name:    smpapi_qry_fs(dialog *sel_dialog)
|
| Action:  Prompts user for filespace(s) to query and then calls
|          perform_query_fs to do the actual query.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|          others           - Any other valid ADSM api rc.
|          RC_NO_MEMORY     - Memory allocation call failed.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int16 smpapi_qry_fs (sel_dialog)
   dialog *sel_dialog;
#else
int16 smpapi_qry_fs (dialog *sel_dialog)
#endif
{
uint16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the query filespace dialog! ***\n");
  }
else rc = process_dialog(perform_query_fs,sel_dialog,fs_query);

return rc;
}

/*----------------------------------------------------------------------+
| Name:    smpapi_del_fs(dialog *sel_dialog)
|
| Action:  Prompts user for filespace(s) to delete and then calls
|          perform_query_fs to do the actual query/delete.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|          others           - Any other valid ADSM api rc.
|          RC_NO_MEMORY     - Memory allocation call failed.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int16 smpapi_del_fs (sel_dialog)
   dialog *sel_dialog;
#else
int16 smpapi_del_fs (dialog *sel_dialog)
#endif
{
uint16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the query filespace dialog! ***\n");
  }
else rc = process_dialog(perform_query_fs,sel_dialog,fs_delete);

return rc;
}

/*----------------------------------------------------------------------+
| Name:    smpapi_qry_mc(dialog *sel_dialog)
|
| Action:  Prompts user for mgmt classes to query and then calls
|          perform_query_mc to do the actual query.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|          others           - Any other valid ADSM api rc.
|          RC_NO_MEMORY     - Memory allocation call failed.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int16  smpapi_qry_mc (sel_dialog)
   dialog *sel_dialog;
#else
int16  smpapi_qry_mc (dialog *sel_dialog)
#endif
{
uint16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the query filespace dialog! ***\n");
  }
else rc = process_dialog(perform_query_mc,sel_dialog,0);
return rc;
}

/*----------------------------------------------------------------------+
| Name:    smpapi_qry_sess(dialog *sel_dialog)
|
| Action:  Displays session values returned from a query session.
|
| Input:   sel_dialog  - Null pointer for this call
|
| Returns: RC_OK            - Successful
|          others           - Any other valid ADSM api rc.
|          RC_NO_MEMORY     - Memory allocation call failed.
|
| Side
| Effects: Resets the global variable dsmSessInfo that is set upon
|          session signon.  The query is re-issued here for
|          completeness of the sample and testing.
|
| Notes:   None
+----------------------------------------------------------------------*/
int16 smpapi_qry_sess()
{
int16 rc;

if (!dsmHandle)
  {
  printf("You must signon before performing this action.\n");
  return(0);
  }

memset(&dsmSessInfo,0x00,sizeof(ApiSessInfo));  /* Zero out block.     */
dsmSessInfo.stVersion = ApiSessInfoVersion;     /* Init struct version */
rc = dsmQuerySessInfo(dsmHandle,                /* Our session handle  */
                      &dsmSessInfo);            /* Output structure.   */
if (rc) printf("*** Error on issueing Query Session.  Rc = %d\n",rc);
else output_sessinfo();

return 0;
}

/*----------------------------------------------------------------------+
| Name:    smpapi_qry_version(dialog *sel_dialog)
|
| Action:  Displays version the API being called is at.
|
| Input:   sel_dialog  - Null pointer for this call
|
| Returns: RC_OK            - Successful
|
| Side
| Effects: Resets the global variable apiLibVer that is set with the API
|          version upon entry to the sample application.  It is done
|          here to actually re-issue the query.
|
| Notes:   None
+----------------------------------------------------------------------*/
int16 smpapi_qry_version()
{

memset(&apiLibVer,0x00,sizeof(dsmApiVersion));
dsmQueryApiVersion(&apiLibVer);
printf("\nAPI Version = %d.%d.%d\n\n",apiLibVer.version,
                                      apiLibVer.release,
                                      apiLibVer.level);

return 0;
}

/*----------------------------------------------------------------------+
| Name:    process_dialog()
|
| Action:  Displays the passed dialog, and after all entries entered
|          calls the passed function with the passed dialog.
|          On return from the called function the dialog memory will
|          be freed for another invocation.
|
| Input:   func()      - Pointer to the function to invoke
|          sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|          RC_NO_MEMORY     - Memory allocation call failed.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static int16 process_dialog(func, sel_dialog, parm1)
   func_proto   func;
   dialog  *sel_dialog;
   uint16  parm1;

#else
static int16 process_dialog(func_proto func,
                     dialog *sel_dialog,
                     uint16 parm1)
#endif
{
uint16   i;
uint8    done;
uint8    done2;
uint16   rc = 0;
char     s[50];
dialog   *dlg_item;

i=0;
done = FALSE;
while (!done)
  {
  dlg_item = &sel_dialog[i];               /* Specific item to work on  */
  if (dlg_item->item_type != DSMAPI_END)   /* In not at end of dialog   */
    {
    if (rc = get_input(dlg_item))          /* Put msg out, check for error*/
      {
      printf("** Error on data input **  rc = %d\n",rc);
      done = TRUE;
      }
    else i++;                              /* Next item.                */
    }
  else                                     /* At end of dialog.         */
    {
    done2 = FALSE;
    while (!done2)
      {
      printf("Are the above responses correct (y/n/q)?\n");
      ourGetS(s);
      switch (*s)
        {
        case 'n' :
        case 'N' :
          free_dialog_buffs(sel_dialog);
          done2 = TRUE;             /* Done with inner loop */
          break;
        case 'q' :
        case 'Q' :
          done = TRUE;              /* Done with outer loop */
          done2 = TRUE;             /* Done with inner loop */
          break;
        case 'y' :
        case 'Y' :
        default  :
          rc = func(sel_dialog,parm1);
          done = TRUE;              /* Done with outer loop */
          done2 = TRUE;             /* Done with inner loop */
        } /* switch */
      } /* while */
    i=0;
    } /* else */
  } /* while */
free_dialog_buffs(sel_dialog);
return rc;
}

/*----------------------------------------------------------------------+
| Name: smpapi_query_exit
.----------------------------------------------------------------------*/
int16 smpapi_query_exit()
{
return 0;
}

/*----------------------------------------------------------------------+
| Name: smpapi_delete_exit
.----------------------------------------------------------------------*/
int16 smpapi_delete_exit()
{
return 0;
}

/*----------------------------------------------------------------------+
| Private routines
.----------------------------------------------------------------------*/

/*----------------------------------------------------------------------+
| Name:    get_input
|
| Action:  Prompts user for input of passed dialog line.  Input is then
|          put into malloc'd storage in the passed buffer.
|
| Returns:
|     RC_OK            - Successful
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static int16 get_input(line)
   dialog *line;
#else
static int16 get_input
  (
  dialog *line
  )
#endif
{
char    *input_buff;
uint32  len;
int16   rc = 0;

input_buff = malloc(2049);
printf("%s",line->item_desc); fflush(stdout);
ourGetS(input_buff);
len = strlen(input_buff);

if (!(line->item_buff = malloc(len + 1)))
  rc = RC_NO_MEMORY;
else
  strcpy(line->item_buff,input_buff);
free(input_buff);
return rc;

}

/*----------------------------------------------------------------------+
| Name:    free_dialog_buffs
|
| Action:  Free any buffers for dialogs and reset to NULL.
|
| Returns:
|     RC_OK            - Successful
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static int  free_dialog_buffs(sel_dialog)
   dialog *sel_dialog;
#else
static int  free_dialog_buffs
  (
  dialog *sel_dialog
  )
#endif
{
dialog *dlg;
uint16 i;
uint8 done;

i=0;
done = FALSE;
while (!done)
  {
  dlg = &sel_dialog[i];
  if (dlg->item_type != DSMAPI_END)
    {
    if (dlg->item_buff)
      {
      free(dlg->item_buff);
      dlg->item_buff = NULL;
      i++;
      }
    }
  else done = TRUE;
  }
}

/*----------------------------------------------------------------------+
| Name:    output_sessinfo()
|
| Action:  Printf the session info structure.  Refer to the dsmapi.h
|          file for format and possible values for the structure.
|
| Input:   None
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
static void output_sessinfo()
{
char t[50];

printf("\ndsmQuerySessInfo Values:\n");
printf("  Server Information:\n");
printf("    Server name: %s\n",dsmSessInfo.serverHost);
printf("    Server port: %u\n",dsmSessInfo.serverPort);
printf("    Server date: %i/%i/%i  %i:%i:%i\n",
                      dsmSessInfo.serverDate.year,
               (int16)dsmSessInfo.serverDate.month,
               (int16)dsmSessInfo.serverDate.day,
               (int16)dsmSessInfo.serverDate.hour,
               (int16)dsmSessInfo.serverDate.minute,
               (int16)dsmSessInfo.serverDate.second);
printf("    Server type: %s\n",dsmSessInfo.serverType);
printf("    Server version: %i.%i.%i.%i\n",dsmSessInfo.serverVer,
                                           dsmSessInfo.serverRel,
                                           dsmSessInfo.serverLev,
                                           dsmSessInfo.serverSubLev);
printf("  Client Information:\n");
printf("    Client node type: %s\n",dsmSessInfo.nodeType);
printf("    Client filespace delimiter: %c\n",dsmSessInfo.fsdelim);
printf("    Client hl & ll   delimiter: %c\n",dsmSessInfo.hldelim);

switch (dsmSessInfo.compression)
  {
  case COMPRESS_YES :
    strcpy(t,"ON"); break;
  case COMPRESS_NO :
    strcpy(t,"OFF"); break;
  case COMPRESS_CD :
    strcpy(t,"Client determined"); break;
  default :
    strcpy(t,"Unknown compress state.");
  }
printf("    Client compression: %s (%iu)\n",t,(uint16)dsmSessInfo.compression);

switch (dsmSessInfo.archDel)
  {
  case ARCHDEL_YES :
    strcpy(t,"Client can delete archived objects"); break;
  case ARCHDEL_NO :
    strcpy(t,"Client CANNOT delete archived objects"); break;
  default :
    strcpy(t,"Unknown archive delete state");
  }
printf("    Client archive delete: %s\n",t);


switch (dsmSessInfo.backDel)
  {
  case BACKDEL_YES :
    strcpy(t,"Client can delete backup objects"); break;
  case ARCHDEL_NO :
    strcpy(t,"Client CANNOT delete backup objects"); break;
  default :
    strcpy(t,"Unknown backup delete state");
  }
printf("    Client backup delete: %s\n",t);
printf("    Maximum objects in multiple object transactions: %u\n",
                dsmSessInfo.maxObjPerTxn);
printf("  General session info:\n");
printf("    Node:  %s\n",dsmSessInfo.id);
printf("    Owner: %s\n",dsmSessInfo.owner);
printf("    API Config file: %s\n",dsmSessInfo.confFile);
printf("  Policy Information:\n");
printf("    Domain name: %s\n",dsmSessInfo.domainName);
printf("    Policyset name: %s\n",dsmSessInfo.policySetName);
printf("    Policy activation date: %i/%i/%i  %i:%i:%i\n",
                      dsmSessInfo.polActDate.year,
               (int16)dsmSessInfo.polActDate.month,
               (int16)dsmSessInfo.polActDate.day,
               (int16)dsmSessInfo.polActDate.hour,
               (int16)dsmSessInfo.polActDate.minute,
               (int16)dsmSessInfo.polActDate.second);
printf("    Default management class: %s\n",dsmSessInfo.dfltMCName);
printf("    Backup retention grace period: %u days\n",
                 dsmSessInfo.gpBackRetn);
printf("    Archive retention grace period: %u days\n",
                 dsmSessInfo.gpArchRetn);
}
