/***********************************************************************
* Name:
*        dapipw.c
*
* Function:
*        Update and change password function for the ADSM Sample API
*
* Environment:
*        This is a PLATFORM-INDEPENDENT source file. As such it may
*        contain no dependencies on any specific operating system
*        environment or hardware platform.
*
***********************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "dsmapitd.h"        /* Standard DSM API header.      */
#include "dsmapifp.h"
#include "dapitype.h"        /* Sample API data types.        */
#include "dsmrc.h"           /* DSM API return codes.         */
/*----------------------------------------------------------------------+
| Public routines
.----------------------------------------------------------------------*/

/*----------------------------------------------------------------------+
| Name:    smpaip_chgpw()
|
| Action:  This function is invoked from the sample program main menu.
|          Prompt the end user for current password and new passwords,
|          and if those match send the new password to the server.
|
| Input:   sel_dialog  - Pointer to dialog table to process or NULL if
|                        no dialog.
|
| Returns: RC_OK             - Successful
|          RC_NOT_IN_SESSION - Client is not in session with the server
|          RC_SESSION_FAILED - A dsm api call failed stopping execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int16 smpapi_chgpw(sel_dialog)
   dialog *sel_dialog;
#else
int16 smpapi_chgpw(dialog *sel_dialog)
#endif
{

   bool_t   done = bFalse;
   bool_t   pw_match = bFalse;
   uint16   pw_trys;
   uint16   rc;
   char current_pw[DSM_MAX_VERIFIER_LENGTH + 1];
   char new_pw1   [DSM_MAX_VERIFIER_LENGTH + 1];
   char new_pw2   [DSM_MAX_VERIFIER_LENGTH + 1];

   pw_trys = 0;
   while (!done)
   {
      printf("\nChanging password for node %s:\n\n",dsmSessInfo.id);
      printf("Enter your current password:"); fflush(stdout);
      ourGetS(current_pw);

      printf("Enter your new password:"); fflush(stdout);
      ourGetS(new_pw1);
      printf("Enter your new password again:"); fflush(stdout);
      ourGetS(new_pw2);

      if ((strcmp(new_pw1,new_pw2)))
      {
         pw_trys++;
         if (pw_trys > 3)
         {
            printf("Password entry failed.  Exiting retry.\n");
            done = bTrue;
         }
         else
         {
            printf("Your new passwords do not match.  Try again.");
         }
      }
      else
      {
         done = bTrue;
         pw_match = bTrue;
      }
   }

   if (!pw_match) return 0;   /* Get out now since pw's did not match */

   rc = dsmChangePW(dsmHandle,current_pw,new_pw1);
   if (rc)
   {
      printf("*** Password change failed.  Rc = %i\n",rc);
   }
   else
   {
      printf("*** Your new password has been accepted and updated.\n");
   }

return 0;
}
