/***********************************************************************
* Name:
*        dapitype.h
*
* Function:
*        Type definitions for the sample ADSM api code
*
* Environment:
*        This is a PLATFORM-INDEPENDENT source file. As such it may
*        contain no dependencies on any specific operating system
*        environment or hardware platform.
*
***********************************************************************/
#define FALSE 0
#define TRUE 1
#define RC_NO_MEMORY      222
#define RC_NOT_IN_SESSION 12
#define RC_SESSION_FAILED 13

/*------------------------------------------------------------------------.
| Dialog definition for the various input fields.
.------------------------------------------------------------------------*/
typedef struct dialog_s
  {
  uint16 item_type;
  char  *item_desc;
  char  *item_buff;
  } dialog;

/*------------------------------------------------------------------------.
| Table definitions for menus and menu entries used in this application.
| These structures will be defined in the data portion of the application.
|
| These structures provide a very simple setup to offer line mode menu
| selection of items.  This way a single code segment can process all
| user input and invoke the proper routines without having to change the
| main line code.
.------------------------------------------------------------------------*/
typedef struct menu_s
  {
  struct menu_entry_s *mentry;  /* Pointer to the menu entries.          */
  char   *instructions;         /* Short instruction for menu.           */
  } menu;

typedef struct menu_entry_s
  {
  uint16 selection;          /* Selection number (0 - n)                 */
  char   *sel_desc;          /* Selection description (Backup...         */
  int16 (*sel_func)();       /* Function to call if selection chosen     */
  menu   *next_menu;         /* Pointer to next menu if nested.          */
  uint8  exit_menu;          /* Boolean indicating if to exit from menu  */
  dialog *sel_dialog;        /* Dialog to use.                           */
  } menu_entry;

#define MENU_END 65324        /* Arbitrary number to designate end of menu*/

/*------------------------------------------------------------------------.
| Linked list typedefs typically used for keeping multiple objects
| together within a transaction.
.------------------------------------------------------------------------*/
typedef struct s_llist
  {
  struct s_llist *next_llist;
  dsmObjName      objName;
  } llist;

/*------------------------------------------------------------------------.
| Item type definitions for dialog processing
.------------------------------------------------------------------------*/
#define DSMAPI_FILESPACE            1
#define DSMAPI_HIGHLEVEL            2
#define DSMAPI_LOWLEVEL             3
#define DSMAPI_FILESIZE             4
#define DSMAPI_NUM_FILES            5
#define DSMAPI_SEED_STRING          6
#define DSMAPI_MNTWAIT              7
#define DSMAPI_OBJOWNER             8
#define DSMAPI_OBJCOMP              9
#define DSMAPI_NODENAME            10
#define DSMAPI_OWNER               11
#define DSMAPI_PASSWORD            12
#define DSMAPI_OBJTYPE             13
#define DSMAPI_MC                  14
#define DSMAPI_DESCRIPTION         15
#define DSMAPI_ACTIVETYPE          16
#define DSMAPI_LOINSDATE           17
#define DSMAPI_HIINSDATE           18
#define DSMAPI_LOEXPDATE           19
#define DSMAPI_HOEXPDATE           20
#define DSMAPI_FASTACTIVE          21
#define DSMAPI_DETAILS             22
#define DSMAPI_CONFIG              23
#define DSMAPI_OPTIONS             24
#define DSMAPI_END              30999

/*------------------------------------------------------------------------.
| Optional parm defs for process dialog calls
.------------------------------------------------------------------------*/
#define Archive_Send      1
#define Backup_Send       2
#define Archive_Get       3
#define Backup_Get        4
#define ABackup_Get       5
#define Restore_Get       6
#define Retrieve_Get      7
#define Backup_Delete     8
#define Archive_Delete    9
#define fs_query          10
#define fs_delete         11

/*------------------------------------------------------------------------.
| Data structure for our object info field on object sends.
.------------------------------------------------------------------------*/
typedef struct
{
   uint8   objType;            /* Type of object sent.                   */
   uint64  size;               /* Size of object sent.                   */
   char    eyecatcher[30];     /* Eye catcher string            */
   char    seed[1];            /* Beginning of seed string, null delimit.*/
} smpObjInfo;

/*------------------------------------------------------------------------.
| Data structure for sample preferences
.------------------------------------------------------------------------*/
typedef struct
{
   uint32  sz_send_buff;       /* Size of buffer to use for data sends.  */
   uint32  sz_recv_buff;       /* Size of buffer to use for data recvs.  */
   bool_t  verbose;            /* Should output be verbose or normal?    */
   uint32  max_restore_items;  /* Max num of items to restore in 1 call  */
   bool_t  qry_details;        /* Show details on queries?               */
   bool_t  use_est;            /* Give small size estimate on sends?     */
   bool_t  echo_in;            /* Echo input lines as they are read?     */
} preferences;

/* ******* G l o b a l s  ********  */

extern dsmApiVersion    apiApplVer;
extern dsmApiVersion    apiLibVer;
extern uint32           dsmHandle;
extern ApiSessInfo      dsmSessInfo;
extern preferences      pref;
extern char             api_eyecatcher[];
