/***********************************************************************
* Name:
*        dapiutil.c
*
* Function:
*        Utility functions for the sample ADSM api code
*
* Public Routines:
*    format_number : Format numbers with commas so they can be read
*    initFS:         Fill in platform specific filespace info      
*
* Description:
*        The file contains utility functions for the ADSM sample api
*        code that do not fit well within other source modules.
***********************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "dsmapitd.h"          /* Standard DSM API header.      */
#include "dsmapifp.h"
#include "dapitype.h"        /* Sample API data types.        */
#include "dapiutil.h"        /* Handy utility routines.       */

/*----------------------------------------------------------------------+
| Name:    format_number()
|
| Action:  Take a asciiz string and format it with commas for readability.
|          Converts 123456789 to 123,456,789
|
| Input:   *s_in - Pointer to input character string.
|
| Output:  *s_out - Pointer to output character buffer.
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   This routine handles formatted numbers up to 49 characters in
|          length, thus buffers of 50 bytes need be passed for the routine
|          to work.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void format_number(s_in, s_out)
   char *s_in;
   char *s_out;
#else
void format_number(char *s_in, char *s_out)
#endif

{
char  *save;
char temp[50];           /* Should be long enough for most numbers.      */
int i;
char separator = ',';    /* This could be a period in other countries.   */

if (!*s_in)
  {*s_out = 0x00; return;}

save = s_in;           /* Remember where it started.                */
s_in += strlen(s_in);  /* Point to end of string.                   */

i = 48;
temp[49] = 0;
while (s_in != save)
  {
  temp[i--] = (*s_in--);
  if (i%4==0) temp[i--] = separator;
  }
temp[i] = (*s_in);
strcpy(s_out,temp+i);
}
/*----------------------------------------------------------------------+
| Name:    initFS()        
|
| Action:  Fill in platform specific filespace info                        
|          
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void initFS(*fsDataP)
   regFSData *fsDataP;
#else
void initFS(regFSData *fsDataP)
#endif
{
strcpy(fsDataP->fsAttr.unixFSAttr.fsInfo,"Sample API FS Info");
fsDataP->fsAttr.unixFSAttr.fsInfoLength =
       strlen(fsDataP->fsAttr.unixFSAttr.fsInfo) + 1;

}
/*----------------------------------------------------------------------+
| Name:    ourGetS()        
|
| Action:  Perform a gets() and display the input if preference
|          echo_in is set to bTrue.
|          
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void ourGetS(*buf)
   char *buf;
#else
void ourGetS(char *buf)
#endif
{
gets(buf);
if (pref.echo_in == bTrue)
  printf("%s\n", buf);
}

/*----------------------------------------------------------------------+
| Name:    FSInfoResp()
|
| Action:  Fill in platform specific filespace info
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void FSInfoResp(*fs_info, *fsResp)
   char *fs_info;
   qryRespFSData *fsResp;
#else
void FSInfoResp(char *fs_info, qryRespFSData *fsResp)
#endif
{
      memcpy(fs_info, fsResp->fsAttr.unixFSAttr.fsInfo,
                             fsResp->fsAttr.unixFSAttr.fsInfoLength);
      memcpy(fs_info+fsResp->fsAttr.unixFSAttr.fsInfoLength,
                           "\0", 1); /* NULL term */
}
