/***********************************************************************
* ADSTAR Distributed Storage Manager (ADSTAR DSM)                      *
* API Client Component                                                 *
*                                                                      *
* (C) Copyright IBM Corporation 1993,1995,1996                         *
***********************************************************************/

/**************************************************************************/
/* Header File Name: dsmapitd.h                                           */
/*                                                                        */
/* Descriptive-name: Definitions for ADSTAR DSM external constants        */
/**************************************************************************/
#ifndef _H_DSMAPITD
#define _H_DSMAPITD
/*========================================================================*/
/*                     D E F I N E S                                      */
/*========================================================================*/
/*-------------------------------------------------------------------------+
|  API Version, Release, and Level to use in dsmApiVersion on dsmInit()    |
+-------------------------------------------------------------------------*/
#define DSM_API_VERSION     2
#define DSM_API_RELEASE     1
#define DSM_API_LEVEL       6

/*-------------------------------------------------------------------------+
| Maximum field lengths                                                    |
+-------------------------------------------------------------------------*/
#define DSM_MAX_CG_DEST_LENGTH        30        /* copy group destination */
#define DSM_MAX_CG_NAME_LENGTH        30        /* copy group name        */
#define DSM_MAX_DESCR_LENGTH          255       /* archive description    */
#define DSM_MAX_DOMAIN_LENGTH         30        /* policy domain name     */
#define DSM_MAX_FSINFO_LENGTH         500       /* filespace info         */
#define DSM_MAX_FSNAME_LENGTH         1024      /* filespace name         */
#define DSM_MAX_FSTYPE_LENGTH         32        /* filespace type         */
#define DSM_MAX_HL_LENGTH             1024      /* object high level name */
#define DSM_MAX_ID_LENGTH             64        /* session node name      */
#define DSM_MAX_LL_LENGTH             256       /* object low level name  */
#define DSM_MAX_MC_NAME_LENGTH        30        /* management class name  */
#define DSM_MAX_OBJINFO_LENGTH        255       /* object info            */
#define DSM_MAX_OWNER_LENGTH          64        /* object owner name      */
#define DSM_MAX_PLATFORM_LENGTH       16        /* application type       */
#define DSM_MAX_PS_NAME_LENGTH        30        /* policy set name        */
#define DSM_MAX_SERVERTYPE_LENGTH     32        /* server platform type   */
#define DSM_MAX_VERIFIER_LENGTH       64        /* password               */
#define DSM_PATH_MAX                  1024      /* API config file path   */
#define DSM_NAME_MAX                  255       /* API config file name   */
#define DSM_MAX_NODE_LENGTH           64        /* node/machine name      */
#define DSM_MAX_RC_MSG_LENGTH         1024      /* msg parm for dsmRCMsg  */

#define DSM_MAX_MC_DESCR_LENGTH       DSM_MAX_DESCR_LENGTH /* mgmt class  */
#define DSM_MAX_SERVERNAME_LENGTH     DSM_MAX_ID_LENGTH /* server name    */

#define DSM_MAX_GET_OBJ               4080     /* max objs on BeginGetData*/

/*-------------------------------------------------------------------------+
| Minimum field lengths                                                    |
+-------------------------------------------------------------------------*/
#define DSM_MIN_COMPRESS_SIZE  2048 /* minimum number of bytes an object  */
                                    /* needs before compression is allowed*/

/*-------------------------------------------------------------------------+
|  Values for object type in dsmObjName structure                          |
+-------------------------------------------------------------------------*/
#define DSM_OBJ_FILE        0x01        /* object has attrib info & data  */
#define DSM_OBJ_DIRECTORY   0x02        /* object has only attribute info */
#define DSM_OBJ_ANY_TYPE    0xFF        /* for use on query               */

/*-------------------------------------------------------------------------+
| Values for copySer in DetailCG structures for Query Mgmt Class response  |
+-------------------------------------------------------------------------*/
#define Copy_Serial_Static          1   /*Copy Serialization Static       */
#define Copy_Serial_Shared_Static   2   /*Copy Serialization Shared Static*/
#define Copy_Serial_Shared_Dynamic  3   /*Copy Serialization Shared Dynamic*/
#define Copy_Serial_Dynamic         4   /*Copy Serialization Dynamic      */

/*-------------------------------------------------------------------------+
| Values for copyMode in DetailCG structures for Query Mgmt Class response |
+-------------------------------------------------------------------------*/
#define Copy_Mode_Modified          1   /*Copy Mode Modified              */
#define Copy_Mode_Absolute          2   /*Copy Mode Absolute              */

/*-------------------------------------------------------------------------+
|  Values for objState in qryBackupData structure                          |
+-------------------------------------------------------------------------*/
#define DSM_ACTIVE            0x01      /* query only active objects      */
#define DSM_INACTIVE          0x02      /* query only inactive objects    */
#define DSM_ANY_MATCH         0xFF      /* query all backup objects       */

/*-------------------------------------------------------------------------+
| Boundary values for dsmDate.year field in qryArchiveData structure       |
+-------------------------------------------------------------------------*/
#define DATE_MINUS_INFINITE         0x0000      /* lowest boundary        */
#define DATE_PLUS_INFINITE          0xFFFF      /* highest upper boundary */

/*-------------------------------------------------------------------------+
| Bits masks for update action parameter on dsmUpdateFS()                  |
+-------------------------------------------------------------------------*/
#define DSM_FSUPD_FSTYPE          ((unsigned) 0x00000002)
#define DSM_FSUPD_FSINFO          ((unsigned) 0x00000004)
#define DSM_FSUPD_OCCUPANCY       ((unsigned) 0x00000020)
#define DSM_FSUPD_CAPACITY        ((unsigned) 0x00000040)

/*-------------------------------------------------------------------------+
|  Values for repository parameter on dsmDeleteFS()                        |
+-------------------------------------------------------------------------*/
#define DSM_ARCHIVE_REP       0x0A      /* archive repository             */
#define DSM_BACKUP_REP        0x0B      /* backup repository              */
#define DSM_REPOS_ALL         0x01      /* all respository types          */

/*-------------------------------------------------------------------------+
|  Values for vote parameter on dsmEndTxn()                                |
+-------------------------------------------------------------------------*/
#define DSM_VOTE_COMMIT  1             /* commit current transaction      */
#define DSM_VOTE_ABORT   2             /* roll back current transaction   */

/*-------------------------------------------------------------------------+
|  Values for various flags returned in ApiSessInfo structure.             |
+-------------------------------------------------------------------------*/
/* Client compression field codes */
#define COMPRESS_YES    1    /* client must compress data             */
#define COMPRESS_NO     2    /* client must NOT compress data         */
#define COMPRESS_CD     3    /* client determined                     */

/* Archive delete permission codes. */
#define ARCHDEL_YES     1    /* archive delete allowed                */
#define ARCHDEL_NO      2    /* archive delete NOT allowed            */

/* Backup delete permission codes. */
#define BACKDEL_YES     1    /* backup delete allowed                 */
#define BACKDEL_NO      2    /* backup delete NOT allowed             */

/*---------------------------------------------------------------------+
| Definitions for "media access class" field.                          |
+---------------------------------------------------------------------*/
/*
 *  The following constants define a hierarchy of media access classes.
 *  Lower numbers indicate media which can supply faster access to data.
 */

/* Fixed: represents the class of local, on-line, fixed media (such as
          hard disks). Represents minimal delays in retrieval. */
#define  MEDIA_FIXED          0x10

/* Library: represents the class of local, mountable media accessible
            through a mechanical mounting device, in which there are
            typically small variations in mount time. */
#define  MEDIA_LIBRARY        0x20

/* Network: represents storage media accessible via a network server. */
#define  MEDIA_NETWORK        0x30

/* Shelf: represents the class of local, mountable media accessible only
          via human intervention. There can be large variations in the
          mount time. */
#define  MEDIA_SHELF          0x40

/* Offsite: represents media stored in an off-site location. The
            media are not accessible via local mounting procedures
            or direct attachment through a network. */
#define  MEDIA_OFFSITE        0x50

/* Unavailable: represents media that are (for whatever reason)
                inaccessible for retrieval. */
#define  MEDIA_UNAVAILABLE    0xF0

/*========================================================================*/
/*                     T Y P E D E F S                                    */
/*========================================================================*/

/* "Undefine" the keyword "signed" for the SunOS non-ANSI compiler. */
#ifdef _SUN_NONANSI_
#define signed
#endif

/*-------------------------------------------------------------------------+
|  Typedefs for different integer types                                    |
+-------------------------------------------------------------------------*/
/* The alpha processor of Digital Equipment Corporation is the only */
/* platform currently supported that has a 64 bit architecture.     */
/* Therefore the '(u)int32' data types are 'int' and not 'long'.    */
#ifndef __alpha

typedef signed   char     int8   ;
typedef unsigned char     uint8  ;
typedef signed   short    int16  ;
typedef unsigned short    uint16 ;
typedef signed   long     int32  ;
typedef unsigned long     uint32 ;

typedef struct
{
   uint32   hi;                       /* Most significant 32 bits.        */
   uint32   lo;                       /* Least significant 32 bits.       */
} uint64 ;

#else

typedef signed   char     int8   ;
typedef unsigned char     uint8  ;
typedef signed   short    int16  ;
typedef unsigned short    uint16 ;
typedef signed   int      int32  ;
typedef unsigned int      uint32 ;

typedef struct
{
   uint32   hi;                       /* Most significant 32 bits.        */
   uint32   lo;                       /* Least significant 32 bits.       */
} uint64 ;

#endif

/*-------------------------------------------------------------------------+
| Type definition for bool_t                                               |
+-------------------------------------------------------------------------*/
/*
 *  Had to create a Boolean type that didn't clash with any other predefined
 *  version in any operating system or windowing system.
 */
typedef enum
{
        bFalse = 0x00,
        bTrue  = 0x01
} bool_t ;

/*-------------------------------------------------------------------------+
|  Type definition for date structure                                      |
+-------------------------------------------------------------------------*/
typedef struct
{
    uint16    year;                  /* year, 16-bit integer (e.g., 1990) */
    uint8     month;                 /* month, 8-bit integer (1 - 12)     */
    uint8     day;                   /* day. 8-bit integer (1 - 31)       */
    uint8     hour;                  /* hour, 8-bit integer (0 - 23)      */
    uint8     minute;                /* minute, 8-bit integer (0 - 59)    */
    uint8     second;                /* second, b-bit integer (0 - 59)    */
} dsmDate ;

/*-------------------------------------------------------------------------+
|  Type definition for Object ID on dsmGetObj() and in dsmGetList structure|
+-------------------------------------------------------------------------*/
typedef uint64  ObjID ;

/*-------------------------------------------------------------------------+
| Type definition for dsmQueryBuff on dsmBeginQuery()                      |
+-------------------------------------------------------------------------*/
typedef void dsmQueryBuff ;

/*-------------------------------------------------------------------------+
| Type definition for dsmGetType parameter on dsmBeginGetData()            |
+-------------------------------------------------------------------------*/
typedef enum
{
        gtBackup = 0x00,                     /* Backup processing type    */
        gtArchive                            /* Archive processing type   */
} dsmGetType ;

/*-------------------------------------------------------------------------+
|  Type definition for dsmQueryType parameter on dsmBeginQuery()           |
+-------------------------------------------------------------------------*/
typedef enum
{
    qtArchive = 0x00,                               /* Archive query type */
    qtBackup,                                        /* Backup query type */
    qtBackupActive,                 /* Fast query for active backup files */
    qtFilespace,                                  /* Filespace query type */
    qtMC                                        /* Mgmt. class query type */
} dsmQueryType ;

/*-------------------------------------------------------------------------+
|  Type definition sendType parameter on dsmBindMC() and dsmSendObj()      |
+-------------------------------------------------------------------------*/
typedef enum
{
    stBackup = 0x00,                         /* Backup processing type    */
    stArchive,                               /* Archive processing type   */
    stBackupMountWait,          /* Backup processing with mountwait on    */
    stArchiveMountWait          /* Archive processing with mountwait on   */
} dsmSendType ;

/*-------------------------------------------------------------------------+
|  Type definition for delType parameter on dsmDeleteObj()                 |
+-------------------------------------------------------------------------*/
typedef enum
{
    dtArchive = 0x00,                              /* Archive delete type */
    dtBackup                           /* Backup delete (deactivate) type */
} dsmDelType ;

/*-------------------------------------------------------------------------+
|  Type definition for API Version on dsmInit() and dsmQueryApiVersion()   |
+-------------------------------------------------------------------------*/
typedef struct
{
        unsigned short version;       /* API version                      */
        unsigned short release;       /* API release                      */
        unsigned short level;         /* API level                        */
} dsmApiVersion;

/*-------------------------------------------------------------------------+
|  Type definition for object name used on BindMC, Send, Delete, Query     |
+-------------------------------------------------------------------------*/
typedef struct S_dsmObjName
{
    char   fs[DSM_MAX_FSNAME_LENGTH + 1] ;              /* Filespace name */
    char   hl[DSM_MAX_HL_LENGTH + 1] ;                 /* High level name */
    char   ll[DSM_MAX_LL_LENGTH + 1] ;                  /* Low level name */
    uint8  objType;          /* for object type values, see defines above */
} dsmObjName;

/*-------------------------------------------------------------------------+
|  Type definition for Backup delete info on dsmDeleteObj()                |
+-------------------------------------------------------------------------*/
typedef struct
{
    uint16       stVersion ;                    /* structure version      */
    dsmObjName   *objNameP ;                    /* object name            */
    uint32       copyGroup ;                    /* copy group             */
} delBack ;

#define  delBackVersion    1

/*-------------------------------------------------------------------------+
|  Type definition for Archive delete info on dsmDeleteObj()               |
+-------------------------------------------------------------------------*/
typedef struct
{
    uint16       stVersion ;                    /* structure version      */
    uint64       objId ;                        /* object ID              */
} delArch ;

#define  delArchVersion  1

/*-------------------------------------------------------------------------+
|  Type definition for delete info on dsmDeleteObj()                       |
+-------------------------------------------------------------------------*/
typedef union
{
    delBack   backInfo ;
    delArch   archInfo ;
} dsmDelInfo ;

/*-------------------------------------------------------------------------+
|  Type definition for Object Attribute parameter on dsmSendObj()          |
+-------------------------------------------------------------------------*/
typedef struct
{
    uint16   stVersion;                              /* Structure version */
    char     owner[DSM_MAX_OWNER_LENGTH + 1];             /* object owner */
    uint64   sizeEstimate;        /* Size estimate in bytes of the object */
    bool_t   objCompressed;              /* Is object already compressed? */
    uint16   objInfoLength;            /* length of object-dependent info */
    char     *objInfo;                           /* object-dependent info */
    char     *mcNameP;                    /* mgmt class name for override */
}ObjAttr;

#define ObjAttrVersion  2

/*-------------------------------------------------------------------------+
| Type definition for mcBindKey returned on dsmBindMC()                    |
+-------------------------------------------------------------------------*/
typedef struct
{
    uint16     stVersion;                   /* structure version          */
    char       mcName[DSM_MAX_MC_NAME_LENGTH + 1];
                                           /* Name of mc bound to object. */
    bool_t     backup_cg_exists;                            /* True/false */
    bool_t     archive_cg_exists;                           /* True/false */
    char       backup_copy_dest[DSM_MAX_CG_DEST_LENGTH + 1];
                                                /* Backup copy dest. name */
    char       archive_copy_dest[DSM_MAX_CG_DEST_LENGTH + 1];
                                                   /* Arch copy dest.name */
} mcBindKey;

#define mcBindKeyVersion  1

/*-------------------------------------------------------------------------+
| Type definition for partial object data for dsmBeginGetData()            |
+-------------------------------------------------------------------------*/
typedef struct
{
    uint16   stVersion;          /* Structure version                     */
    uint64   partialObjOffset;   /* offset into object to begin reading   */
    uint64   partialObjLength;   /* amount of object to read              */
} PartialObjData ;               /* partial object data                   */

#define PartialObjDataVersion 1  /*                                       */

/*-------------------------------------------------------------------------+
| Type definition for object list on dsmBeginGetData()                     |
+-------------------------------------------------------------------------*/
typedef struct
{
    uint16   stVersion ;         /* structure version                     */
    uint32   numObjId ;          /* number of object IDs in the list      */
    ObjID    *objId    ;         /* list of object IDs to restore         */
    PartialObjData *partialObjData ; /* list of partial object data       */
} dsmGetList ;

#define dsmGetListVersion     2  /* default if not using Partial Obj data */
#define dsmGetListPORVersion  3  /* version if using Partial Obj data     */

/*-------------------------------------------------------------------------+
|  Type definition for DataBlk used to Get or Send data                    |
+-------------------------------------------------------------------------*/
typedef struct
{
    uint16 stVersion ;                    /* structure version            */
    uint32 bufferLen;                    /* Length of buffer passed below */
    uint32 numBytes;                  /* Actual number of bytes read from */
                                              /* or written to the buffer */
    char   *bufferPtr;                                     /* Data buffer */
} DataBlk;

#define DataBlkVersion  1

/*-------------------------------------------------------------------------+
|  Type definition for Mgmt Class queryBuffer on dsmBeginQuery()           |
+-------------------------------------------------------------------------*/
typedef struct S_qryMCData
{
    uint16   stVersion;                              /* structure version */
    char     *mcName;                                  /* Mgmt class name */
                                                /* single name to get one */
                                            /* or empty string to get all */
    bool_t   mcDetail;                            /* Want details or not? */
} qryMCData;

#define qryMCDataVersion  1

/*-------------------------------------------------------------------------+
|  Type definition for Archive Copy Group details on Query MC response     |
+-------------------------------------------------------------------------*/
typedef struct S_archDetailCG
{
     char     cgName[DSM_MAX_CG_NAME_LENGTH + 1];       /* Copy group name */
     uint16   frequency;                       /* Copy (archive) frequency */
     uint16   retainVers;                                /* Retain version */
     uint8    copySer;       /* for copy serialization values, see defines */
     uint8    copyMode;         /* for copy mode values, see defines above */
     char     destName[DSM_MAX_CG_DEST_LENGTH + 1];      /* Copy dest name */
} archDetailCG;

/*-------------------------------------------------------------------------+
|  Type definition for Backup Copy Group details on Query MC response      |
+-------------------------------------------------------------------------*/
typedef struct S_backupDetailCG
{
    char     cgName[DSM_MAX_CG_NAME_LENGTH + 1];       /* Copy group name */
    uint16   frequency;                               /* Backup frequency */
    uint16   verDataExst;                         /* Versions data exists */
    uint16   verDataDltd;                        /* Versions data deleted */
    uint16   retXtraVers;                        /* Retain extra versions */
    uint16   retOnlyVers;                         /* Retain only versions */
    uint8    copySer;       /* for copy serialization values, see defines */
    uint8    copyMode;         /* for copy mode values, see defines above */
    char     destName[DSM_MAX_CG_DEST_LENGTH + 1];      /* Copy dest name */
} backupDetailCG;

/*-------------------------------------------------------------------------+
|  Type definition for Query Mgmt Class detail response on dsmGetNextQObj()|
+-------------------------------------------------------------------------*/
typedef struct S_qryRespMCDetailData
{
    uint16          stVersion;                       /* structure version */
    char            mcName[DSM_MAX_MC_NAME_LENGTH + 1];        /* mc name */
    char            mcDesc[DSM_MAX_MC_DESCR_LENGTH + 1]; /*mc description */
    archDetailCG    archDet;                 /* Archive copy group detail */
    backupDetailCG  backupDet;                /* Backup copy group detail */
} qryRespMCDetailData;

#define qryRespMCDetailDataVersion  1

/*-------------------------------------------------------------------------+
| Type definition for Query Mgmt Class summary response on dsmGetNextQObj()|
+-------------------------------------------------------------------------*/
typedef struct S_qryRespMCData
{
    uint16   stVersion;                              /* structure version */
    char     mcName[DSM_MAX_MC_NAME_LENGTH + 1];               /* mc name */
    char     mcDesc[DSM_MAX_MC_DESCR_LENGTH + 1];       /* mc description */
}qryRespMCData;

#define qryRespMCDataVersion  1

/*-------------------------------------------------------------------------+
|  Type definition for Archive queryBuffer on dsmBeginQuery()              |
+-------------------------------------------------------------------------*/
typedef struct S_qryArchiveData
{
    uint16       stVersion;                          /* structure version */
    dsmObjName   *objName;                     /* Full dsm name of object */
    char         *owner;                                    /* owner name */
                        /* for maximum date boundaries, see defines above */
    dsmDate      insDateLowerBound;      /* low bound archive insert date */
    dsmDate      insDateUpperBound;       /* hi bound archive insert date */
    dsmDate      expDateLowerBound;          /* low bound expiration date */
    dsmDate      expDateUpperBound;           /* hi bound expiration date */
    char         *descr;                           /* archive description */
} qryArchiveData;

#define qryArchiveDataVersion  1

/*-------------------------------------------------------------------------+
| Type definition for Query Archive response on dsmGetNextQObj()           |
+-------------------------------------------------------------------------*/
typedef struct S_qryRespArchiveData
{
    uint16      stVersion;                           /* structure version */
    dsmObjName  objName;                      /* Filespace name qualifier */
    uint32      copyGroup;                           /* copy group number */
    char        mcName[DSM_MAX_MC_NAME_LENGTH + 1];            /* mc name */
    char        owner[DSM_MAX_OWNER_LENGTH + 1];            /* owner name */
    uint64      objId;                                  /* Unique copy id */
    uint64      restoreOrder;                            /* restore order */
    uint8       mediaClass;                         /* media access class */
    dsmDate     insDate;                        /* archive insertion date */
    dsmDate     expDate;                    /* expiration date for object */
    char        descr[DSM_MAX_DESCR_LENGTH + 1];   /* archive description */
    uint16      objInfolen;             /* length of object-dependent info*/
    char        objInfo[DSM_MAX_OBJINFO_LENGTH];  /*object-dependent info */
} qryRespArchiveData;

#define qryRespArchiveDataVersion  1

/*-------------------------------------------------------------------------+
|  Type definition for Archive sendBuff parameter on dsmSendObj()          |
+-------------------------------------------------------------------------*/
typedef struct S_sndArchiveData
{
    uint16   stVersion;                              /* structure version */
    char     *descr;                               /* archive description */
} sndArchiveData;

#define sndArchiveDataVersion  1

/*-------------------------------------------------------------------------+
|  Type definition for Backup queryBuffer on dsmBeginQuery()               |
+-------------------------------------------------------------------------*/
typedef struct S_qryBackupData
{
    uint16      stVersion;                           /* structure version */
    dsmObjName  *objName;                      /* full dsm name of object */
    char        *owner;                                     /* owner name */
    uint8       objState;                        /* object state selector */
                                /* for possible values, see defines above */
} qryBackupData;

#define qryBackupDataVersion  1

/*-------------------------------------------------------------------------+
| Type definition for Query Backup response on dsmGetNextQObj()            |
+-------------------------------------------------------------------------*/
typedef struct S_qryRespBackupData
{
    uint16      stVersion;                           /* structure version */
    dsmObjName  objName;                       /* full dsm name of object */
    uint32      copyGroup;                           /* copy group number */
    char        mcName[DSM_MAX_MC_NAME_LENGTH + 1];            /* mc name */
    char        owner[DSM_MAX_OWNER_LENGTH + 1];            /* owner name */
    uint64      objId;                                /* Unique object id */
    uint64      restoreOrder;                            /* restore order */
    uint8       mediaClass;                         /* media access class */
    uint8       objState;                      /* Obj state, active, etc. */
    dsmDate     insDate;                         /* backup insertion date */
    dsmDate     expDate;                    /* expiration date for object */
    uint16      objInfolen;             /* length of object-dependent info*/
    char        objInfo[DSM_MAX_OBJINFO_LENGTH];  /*object-dependent info */
} qryRespBackupData;

#define qryRespBackupDataVersion  1

/*-------------------------------------------------------------------------+
|  Type definition for Active Backup queryBuffer on dsmBeginQuery()
|
|  Notes:  For the active backup query, only the fs (filespace) and objType
|          fields of objName need be set.  objType can only be set to
|          DSM_OBJ_FILE or DSM_OBJ_DIRECTORY.  DSM_OBJ_ANY_TYPE will not
|          find a match on the query.
+-------------------------------------------------------------------------*/
typedef struct S_qryABackupData
{
    uint16      stVersion;                           /* structure version */
    dsmObjName  *objName;                     /* Only fs and objtype used */
} qryABackupData;

#define qryABackupDataVersion  1

/*-------------------------------------------------------------------------+
| Type definition for Query Active Backup response on dsmGetNextQObj()     |
+-------------------------------------------------------------------------*/
typedef struct S_qryARespBackupData
{
    uint16      stVersion;                           /* structure version */
    dsmObjName  objName;                       /* full dsm name of object */
    uint32      copyGroup;                           /* copy group number */
    char        mcName[DSM_MAX_MC_NAME_LENGTH + 1];/*management class name*/
    char        owner[DSM_MAX_OWNER_LENGTH + 1];            /* owner name */
    dsmDate     insDate;                         /* backup insertion date */
    uint16      objInfolen;             /* length of object-dependent info*/
    char        objInfo[DSM_MAX_OBJINFO_LENGTH];  /*object-dependent info */
} qryARespBackupData;

#define qryARespBackupDataVersion  1

/*-------------------------------------------------------------------------+
|  Type definition for DOS Filespace attributes                            |
+-------------------------------------------------------------------------*/
typedef struct
{
    char         driveLetter ;          /* drive letter for filespace    */
    uint16       fsInfoLength;          /* fsInfo length used            */
    char         fsInfo[DSM_MAX_FSINFO_LENGTH];/*caller-determined data  */
} dsmDosFSAttrib ;

/*-------------------------------------------------------------------------+
|  Type definition for UNIX Filespace attributes                           |
+-------------------------------------------------------------------------*/
typedef struct
{
    uint16       fsInfoLength;          /* fsInfo length used            */
    char         fsInfo[DSM_MAX_FSINFO_LENGTH];/*caller-determined data  */
} dsmUnixFSAttrib ;

/*-------------------------------------------------------------------------+
|  Type definition for Filespace attributes on all Filespace calls         |
+-------------------------------------------------------------------------*/
typedef union
{
    dsmDosFSAttrib   dosFSAttr ;
    dsmUnixFSAttrib  unixFSAttr ;
} dsmFSAttr ;

/*-------------------------------------------------------------------------+
|  Type definition for fsUpd parameter on dsmUpdateFS()
+-------------------------------------------------------------------------*/
typedef struct S_dsmFSUpd
{
    uint16      stVersion ;             /* structure version              */
    char        *fsType ;               /* filespace type                 */
    uint64      occupancy ;             /* occupancy estimate             */
    uint64      capacity  ;             /* capacity estimate              */
    dsmFSAttr   fsAttr ;                /* platform specific attributes   */
} dsmFSUpd ;

#define dsmFSUpdVersion  1

/*-------------------------------------------------------------------------+
|  Type definition for Filespace queryBuffer on dsmBeginQuery()            |
+-------------------------------------------------------------------------*/
typedef struct S_qryFSData
{
    uint16  stVersion;                          /* structure version */
    char    *fsName;                            /* File space name */
} qryFSData;

#define qryFSDataVersion  1

/*-------------------------------------------------------------------------+
| Type definition for Query Filespace response on dsmGetNextQObj()         |
+-------------------------------------------------------------------------*/
typedef struct S_qryRespFSData
{
    uint16   stVersion;                              /* structure version */
    char     fsName[DSM_MAX_FSNAME_LENGTH + 1];         /* Filespace name */
    char     fsType[DSM_MAX_FSTYPE_LENGTH + 1] ;        /* Filespace type */
    uint64   occupancy;                       /* Occupancy est. in bytes. */
    uint64   capacity;                         /* Capacity est. in bytes. */
    dsmFSAttr fsAttr ;                        /*general fs info or attribs*/
} qryRespFSData;

#define qryRespFSDataVersion  1

/*-------------------------------------------------------------------------+
|  Type definition for regFilespace parameter on dsmRegisterFS()
+-------------------------------------------------------------------------*/
typedef struct S_regFSData
{
    uint16    stVersion;                             /* structure version */
    char     *fsName;                                   /* Filespace name */
    char     *fsType;                                   /* Filespace type */
    uint64    occupancy;                      /* Occupancy est. in bytes. */
    uint64    capacity;                        /* Capacity est. in bytes. */
    dsmFSAttr fsAttr ;                        /*general fs info or attribs*/
} regFSData;

#define regFSDataVersion  1

/*-------------------------------------------------------------------------+
|  Type definition for session info response on dsmQuerySessionInfo()      |
+-------------------------------------------------------------------------*/
typedef struct
{
  uint16     stVersion;              /* Structure version                */
     /*------------------------------------------------------------------*/
     /*           Server information                                     */
     /*------------------------------------------------------------------*/
  char       serverHost[DSM_MAX_SERVERNAME_LENGTH+1];
                                     /* Network host name of DSM server  */
  uint16     serverPort;             /* Server comm port on host         */
  dsmDate    serverDate;             /* Server's date/time               */
  char       serverType[DSM_MAX_SERVERTYPE_LENGTH+1];
                                     /* Server's execution platform      */
  uint16     serverVer;              /* Server's version number          */
  uint16     serverRel;              /* Server's release number          */
  uint16     serverLev;              /* Server's level number            */
  uint16     serverSubLev;           /* Server's sublevel number         */
     /*------------------------------------------------------------------*/
     /*           Client Defaults                                        */
     /*------------------------------------------------------------------*/
  char       nodeType[DSM_MAX_PLATFORM_LENGTH+1]; /*node/application type*/
  char       fsdelim;                /* File space delimiter             */
  char       hldelim;                /* Delimiter betw highlev & lowlev  */
  uint8      compression;            /* Compression flag                 */
  uint8      archDel;                /* Archive delete permission        */
  uint8      backDel;                /* Backup  delete permission        */
  uint32     maxBytesPerTxn;         /* for future use                   */
  uint16     maxObjPerTxn;           /* The max objects allowed in a txn */
     /*------------------------------------------------------------------*/
     /*           Session Information                                    */
     /*------------------------------------------------------------------*/
  char       id[DSM_MAX_ID_LENGTH+1];    /* Sign-in id node name         */
  char       owner[DSM_MAX_OWNER_LENGTH+1]; /* Sign-in owner             */
                                     /*   (for multi-user platforms)     */
  char       confFile[DSM_PATH_MAX + DSM_NAME_MAX +1];
                                     /* len is platform dep              */
                                     /* dsInit name of appl config file  */
  uint8      opNoTrace;              /* dsInit option - NoTrace = 1      */
     /*------------------------------------------------------------------*/
     /*           Policy Data                                            */
     /*------------------------------------------------------------------*/
  char       domainName[DSM_MAX_DOMAIN_LENGTH+1]; /* Domain name         */
  char       policySetName[DSM_MAX_PS_NAME_LENGTH+1];
                                     /* Active policy set name           */
  dsmDate    polActDate;             /* Policy set activation date       */
  char       dfltMCName[DSM_MAX_MC_NAME_LENGTH+1];/* Default Mgmt Class  */
  uint16     gpBackRetn;             /* Grace-period backup retention    */
  uint16     gpArchRetn;             /* Grace-period archive retention   */
} ApiSessInfo;

#define ApiSessInfoVersion  1


#endif /* _H_DSMAPITD */
