/***********************************************************************
* Name:
*        xapicont.c
*
* Function:
*        Functions invoked via menu selections
*
* Environment:
*        This is a PLATFORM-INDEPENDENT source file. As such it may
*        contain no dependencies on any specific operating system
*        environment or hardware platform.
*
* Description:
*        The file contains the functions that are invoked from the
*        menu processing code and tables.
***********************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "custom.h"         /* Definitions for platform specific types */
#include "xapiour.h"        /* our API functions                       */
#include "xapidef.h"        /* XOpen Sample API type definitions       */
#include "xapisess.h"       /* session control                         */
#include "xapiqry.h"        /* query Backup Services                   */
#include "xapirecv.h"       /* receive objects                         */
#include "xapisend.h"       /* send objects                            */
#include "xapidel.h"        /* delete objects                          */

#ifdef _NO_PROTO_
typedef BSA_Int16 (*func_proto)();
#else
typedef BSA_Int16 (*func_proto)(dialog *, BSA_UInt16 );
#endif

/*----------------------------------------------------------------------+
| Private routines
.----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static BSA_Int16 get_input();
static BSA_Int16 process_dialog();
static BSA_Int16 free_dialog_buffs();
#else /* use ANSI C required prototypes */
static BSA_Int16 get_input(dialog *line);
static BSA_Int16 process_dialog(
                     func_proto func,
                     dialog *sel_dialog,
                     BSA_UInt16 parm1);
static BSA_Int16 free_dialog_buffs(
                     dialog *sel_dialog);
#endif

/*----------------------------------------------------------------------+
| Public routines
.----------------------------------------------------------------------*/

/*----------------------------------------------------------------------+
| Name:    smpxapi_exit()
|
| Action:  Performs any cleanup necessary before exiting the sample
|          XOpen api main menu processing.
|
| Input:   sel_dialog       - Pointer to dialog table if any.
|
| Returns: RC_OK            - Successful
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_exit(sel_dialog)
   dialog *sel_dialog;
#else /* use ANSI C required prototypes */
BSA_Int16 smpxapi_exit(dialog *sel_dialog)
#endif

{

perform_exit();
return RC_OK;
} /* smpxapi_exit() */

/*----------------------------------------------------------------------+
| Name:    smpxapi_signon()
|
| Action:  Prompts user for node name and password and logs on to the
|          DSM server.
|
| Input:   sel_dialog       - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|          others           - Any other valid ADSM api rc.
|          RC_NO_MEMORY     - Memory allocation call failed.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_signon (sel_dialog)
   dialog *sel_dialog;
#else /* use ANSI C required prototypes */
BSA_Int16 smpxapi_signon (dialog *sel_dialog)
#endif
{
BSA_Int16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the signon dialog! ***\n");
  }
else rc = process_dialog(perform_signon,sel_dialog,0);

return rc;
} /* smpxapi_signon */

/*----------------------------------------------------------------------+
| Name:    smpxapi_backup()
|
| Action:  Prompts user for backup objects to create and then sends
|          those objects to the DSM server.
|
| Input:   sel_dialog       - Pointer to dialog table to process.
|
| Returns: RC_OK             - Successful
|          RC_NOT_IN_SESSION - Client is not in session with the
|                              server
|          RC_SESSION_FAILED - A X/Open api call failed stopping
|                              execution
|          RC_NO_SEED_STRING - No seed string is spezified
|          RC_LESS_BUFF_SIZE - Send buffer size is less than seed
|                              string size.
|          others            - Any other valid X/Open api rc.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_backup (sel_dialog)
   dialog *sel_dialog;
#else
BSA_Int16 smpxapi_backup (dialog *sel_dialog)
#endif
{
BSA_Int16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the backup dialog! ***\n");
  }
else rc = process_dialog((func_proto)perform_send,sel_dialog,
                         BSA_Backup_Send);

return rc;
} /* smpxapi_backup() */

/*----------------------------------------------------------------------+
| Name:    smpxapi_archive()
|
| Action:  Prompts user for archive objects to create and then sends those
|          objects to the DSM server.
|
| Input:   sel_dialog        - Pointer to dialog table to process.
|
| Returns: RC_OK             - Successful
|          RC_NOT_IN_SESSION - Client is not in session with the
|                              server
|          RC_SESSION_FAILED - A X/Open api call failed stopping
|                              execution
|          RC_NO_SEED_STRING - No seed string is spezified
|          RC_LESS_BUFF_SIZE - Send buffer size is less than seed
|                              string size.
|          others            - Any other valid X/Open API rc.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smxpapi_archive (sel_dialog)
   dialog *sel_dialog;
#else
BSA_Int16 smpxapi_archive (dialog *sel_dialog)
#endif
{
BSA_Int16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the archive dialog! ***\n");
  }
else rc = process_dialog((func_proto)perform_send,sel_dialog,
                         BSA_Archive_Send);

return rc;
} /* smxpapi_archive() */

/*----------------------------------------------------------------------+
| Name:    smpxapi_restore(dialog *sel_dialog)
|
| Action:  Prompts user for items to restore and then does so.
|
| Input:   sel_dialog       - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|          others           - Any other valid X/Open api rc.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_restore(sel_dialog)
   dialog *sel_dialog;
#else
BSA_Int16 smpxapi_restore(dialog *sel_dialog)
#endif
{
BSA_Int16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the restore dialog! ***\n");
  }
else rc = process_dialog((func_proto)perform_receive,
                         sel_dialog,
                         BSA_Restore_Get);

return rc;
} /* smpxapi_restore() */

/*----------------------------------------------------------------------+
| Name:    smpxapi_retrieve(dialog *sel_dialog)
|
| Action:  Prompts user for items to retrieve and then does so.
|
| Input:   sel_dialog      - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|          others           - Any other valid ADSM api rc.
|          RC_NO_MEMORY     - Memory allocation call failed.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_retrieve(sel_dialog)
   dialog *sel_dialog;
#else
BSA_Int16 smpxapi_retrieve(dialog *sel_dialog)
#endif
{
BSA_Int16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the restore dialog! ***\n");
  }
else rc = process_dialog((func_proto)perform_receive,
                         sel_dialog,
                         BSA_Retrieve_Get);

return rc;
} /* smpxapi_retrieve() */

/*----------------------------------------------------------------------+
| Name:    smpxapi_mar_backup()
|
| Action:  Prompts a user for backup objects and marks them inactive.
|
| Input:   sel_dialog       - Pointer to dialog table to process.
|
| Returns: RC_OK             - Successful
|          RC_NOT_IN_SESSION - Client is not in session with the
|                              server
|          RC_SESSION_FAILED - A X/Open api call failed stopping
|                              execution
|          others            - Any other valid X/Open api rc.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_mar_backup(sel_dialog)
   dialog *sel_dialog;
#else
BSA_Int16 smpxapi_mar_backup(dialog *sel_dialog)
#endif
{
BSA_Int16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the delete backup dialog! ***\n");
  }
else rc = process_dialog((func_proto)perform_mar_backup,
                         sel_dialog,
                         0);

return rc;
} /* smpxapi_mar_backup() */

/*----------------------------------------------------------------------+
| Name:    smpxapi_del_archive()
|
| Action:  Prompts a user for archive objects and deletes them. 
|
| Input:   sel_dialog       - Pointer to dialog table to process.
|
| Returns: RC_OK             - Successful
|          RC_NOT_IN_SESSION - Client is not in session with the
|                              server
|          RC_SESSION_FAILED - A X/Open api call failed stopping
|                              execution
|          others            - Any other valid X/Open api rc.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_del_archive(sel_dialog)
   dialog *sel_dialog;
#else
BSA_Int16 smpxapi_del_archive(dialog *sel_dialog)
#endif
{
BSA_Int16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the delete archive dialog! ***\n");
  }
else rc = process_dialog((func_proto)perform_del_archive,
                         sel_dialog,
                         0);

return rc;
} /* smpxapi_del_archive() */

/*----------------------------------------------------------------------+
| Name:    smpxapi_del_exit()
|
| Action:  Returns from delete menu
|
| Input:   None
|
| Output:  None
|
| Returns: RC_OK            - Successful
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_del_exit()
   dialog *sel_dialog;
#else /* use ANSI C required prototypes */
BSA_Int16 smpxapi_del_exit()
#endif

{

return RC_OK;
} /* smpxapi_del_exit() */

/*----------------------------------------------------------------------+
| Name:    smpxapi_chgpw()
|
| Action:  Changes the signon password.
|          XOpen api main menu processing.
|
| Input:   sel_dialog       - Pointer to dialog table if any.
|
| Returns: RC_OK            - Successful
|          RC_NO_MEMORY     - There is not enough memory available
|          others           - Any other valid ADSM api rc.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_chgpw(sel_dialog)
   dialog *sel_dialog;
#else /* use ANSI C required prototypes */
BSA_Int16 smpxapi_chgpw(dialog *sel_dialog)
#endif

{
BSA_Int16 rc = RC_OK;

if (!sel_dialog)
{
   printf("*** Error in coding the change password dialog! ***\n");
}
else 
{
   rc = process_dialog(perform_chgpw,sel_dialog,0);
} /* if */

return rc;
} /* smpxapi_chgpw() */

/*----------------------------------------------------------------------+
| Name:    smpxapi_qry_backup()
|
| Action:  Queries the server for backup objects.
|
| Input:   sel_dialog       - Pointer to dialog table to process.
|
| Returns: RC_OK             - Successful
|          RC_NOT_IN_SESSION - Client is not in session with the
|                              server
|          RC_SESSION_FAILED - A X/Open api call failed stopping
|                              execution
|          others            - Any other valid X/Open api rc.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_qry_backup (sel_dialog)
   dialog *sel_dialog;
#else
BSA_Int16 smpxapi_qry_backup (dialog *sel_dialog)
#endif
{
BSA_Int16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the query backup dialog! ***\n");
  }
else rc = process_dialog((func_proto)perform_query_object, 
                         sel_dialog,
                         BSA_Backup_Query);

return rc;
} /* smpxapi_qry_backup() */

/*----------------------------------------------------------------------+
| Name:    smpxapi_qry_archive()
|
| Action:  Queries the server for archive objects.
|
| Input:   sel_dialog       - Pointer to dialog table to process.
|
| Returns: RC_OK             - Successful
|          RC_NOT_IN_SESSION - Client is not in session with the
|                              server
|          RC_SESSION_FAILED - A X/Open api call failed stopping
|                              execution
|          others            - Any other valid X/Open api rc.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_qry_archive (sel_dialog)
   dialog *sel_dialog;
#else
BSA_Int16 smpxapi_qry_archive (dialog *sel_dialog)
#endif
{
BSA_Int16 rc = 0;

if (!sel_dialog)
  {
  printf("*** Error in coding the query archive dialog! ***\n");
  }
else rc = process_dialog((func_proto)perform_query_object, 
                         sel_dialog,
                         BSA_Archive_Query);

return rc;
} /* smpxapi_qry_archive() */

/*----------------------------------------------------------------------+
| Name:    smpxapi_query_exit()
|
| Action:  Returns from query menu
|
| Input:   None
|
| Output:  None
|
| Returns: RC_OK            - Successful
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_query_exit()
   dialog *sel_dialog;
#else /* use ANSI C required prototypes */
BSA_Int16 smpxapi_query_exit()
#endif

{

return RC_OK;
} /* smpxapi_query_exit() */

/*----------------------------------------------------------------------+
| Name:    smpxapi_qry_version()
|
| Action:  Determines the API verion.
|
| Input:   sel_dialog       - Pointer to dialog table if any.
|                             Null pointer for this call 
|
| Output:  None
|
| Returns: RC_OK            - Successful
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_qry_version(sel_dialog)
   dialog *sel_dialog;
#else /* use ANSI C required prototypes */
BSA_Int16 smpxapi_qry_version(dialog *sel_dialog)
#endif

{

perform_qry_version();
return RC_OK;
} /* smpxapi_qry_version() */

/*----------------------------------------------------------------------+
| Name:    smpxapi_qry_sess()
|
| Action:  Prints the session information.
|
| Input:   sel_dialog       - Pointer to dialog table if any.
|                             Null pointer for this call 
|
| Output:  None
|
| Returns: RC_OK            - Successful
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_qry_sess(sel_dialog)
   dialog *sel_dialog;
#else /* use ANSI C required prototypes */
BSA_Int16 smpxapi_qry_sess(dialog *sel_dialog)
#endif

{

BSA_Int16 rc;

rc = perform_qry_sess();

return RC_OK;
} /* smpxapi_qry_sess() */

/*----------------------------------------------------------------------+
| Name:    smpxapi_pref_query()
|
| Action:  Prints the preferences to standard output.
|
| Input:   sel_dialog       - Pointer to dialog table if any.
|                             Null pointer for this call
|
| Output:  None
|
| Returns: RC_OK            - Successful
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_pref_query(sel_dialog)
   dialog *sel_dialog;
#else /* use ANSI C required prototypes */
BSA_Int16 smpxapi_pref_query(dialog *sel_dialog)
#endif

{

perform_pref_query();
return RC_OK;
} /* smpxapi_pref_query() */

/*----------------------------------------------------------------------+
| Name:    smpxapi_pref_sendbuff()
|
| Action:  Changes the size of the send buffer size.
|
| Input:   sel_dialog       - Pointer to dialog table if any.
|
| Returns: RC_OK            - Successful
|          RC_NO_MEMORY     - There is not enough memory available
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_pref_sendbuff(sel_dialog)
   dialog *sel_dialog;
#else /* use ANSI C required prototypes */
BSA_Int16 smpxapi_pref_sendbuff(dialog *sel_dialog)
#endif

{
BSA_Int16 rc = RC_OK;

if (!sel_dialog)
{
   printf("*** Error in coding the change send buffer size dialog! ***\n");
}
else
{
   rc = process_dialog(perform_pref_sendbuff,sel_dialog,0);
} /* if */

return rc;
} /* smpxapi_pref_sendbuff() */

/*----------------------------------------------------------------------+
| Name:    smpxapi_pref_recvbuff()
|
| Action:  Changes the size of the receive buffer size.
|
| Input:   sel_dialog       - Pointer to dialog table if any.
|
| Returns: RC_OK            - Successful
|          RC_NO_MEMORY     - There is not enough memory available
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_pref_recvbuff(sel_dialog)
   dialog *sel_dialog;
#else /* use ANSI C required prototypes */
BSA_Int16 smpxapi_pref_recvbuff(dialog *sel_dialog)
#endif

{
BSA_Int16 rc = RC_OK;

if (!sel_dialog)
{
   printf("*** Error in coding the change receive buffer size dialog! ***\n");
}
else
{
   rc = process_dialog(perform_pref_recvbuff,sel_dialog,0);
} /* if */

return rc;
} /* smpxapi_pref_recvbuff() */

/*----------------------------------------------------------------------+
| Name:    smpxapi_pref_verbose()
|
| Action:  Changes the value of the verbose flag.
|
| Input:   sel_dialog       - Pointer to dialog table if any.
|
| Returns: RC_OK            - Successful
|          RC_NO_MEMORY     - There is not enough memory available
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_pref_verbose(sel_dialog)
   dialog *sel_dialog;
#else /* use ANSI C required prototypes */
BSA_Int16 smpxapi_pref_verbose(dialog *sel_dialog)
#endif

{
BSA_Int16 rc = RC_OK;

if (!sel_dialog)
{
   printf("*** Error in coding the change verbose flag dialog! ***\n");
}
else
{
   rc = process_dialog(perform_pref_verbose,sel_dialog,0);
} /* if */

return rc;
} /* smpxapi_pref_verbose() */

/*----------------------------------------------------------------------+
| Name:    smpxapi_pref_details()
|
| Action:  Changes the value of the details flag.
|
| Input:   sel_dialog       - Pointer to dialog table if any.
|
| Returns: RC_OK            - Successful
|          RC_NO_MEMORY     - There is not enough memory available
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_pref_details(sel_dialog)
   dialog *sel_dialog;
#else /* use ANSI C required prototypes */
BSA_Int16 smpxapi_pref_details(dialog *sel_dialog)
#endif

{
BSA_Int16 rc = RC_OK;

if (!sel_dialog)
{
   printf("*** Error in coding the change details flag dialog! ***\n");
}
else
{
   rc = process_dialog(perform_pref_details,sel_dialog,0);
} /* if */

return rc;
} /* smpxapi_pref_details() */

/*----------------------------------------------------------------------+
| Name:    smpxapi_pref_exit()
|
| Action:  Returns from preference menu
|
| Input:   None
|
| Output:  None
|
| Returns: RC_OK            - Successful
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 smpxapi_pref_exit()
   dialog *sel_dialog;
#else /* use ANSI C required prototypes */
BSA_Int16 smpxapi_pref_exit()
#endif

{

return RC_OK;
} /* smpxapi_pref_exit() */

/*----------------------------------------------------------------------+
| Private routines
.----------------------------------------------------------------------*/

/*----------------------------------------------------------------------+
| Name:    process_dialog()
|
| Action:  Displays the passed dialog, and after all entries entered
|          calls the passed function with the passed dialog.
|          On return from the called function the dialog memory will
|          be freed for another invocation.
|
| Input:   func()      - Pointer to the function to invoke
|          sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK             - Successful
|          RC_NOT_IN_SESSION - Client is not in session with the
|                              server
|          RC_SESSION_FAILED - A X/Open api call failed stopping
|                              execution
|          RC_NO_SEED_STRING - No seed string is spezified
|          RC_LESS_BUFF_SIZE - Send buffer size is less than seed
|                              string size.
|          others            - Any other valid X/Open api rc.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static BSA_Int16 process_dialog(func, sel_dialog, parm1)
   func_proto   func;
   dialog       *sel_dialog;
   BSA_UInt16   parm1;

#else
static BSA_Int16 process_dialog(func_proto func,
                                dialog *sel_dialog,
                                BSA_UInt16 parm1)
#endif
{
BSA_UInt16   i;
BSA_UInt16   done;
BSA_UInt16   done2;
BSA_UInt16   rc = 0;
char         s[50];
dialog       *dlg_item;

i=0;
done = FALSE;
while (!done)
  {
  dlg_item = &sel_dialog[i];                /* Specific item to work on  */
  if (dlg_item->item_type != BSA_API_END)   /* In not at end of dialog   */
    {
    if (rc = get_input(dlg_item))          /* Put msg out, check for error*/
      {
      printf("** Error on data input **  rc = %d\n",rc);
      done = TRUE;
      }
    else i++;                              /* Next item.                */
    }
  else                                     /* At end of dialog.         */
    {
    done2 = FALSE;
    while (!done2)
      {
      printf("Are the above responses correct (y/n/q)?\n");
      gets(s);
      switch (*s)
        {
        case 'n' :
        case 'N' :
          free_dialog_buffs(sel_dialog);
          done2 = TRUE;             /* Done with inner loop */
          break;
        case 'q' :
        case 'Q' :
          done = TRUE;              /* Done with outer loop */
          done2 = TRUE;             /* Done with inner loop */
          break;
        case 'y' :
        case 'Y' :
        default  :
          rc = func(sel_dialog,parm1);
          done = TRUE;              /* Done with outer loop */
          done2 = TRUE;             /* Done with inner loop */
        } /* switch */
      } /* while */
    i=0;
    } /* else */
  } /* while */
free_dialog_buffs(sel_dialog);
return rc;
}

/*----------------------------------------------------------------------+
| Name:    get_input
|
| Action:  Prompts user for input of passed dialog line.  Input is then
|          put into malloc'd storage in the passed buffer.
|
| Returns:
|     RC_OK            - Successful
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static BSA_Int16 get_input(line)
   dialog *line;
#else /* use ANSI C required prototypes */
static BSA_Int16 get_input
  (
  dialog *line
  )
#endif
{
char    *input_buff;
BSA_UInt32  len;
BSA_Int16   rc = 0;

input_buff = malloc(2049);
printf("%s",line->item_desc);
gets(input_buff);
len = strlen(input_buff);

if (!(line->item_buff = malloc(len + 1)))
  rc = RC_NO_MEMORY;
else
  strcpy(line->item_buff,input_buff);
free(input_buff);
return rc;

}

/*----------------------------------------------------------------------+
| Name:    free_dialog_buffs
|
| Action:  Free any buffers for dialogs and reset to NULL.
|
| Returns:
|     RC_OK            - Successful
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static BSA_Int16 free_dialog_buffs(sel_dialog)
   dialog *sel_dialog;
#else /* use ANSI C required prototypes */
static BSA_Int16 free_dialog_buffs
  (
  dialog *sel_dialog
  )
#endif
{
dialog *dlg;
BSA_UInt16 i;
BSA_UInt16 done;

i=0;
done = FALSE;
while (!done)
  {
  dlg = &sel_dialog[i];
  if (dlg->item_type != BSA_API_END)
    {
    if (dlg->item_buff)
      {
      free(dlg->item_buff);
      dlg->item_buff = NULL;
      i++;
      }
    }
  else done = TRUE;
  }
}
