/***********************************************************************
* Name:
*        xapidef.h
*
* Function:
*        Type definitions for the XOpen sample api code
*
* Environment:
*        This is a PLATFORM-INDEPENDENT source file. As such it may
*        contain no dependencies on any specific operating system
*        environment or hardware platform.
*
***********************************************************************/
#ifndef _H_XAPIDEF
#define _H_XAPIDEF

#ifndef TRUE
#define TRUE  0x01
#endif
#ifndef FALSE
#define FALSE 0x00
#endif

#define RC_OK             0
#define RC_NO_MEMORY      222
#define RC_NOT_IN_SESSION 12
#define RC_SESSION_FAILED 13
#define RC_NO_SEED_STRING 14
#define RC_LESS_BUFF_SIZE 15
#define RC_CHECK_FAILURE  16
 
/*------------------------------------------------------------------------.
| Dialog definition for the various input fields.
.------------------------------------------------------------------------*/
typedef struct dialog_s
  {
  BSA_UInt16 item_type;
  char       *item_desc;
  char       *item_buff;
  } dialog;

/*------------------------------------------------------------------------.
| Table definitions for menus and menu entries used in this application.
| These structures will be defined in the data portion of the application.
|
| These structures provide a very simple setup to offer line mode menu
| selection of items.  This way a single code segment can process all
| user input and invoke the proper routines without having to change the
| main line code.
.------------------------------------------------------------------------*/
typedef struct menu_s
  {
  struct menu_entry_s *mentry;  /* Pointer to the menu entries.          */
  char   *instructions;         /* Short instruction for menu.           */
  } menu;

typedef struct menu_entry_s
  {
  BSA_UInt16 selection;     /* Selection number (0 - n)                 */
  char       *sel_desc;     /* Selection description (Backup...         */
  BSA_Int16  (*sel_func)(); /* Function to call if selection chosen     */
  menu       *next_menu;    /* Pointer to next menu if nested.          */
  BSA_UInt16 exit_menu;     /* Boolean indicating if to exit from menu  */
  dialog     *sel_dialog;   /* Dialog to use.                           */
  } menu_entry;

#define MENU_END 65324        /* Arbitrary number to designate end of menu*/

/*------------------------------------------------------------------------.
| Item type definitions for dialog processing
.------------------------------------------------------------------------*/
#define BSA_API_NODENAME             0
#define BSA_API_OWNER                1
#define BSA_API_PASSWORD             2
#define BSA_API_OBJSPACE             3
#define BSA_API_PATH                 4
#define BSA_API_OBJTYPE              5
#define BSA_API_RESTYPE              6 
#define BSA_API_BSAOBJOWN            7
#define BSA_API_APPOBJOWN            8
#define BSA_API_FILESIZE             9 
#define BSA_API_NUM_FILES           10
#define BSA_API_SEED_STRING         11
#define BSA_API_DESCRIPTION         12
#define BSA_API_OBJINFO             13
#define BSA_API_LOWCREATE           14
#define BSA_API_UPCREATE            15
#define BSA_API_LOWEXPI             16
#define BSA_API_UPEXPI              17
#define BSA_API_STATUS              18
#define BSA_API_LIFECYCLE           19
#define BSA_API_COPYGROUP           20
#define BSA_API_CURRENTPW           21
#define BSA_API_NEWPW1              22
#define BSA_API_NEWPW2              23
#define BSA_API_SENDBUFFSIZE        24
#define BSA_API_RECVBUFFSIZE        25
#define BSA_API_VERBOSEFLAG         26
#define BSA_API_DETAILSFLAG         27
#define BSA_API_END              30999

/*------------------------------------------------------------------------.
| Optional parm defs for process dialog calls
.------------------------------------------------------------------------*/
#define BSA_Backup_Send       1
#define BSA_Archive_Send      2
#define BSA_Restore_Get       3
#define BSA_Retrieve_Get      4
#define BSA_Backup_Query      5
#define BSA_Archive_Query     6

#endif

