/***********************************************************************
* Name:
*        xapidel.c
*
* Function:
*        Functions invoked via menu selections
*
* Environment:
*        This is a PLATFORM-INDEPENDENT source file. As such it may
*        contain no dependencies on any specific operating system
*        environment or hardware platform.
*
* Description:
*        The file contains the functions that are invoked from the
*        menu processing code and tables.
***********************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
 
#include "xapidel.h"        /* delete objects                          */
#include "xapiutil.h"       /* Handy utility routines.                 */
#include "xapiour.h"        /* our API functions                       */
#include "xapipref.h"       /* default values for buffer sizes        */
#include "xapilist.h"       /* type definitions and prototypes for the */
                            /* object descriptor list                  */
#include "xapiqry.h"        /* query Backup Services                   */

/*----------------------------------------------------------------------+
| Private routines
.----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 query_and_delete_objects();
BSA_Int16 delete_objects();
BSA_Int16 delete_all_objects();
BSA_Int16 delete_one_object();
BSA_Int16 query_and_mark_inactive();
BSA_Int16 mark_objects_inactive();
BSA_Int16 mark_all_objects();
BSA_Int16 mark_one_object();
#else /* use ANSI C required prototypes */
BSA_Int16 query_and_delete_objects
                            (QueryDescriptor *queryDescriptorPtr);
BSA_Int16 delete_objects(TList objectDescList);
BSA_Int16 delete_all_objects(TList objectDescList);
BSA_Int16 delete_one_object(ObjectDescriptor *objectDescriptorPtr);
BSA_Int16 query_and_mark_inactive(ObjectName objName);
BSA_Int16 mark_objects_inactive(TList objectDescList);
BSA_Int16 mark_all_objects(TList objectDescList);
BSA_Int16 mark_one_object(ObjectDescriptor *objectDescriptorPtr);
#endif 

/*----------------------------------------------------------------------+
| Public routines
.----------------------------------------------------------------------*/
 
/*----------------------------------------------------------------------+
| Name:    perform_del_archive()
|
| Action:  Take the information passed in the dialog to this function,
|          build the proper query, issue it, and delete all archive
|          objects. 
|
| Input:   sel_dialog        - Pointer to dialog table to process.
|
| Returns: RC_OK             - Successful
|          RC_NO_MEMORY      - Not enough memory
|          RC_NOT_IN_SESSION - Client is not in session with the server
|          RC_SESSION_FAILED - A X/Open api call failed stopping
|                              execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 perform_del_archive(sel_dialog, get_type)
   dialog     *sel_dialog;
   BSA_UInt16 del_type;
#else
BSA_Int16 perform_del_archive(dialog     *sel_dialog,
                              BSA_UInt16 del_type)
#endif
{
 
BSA_Int16       rc;
BSA_Int16        i;
QueryDescriptor *queryDescriptorPtr;
dialog          *dlg;

/*----------------------------------------------------------------------
| Are we signed on?
.----------------------------------------------------------------------*/
rc = check_signon();
if (rc)
{
   return rc;
}

/*----------------------------------------------------------------------
| Create a Query Descriptor and initialize the input parameters.
.----------------------------------------------------------------------*/
queryDescriptorPtr = create_queryDesc();
if (queryDescriptorPtr == NULL)
{
   /* no memory available */
   rc = RC_NO_MEMORY;
   return rc;
} /* if */

queryDescriptorPtr -> copyType = BSACopyType_ARCHIVE;
 
/*-----------------------------------------------------------------------
|  First pull all values out of the passed dialog for our use.
.-----------------------------------------------------------------------*/
i = 0;
dlg = &sel_dialog[i];
while (dlg->item_type != BSA_API_END)
{
   switch (dlg->item_type)
   {
      case BSA_API_OBJSPACE :
         strcpy(queryDescriptorPtr->objName.objectSpaceName,
                dlg->item_buff);
         break;
      case BSA_API_PATH :
         strcpy(queryDescriptorPtr->objName.pathName,
                dlg->item_buff);
         break;
      case BSA_API_OBJTYPE :
         switch (*dlg->item_buff)
         {
            case 'A':
            case 'a':
               queryDescriptorPtr->objectType =  BSAObjectType_ANY;
               break;
            case 'F':
            case 'f':
               queryDescriptorPtr->objectType =  BSAObjectType_FILE;
               break;
            case 'D':
            case 'd':
               queryDescriptorPtr->objectType =  BSAObjectType_DIRECTORY;
               break;
            case 'B':
            case 'b':
               queryDescriptorPtr->objectType =  BSAObjectType_DATABASE;
               break;
            default:
               queryDescriptorPtr->objectType =  BSAObjectType_FILE;
               break;
         }
         break;
      case BSA_API_RESTYPE :
         strcpy(queryDescriptorPtr->resourceType,dlg->item_buff);
         break;
      case BSA_API_BSAOBJOWN :
         strcpy(queryDescriptorPtr->owner.bsaObjectOwner,dlg->item_buff);
         break;
      case BSA_API_APPOBJOWN :
         strcpy(queryDescriptorPtr->owner.appObjectOwner,dlg->item_buff);
         break;
      case BSA_API_DESCRIPTION :
         if (strlen(dlg->item_buff) == 0)
         {
            /* use wildcard * */
            strcpy(queryDescriptorPtr->desc, "*");
         }
         else
         {
            strcpy(queryDescriptorPtr->desc, dlg->item_buff);
         } /* if */
         break;
      case BSA_API_LIFECYCLE :
         strcpy(queryDescriptorPtr->lGName,dlg->item_buff);
         break;
      case BSA_API_COPYGROUP :
         strcpy(queryDescriptorPtr->cGName,dlg->item_buff);
         break;
      case BSA_API_LOWCREATE :
         queryDescriptorPtr->createTimeLB = get_time(dlg->item_buff,
                                                 ADSM_LOWEST_BOUND);
         break;
      case BSA_API_UPCREATE :
         queryDescriptorPtr->createTimeUB = get_time(dlg->item_buff,
                                                ADSM_HIGHEST_BOUND);
         break;
      case BSA_API_LOWEXPI :
         queryDescriptorPtr->expireTimeLB = get_time(dlg->item_buff,
                                                 ADSM_LOWEST_BOUND);
         break;
      case BSA_API_UPEXPI :
         queryDescriptorPtr->expireTimeUB = get_time(dlg->item_buff,
                                                ADSM_HIGHEST_BOUND);
         break;
      case BSA_API_STATUS :
         switch (*dlg->item_buff)
         {
            case 'A':
            case 'a':
               queryDescriptorPtr->status =  BSAObjectStatus_ACTIVE;
               break;
            case 'I':
            case 'i':
               queryDescriptorPtr->status =  BSAObjectStatus_INACTIVE;
               break;
            case 'N':
            case 'n':
               queryDescriptorPtr->status =  BSAObjectStatus_ANY;
               break;
            default:
               queryDescriptorPtr->status =  BSAObjectStatus_ACTIVE;
               break;
         }
         break;         
      default :
         ;
   }
   i++;
   dlg = &sel_dialog[i];
}
 
/*----------------------------------------------------------------------
| Delete all the Objects.
.----------------------------------------------------------------------*/
rc = query_and_delete_objects(queryDescriptorPtr);
if (rc)
{
   return rc;
}

/*----------------------------------------------------------------------
| Do not forget to free the query object descriptor.
.----------------------------------------------------------------------*/
dispose_queryDesc( queryDescriptorPtr );

return RC_OK;
} /*  perform_del_archive() */

/*----------------------------------------------------------------------+
| Name:    perform_mar_backup()
|
| Action:  Marks backup objects as inactive. 
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|          others           - Any other valid X/Open api rc.
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 perform_mar_backup(sel_dialog, parm1)
   dialog *sel_dialog;
   BSA_UInt16 parm1;
#else /* use ANSI C required prototypes */
BSA_Int16 perform_mar_backup(dialog *sel_dialog,BSA_UInt16 parm1)
#endif

{

BSA_Int16      rc = RC_OK;
ObjectName     objName;
dialog         *dlg;
BSA_Int16      i;

/*----------------------------------------------------------------------
| Are we signed on?
.----------------------------------------------------------------------*/
rc = check_signon();
if (rc)
{
   return rc;
} /* if */

i = 0;
dlg = &sel_dialog[i];
while (dlg->item_type != BSA_API_END)
{
   switch (dlg->item_type)
   {
   case BSA_API_OBJSPACE:
      strcpy(objName.objectSpaceName, dlg->item_buff);
      break;
   case BSA_API_PATH:
      strcpy(objName.pathName, dlg->item_buff);
      break;
   default :
      printf("*** Delete backup dialog does not match code ");
      printf("in perform_mar_backup! ***\n");
   } /* switch */
   i++;
   dlg = &sel_dialog[i];
} /* while */

rc = query_and_mark_inactive(objName);

return rc;
} /* perform_mar_backup() */

/*----------------------------------------------------------------------+
| Private routines
.----------------------------------------------------------------------*/

/*----------------------------------------------------------------------+
| Name:    query_and_delete_objects()
|
| Action:  Query and delete all objects matching the passed
|          query descriptor. 
|
| Input:   *queryDescriptorPtr - query descriptor for objects to
|                                delete
|
| Returns: RC_OK             - Successful
|          RC_NOT_IN_SESSION - Client is not in session with the server
|          RC_SESSION_FAILED - A X/Open api call failed stopping
|                              execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 query_and_delete_objects(queryDescriptorPtr)
   QueryDescriptor *queryDescriptorPtr;
#else
BSA_Int16 query_and_delete_objects(QueryDescriptor *queryDescriptorPtr)
#endif
{

BSA_Int16       rc;
TList           objectDescList;
 
rc = get_objectDesc_list(queryDescriptorPtr, &objectDescList);
if (rc == RC_OK)
{
   if (IsEmpty_List(objectDescList))
   {
      printf("No object matches the query condition.\n");
      rc = RC_OK;
   }
   else
   {
      rc = delete_objects(objectDescList);
   } /* if */
} /* if */
Dispose_List(&objectDescList);  
return rc;
} /* query_and_delete_objects() */

/*----------------------------------------------------------------------+
| Name:    delete_objects() 
|
| Action:  Starts a transaction and deletes all archive objects using
|          the object descriptor list. After all objects have been 
|          deleted the transaction will be commited.
|          If an error condition occurs the transaction will be
|          aborted. 
|
| Input:   objectDescList      - object descriptor list
|
| Returns: RC_OK               - Successful
|          RC_NO_MEMORY        - Not enough memory
|          RC_NOT_IN_SESSION   - Client is not in session with the server
|          RC_SESSION_FAILED   - A X/Open api call failed stopping
|                                execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 delete_objects(objectDescList)
   TList           objectDescList;
#else
BSA_Int16 delete_objects(TList objectDescList)
#endif
{

BSA_Int16        rc;

rc = ourBeginTxn();
if (!rc)
{
   rc = delete_all_objects(objectDescList);
   if (rc)
   {
      ourEndTxn( BSAVote_ABORT );
   }
   else
   {
      ourEndTxn( BSAVote_COMMIT );
   } /* if */
} /* if */

return rc;
} /* delete_objects() */

/*----------------------------------------------------------------------+
| Name:    delete_all_objects() 
|
| Action:  Step through the list and deletes any object. 
|
| Input:   objectDescList      - object descriptor list
|
| Returns: RC_OK               - Successful
|          RC_NO_MEMORY        - Not enough memory
|          RC_NOT_IN_SESSION   - Client is not in session with the server
|          RC_SESSION_FAILED   - A X/Open api call failed stopping
|                                execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 delete_all_objects(objectDescList)
   TList           objectDescList;
#else
BSA_Int16 delete_all_objects(TList objectDescList)
#endif
{

BSA_Int16        rc;
BSA_Int16        rc_max;
ObjectDescriptor *objectDescriptorPtr;

rc_max = RC_OK;
GoStart_List(objectDescList);
while (!OffEnd_List(objectDescList))
{ 
   objectDescriptorPtr = Current_List(objectDescList);
   StepNext_List(objectDescList);
   rc = delete_one_object(objectDescriptorPtr);
   rc_max = max16(rc, rc_max);
   if (rc)
   {
      printf("Object: %s %s cannot be deleted.\n",
              objectDescriptorPtr->objName.objectSpaceName,
              objectDescriptorPtr->objName.pathName);
   } /* if */
} /* while */

return rc_max;
} /* delete_all_objects() */

/*----------------------------------------------------------------------+
| Name:    delete_one_object() 
|
| Action:  Deletes one single archive object.
|
| Input:   *objectDescriptorPtr - Object descriptor of the object that
|                                 will be received.
|
| Returns: RC_OK                - Successful
|          RC_NO_MEMORY         - Not enough memory
|          RC_NOT_IN_SESSION    - Client is not in session with the server
|          RC_SESSION_FAILED    - A X/Open api call failed stopping
|                                 execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 delete_one_object()
   ObjectDescriptor *objectDescriptorPtr;
#else
BSA_Int16 delete_one_object(ObjectDescriptor *objectDescriptorPtr)
#endif
{

BSA_Int16        rc = RC_OK;
BSA_Boolean      verbose;

verbose = get_verbose();
rc = ourDeleteObject(objectDescriptorPtr -> copyType,
                     &(objectDescriptorPtr -> objName),
                     &(objectDescriptorPtr -> copyId));
if (rc == BSA_RC_OK)
{
   printf("Deleted: ");
   if (verbose == TRUE)
   {
      printf("\n");
      print_object_descriptor(objectDescriptorPtr);
   }
   else
   {
      printf("%s %s\n", 
             objectDescriptorPtr -> objName.objectSpaceName,
             objectDescriptorPtr -> objName.pathName);  
   } /* if */
} /* if */

return rc;
} /* delete_one_object() */

/*----------------------------------------------------------------------+
| Name:    query_and_mark_inactive()
|
| Action:  Query and mark all objects inactive which match the passed
|          object name. 
|
| Input:   *queryDescriptorPtr - query descriptor for objects to
|                                delete
|
| Returns: RC_OK             - Successful
|          RC_NO_MEMORY      - Not enough memory
|          RC_NOT_IN_SESSION - Client is not in session with the server
|          RC_SESSION_FAILED - A X/Open api call failed stopping
|                              execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 query_and_mark_inactive(objName)
   ObjectName objName;
#else
BSA_Int16 query_and_mark_inactive(ObjectName objName)
#endif
{

BSA_Int16       rc;
TList           objectDescList;
QueryDescriptor *queryDescriptorPtr;

/*----------------------------------------------------------------------
| Create a Query Descriptor and initialize the input parameters.
.----------------------------------------------------------------------*/
queryDescriptorPtr = create_queryDesc();
if (queryDescriptorPtr == NULL)
{
   /* no memory available */
   rc = RC_NO_MEMORY;
   return rc;
} /* if */
strcpy(queryDescriptorPtr -> objName.objectSpaceName,
       objName.objectSpaceName);
strcpy(queryDescriptorPtr -> objName.pathName, 
       objName.pathName);
queryDescriptorPtr -> copyType = BSACopyType_BACKUP;
queryDescriptorPtr -> objectType = BSAObjectType_ANY; 
queryDescriptorPtr -> status = BSAObjectStatus_ACTIVE;
 
rc = get_objectDesc_list(queryDescriptorPtr, &objectDescList);
if (rc == RC_OK)
{
   if (IsEmpty_List(objectDescList))
   {
      printf("No object matches the query condition.\n");
      rc = RC_OK;
   }
   else
   {
      rc = mark_objects_inactive(objectDescList);
   } /* if */
} /* if */

/*----------------------------------------------------------------------
| Do not forget to free the query descriptor and the list.
.----------------------------------------------------------------------*/
dispose_queryDesc( queryDescriptorPtr );
Dispose_List(&objectDescList);  
return rc;
} /* query_and_mark_inactive() */

/*----------------------------------------------------------------------+
| Name:    mark_objects_inactive()
|
| Action:  Starts a transaction and marks all backup objects inactive 
|          using the object descriptor list. After all objects have been
|          marked inactive the transaction will be commited.
|          If an error condition occurs the transaction will be
|          aborted.
|
| Input:   objectDescList      - object descriptor list
|
| Returns: RC_OK               - Successful
|          RC_NO_MEMORY        - Not enough memory
|          RC_NOT_IN_SESSION   - Client is not in session with the server
|          RC_SESSION_FAILED   - A X/Open api call failed stopping
|                                execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 mark_objects_inactive(objectDescList)
   TList           objectDescList;
#else
BSA_Int16 mark_objects_inactive(TList objectDescList)
#endif
{

BSA_Int16        rc;

rc = ourBeginTxn();
if (!rc)
{
   rc = mark_all_objects(objectDescList);
   if (rc)
   {
      ourEndTxn( BSAVote_ABORT );
   }
   else
   {
      ourEndTxn( BSAVote_COMMIT );
   } /* if */
} /* if */

return rc;
} /* mark_objects_inactive() */

/*----------------------------------------------------------------------+
| Name:    mark_all_objects()
|
| Action:  Step through the list and mark all objects inactive.
|
| Input:   objectDescList      - object descriptor list
|
| Returns: RC_OK               - Successful
|          RC_NO_MEMORY        - Not enough memory
|          RC_NOT_IN_SESSION   - Client is not in session with the server
|          RC_SESSION_FAILED   - A X/Open api call failed stopping
|                                execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 mark_all_objects(objectDescList)
   TList           objectDescList;
#else
BSA_Int16 mark_all_objects(TList objectDescList)
#endif
{

BSA_Int16        rc;
BSA_Int16        rc_max;
ObjectDescriptor *objectDescriptorPtr;

rc_max = RC_OK;
GoStart_List(objectDescList);
while (!OffEnd_List(objectDescList))
{
   objectDescriptorPtr = Current_List(objectDescList);
   StepNext_List(objectDescList);
   rc = mark_one_object(objectDescriptorPtr);
   rc_max = max16(rc, rc_max);
   if (rc != BSA_RC_OK)
   {
      printf("Object: %s %s cannot marked inactive.\n",
              objectDescriptorPtr->objName.objectSpaceName,
              objectDescriptorPtr->objName.pathName);
   } /* if */
} /* while */

return rc_max;
} /* mark_all_objects() */

/*----------------------------------------------------------------------+
| Name:    mark_one_object()
|
| Action:  Marks one single object inactive.
|
| Input:   *objectDescriptorPtr - Object descriptor of the object that
|                                 will be received.
|
| Returns: RC_OK                - Successful
|          RC_NO_MEMORY         - Not enough memory
|          RC_NOT_IN_SESSION    - Client is not in session with the server
|          RC_SESSION_FAILED    - A X/Open api call failed stopping
|                                 execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 mark_one_object()
   ObjectDescriptor *objectDescriptorPtr;
#else
BSA_Int16 mark_one_object(ObjectDescriptor *objectDescriptorPtr)
#endif
{

BSA_Int16        rc = RC_OK;
BSA_Boolean      verbose;

verbose = get_verbose();
rc = ourMarkObjectInactive(&(objectDescriptorPtr->objName));
if (rc == BSA_RC_OK)
{
   printf("Marked Inactive: ");
   if (verbose == TRUE)
   {
      printf("\n");
      print_object_descriptor(objectDescriptorPtr);
   }
   else
   {
      printf("%s %s\n", 
             objectDescriptorPtr -> objName.objectSpaceName,
             objectDescriptorPtr -> objName.pathName);  
   } /* if */
} /* if */

return rc;
} /* mark_one_object() */

