/***********************************************************************
* Name:
*        xapidisp.c
*
* Function:
*        Module to display the menus, accept user input and
*        perform corresponding function calls.
*
* Status:
*        Version 1, Release 0
*
* Environment:
*        This is a PLATFORM-INDEPENDENT source file. As such it may
*        contain no dependencies on any specific operating system
*        environment or hardware platform.
*
* Description:
*        The file contains the functions to display menus and
*        handle user input.
***********************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "xapidisp.h"     /* Display menus and accept user input      */
#include "custom.h"       /* Definitions for platform specific types  */
#include "xapidef.h"      /* XOpen Sample API type definitions        */
#include "xapidata.h"     /* Menu data                                */

/*----------------------------------------------------------------------+
| Private routines
.----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static void Process_Menu();
static void Display_Menu();
#else /* use ANSI C required prototypes */
static void Process_Menu(menu *xmenu);
static void Display_Menu(menu *xmenu);
#endif

/*----------------------------------------------------------------------+
| Public routines
.----------------------------------------------------------------------*/

/*----------------------------------------------------------------------+
| Name:    Process_Main_Menu()
|
| Action:  display the main menu and accept
|          user input.  Perform proper function calls based on user
|          input as directed from menu structure.
|
| Input:   None
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void Process_Main_Menu()
#else /* use ANSI C required prototypes */
void Process_Main_Menu()
#endif

{

Process_Menu(main_menu);

return;
} /* Process_Main_Menu() */

/*----------------------------------------------------------------------+
| Private routines
.----------------------------------------------------------------------*/

/*----------------------------------------------------------------------+
| Name:    Process_Menu()
|
| Action:  Take the passed menu data structure and display it and accept
|          user input.  Perform proper function calls based on user
|          input as directed from menu structure.
|
| Input:   xmenu            - Pointer to the menu to process.
|
| Returns: RC_OK            - Successful
|          RC_NO_MEMORY     - Memory allocation call failed.
|
| Side
| Effects: None
|
| Notes:   This is a recursive procedure to handle nested menus.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static void Process_Menu(xmenu)
   menu *xmenu;
#else /* use ANSI C required prototypes */
static void Process_Menu(menu *xmenu)
#endif

{
unsigned char menu_input[512];
int i;
int sel;  /* for selected menu entry */
int rc;
char valid_entry;
char done;
menu_entry *menu_item;

valid_entry = FALSE;
done        = FALSE;

/*---------------------------------------------------------------------*/
/* Loop through the menu processing good entries and flagging bad.     */
/* Exit when user indicates by invoking the proper function.           */
/*---------------------------------------------------------------------*/
while (!valid_entry || !done)
  {
  memset((void*)menu_input,0x00,sizeof(menu_input)); /* clear out buff.*/
  Display_Menu(xmenu);        /* Display the menu being processed   */
  gets((char *)menu_input);   /* Get user input.                    */
  if (menu_input[0] == 0x00) 
  {
    strcpy((char *)menu_input,"9999"); /* No input */
  } /* if */
  sel = atoi((char *)menu_input);/* Convert to integer for compares.   */

  /* Process each item in the menu to see if it was chosen.         */
  for(i=0;;i++)             /*  Process till we find the end or hit */
    {
    menu_item = &xmenu->mentry[i];
    if (menu_item->selection == sel)  /* Got a match?  */
      {
      valid_entry = TRUE;             /* Yep..set to true.   */
      if (menu_item->sel_func)        /* Supposed to call a function? */
        {
        rc = (*menu_item->sel_func)(menu_item->sel_dialog); /* Call function*/
        if (menu_item->exit_menu) done = TRUE;              /* Exit menu    */
        }
      else
        {
        if (menu_item->next_menu)        /* Nested menu?  */
          {
          Process_Menu(menu_item->next_menu);      /* Recursive menu call */
          if (menu_item->exit_menu) done = TRUE;   /* Exit menu  */
          }
        else
          {
          printf("Code error on definition of menu item %d %s\n",
                  menu_item->selection,menu_item->sel_desc);
          printf("Sample XOpen API exiting.\n");
          exit(99);
          }
        }
      break;  /* Found & processed entry. */
      }
    if (menu_item->selection == MENU_END)
      {
      valid_entry = FALSE;
      break;
      }
    }
  if (!valid_entry) printf("Invalid selection.  Try again.\n",menu_input);
  }
}

/*----------------------------------------------------------------------+
| Name:    display_menu()
|
| Action:  Take the passed menu data structure and display it to the
|          screen.  Once displayed return to the caller.
|
| Input:   xmenu       - Pointer to the menu to process.
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static void Display_Menu(xmenu)
   menu *xmenu;
#else /* use ANSI C required prototypes */
static void Display_Menu(menu *xmenu)
#endif

{
int i;
menu_entry *menu_item;

printf("%s\n",xmenu->instructions);

for(i=0;;i++)
  {
  menu_item = &xmenu->mentry[i];
  if (menu_item->selection == MENU_END) break;
  printf("%2d.  %s\n",menu_item->selection,
                    menu_item->sel_desc);
  }

printf("\nEnter selection ==>");

}


