/***********************************************************************
* Name:
*        xapipref.c
*
* Function:
*        This module contains the preferences such as default
*        values for the send/receive buffer size, verbose output
*        flag and flag for query details.
*
* Status:
*        Version 1, Release 0
*
* Environment:
*        This is a PLATFORM-INDEPENDENT source file. As such it may
*        contain no dependencies on any specific operating system
*        environment or hardware platform.
*
* Description:
*
***********************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "xapipref.h"       /* default values for buffer sizes        */

#ifndef TRUE
#define TRUE  0x01
#endif
#ifndef FALSE
#define FALSE 0x00
#endif

/*------------------------------------------------------------------------.
| Data structure for sample preferences
.------------------------------------------------------------------------*/
typedef struct
{
   BSA_UInt32  sz_send_buff;     /* Size of buffer to use for data sends.*/
   BSA_UInt32  sz_recv_buff;     /* Size of buffer to use for data recvs.*/
   BSA_Boolean verbose;          /* Should output be verbose or normal?  */
   BSA_Boolean qry_details;      /* Show details on queries?             */
} preferences;

static preferences  pref;
static char         *xapi_eyecatcher = "X/Open API Verify Data";

/*----------------------------------------------------------------------+
| Name:    set_preferences()
|
| Action:  Set the preference to default values.
|
| Input:   None 
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void set_preferences()
#else /* use ANSI C required prototypes */
void set_preferences()
#endif
{
pref.sz_send_buff = 32000;     /* Send buffer size */
pref.sz_recv_buff = 32000;     /* Receive buffer size */
pref.verbose      = TRUE;      /* Show verbose output? */
pref.qry_details  = TRUE;      /* Show details on queries? */

return;
} /* set_preferences() */

/*----------------------------------------------------------------------+
| Name:    get_send_buff_size
|
| Action:  Retruns the size of the send buffer.
|
| Input:   None
|
| Returns: size of the send buffer
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_UInt32 get_send_buff_size()
#else /* use ANSI C required prototypes */
BSA_UInt32 get_send_buff_size()
#endif
{

return pref.sz_send_buff;
} /* get_send_buff_size() */

/*----------------------------------------------------------------------+
| Name:    get_recv_buff_size
|
| Action:  Retruns the size of the receive buffer.
|
| Input:   None
|
| Returns: size of the receive buffer
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_UInt32 get_recv_buff_size()
#else /* use ANSI C required prototypes */
BSA_UInt32 get_recv_buff_size()
#endif
{

return pref.sz_recv_buff;
} /* get_recv_buff_size() */

/*----------------------------------------------------------------------+
| Name:    set_send_buff_size
|
| Action:  Sets the size of the send buffer.
|
| Input:   size of the send buffer
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void set_send_buff_size(size)
   BSA_UInt32 size
#else /* use ANSI C required prototypes */
void set_send_buff_size(BSA_UInt32 size)
#endif
{

pref.sz_send_buff = size;

return;
} /* set_send_buff_size() */

/*----------------------------------------------------------------------+
| Name:    set_recv_buff_size
|
| Action:  Sets the size of the receive buffer.
|
| Input:   size of the receive buffer
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void set_recv_buff_size(size)
   BSA_UInt32 size
#else /* use ANSI C required prototypes */
void set_recv_buff_size(BSA_UInt32 size)
#endif
{

pref.sz_recv_buff = size;

return;
} /* set_recv_buff_size() */

/*----------------------------------------------------------------------+
| Name:    get_verbose
|
| Action:  Retruns the value of the verbose flag.
|
| Input:   None
|
| Returns: TRUE or FALSE
|
| Side
| Effects: None
|
| Notes:   The verbose flag indicate whether a more detailed
|          output will be printed or not.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Boolean get_verbose()
#else /* use ANSI C required prototypes */
BSA_Boolean get_verbose()
#endif
{

BSA_Boolean rc;

if (pref.verbose)
{
   rc = TRUE;
}
else
{
   rc = FALSE;
} /* if */

return rc;
} /* get_verbose() */

/*----------------------------------------------------------------------+
| Name:    set_verbose
|
| Action:  Sets the value of the verbose flag.
|
| Input:   verbose - TRUE or FALSE
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   The verbose flag indicate whether a more detailed
|          output will be printed or not.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void set_verbose(verbose)
   BSA_Boolean verbose
#else /* use ANSI C required prototypes */
void set_verbose(BSA_Boolean verbose)
#endif
{

if (verbose)
{
   pref.verbose = TRUE;
}
else
{
   pref.verbose = FALSE;
} /* if */

return;
} /* set_verbose() */

/*----------------------------------------------------------------------+
| Name:    get_qry_detail
|
| Action:  Retruns the value of the query details flag.
|
| Input:   None
|
| Returns: TRUE or FALSE
|
| Side
| Effects: None
|
| Notes:   The query details flag indicates whether a more detailed
|          output will be printed or not.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Boolean get_qry_detail()
#else /* use ANSI C required prototypes */
BSA_Boolean get_qry_detail()
#endif
{

BSA_Boolean rc;

if (pref.qry_details)
{
   rc = TRUE;
}
else
{
   rc = FALSE;
} /* if */

return rc;
} /* get_qry_detail() */

/*----------------------------------------------------------------------+
| Name:    set_qry_detail
|
| Action:  Sets the value of the query details flag.
|
| Input:   detail - TRUE or FALSE
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   The query detail flag indicates whether a more detailed
|          output will be printed or not.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void set_qry_detail(detail)
   BSA_Boolean detail
#else /* use ANSI C required prototypes */
void set_qry_detail(BSA_Boolean detail)
#endif
{

if (detail)
{
   pref.qry_details = TRUE;
}
else
{
   pref.qry_details = FALSE;
} /* if */

return;
} /* set_qry_detail() */

/*----------------------------------------------------------------------+
| Name:    set_eyecatcher()
|
| Action:  Set the eye catcher string.
|
| Input:   eye catcher string
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   The passed string will be used to determine whether an
|          object on the server has been created with this sample
|          application. In this case the stored data can be verified.
|          The passed string will be copied to the xapi_eyecatcher
|          variable.
|          
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void set_eyecatcher()
#else /* use ANSI C required prototypes */
void set_eyecatcher(char *eyecatcher)
#endif
{

BSA_UInt16 length;

length = strlen(eyecatcher);
free(xapi_eyecatcher);
xapi_eyecatcher = malloc(length);
if (xapi_eyecatcher != NULL)
{
   strcpy(xapi_eyecatcher, eyecatcher);
} /* if */

return;
} /* set_eyecatcher() */

/*----------------------------------------------------------------------+
| Name:    get_eyecatcher_copy()
|
| Action:  Returns a copy of the eye catcher string.
|
| Input:   None
|
| Returns: Copy of the eye catcher string or NULL when no memory is
|          available
|
| Side
| Effects: None
|
| Notes:   This function allocates memory and copies the eye catcher
|          string to the allocated memory.
|          
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
char *get_eyecatcher_copy()
#else /* use ANSI C required prototypes */
char *get_eyecatcher_copy()
#endif
{

char       *copy;
BSA_UInt16 length;

length = strlen(xapi_eyecatcher);
copy   = malloc(length);
if (copy != NULL)
{
   strcpy(copy, xapi_eyecatcher);
} /* if */

return copy;
} /* get_eyecatcher_copy() */

/*----------------------------------------------------------------------+
| Name:    is_equal_eyecatcher()
|
| Action:  Compares the eye catcher string with the passed string.
|
| Input:   a string
|
| Returns: TRUE if the passed string is equal to the eye catcher string
|          otherwise FALSE
|
| Side
| Effects: None
|
| Notes:   None
|          
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Boolean is_equal_eyecatcher()
#else /* use ANSI C required prototypes */
BSA_Boolean is_equal_eyecatcher(char *eyecatcher)
#endif
{

int         result;
BSA_Boolean rc;

result = strcmp(xapi_eyecatcher, eyecatcher);
if (result == 0)
{
   rc = TRUE;
}
else
{
   rc = FALSE;
} /* if */

return rc;
} /* is_equal_eyecatcher() */

