/***********************************************************************
* Name:
*        xapiqry.c
*
* Function:
*        Functions invoked via menu selections
*
* Environment:
*        This is a PLATFORM-INDEPENDENT source file. As such it may
*        contain no dependencies on any specific operating system
*        environment or hardware platform.
*
* Description:
*        The file contains the functions that are invoked from the
*        menu processing code and tables.
***********************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
 
#include "xapiqry.h"        /* query Backup Services                   */
#include "xapiutil.h"       /* Handy utility routines.                 */
#include "xapiour.h"        /* our API functions                       */
#include "xapint64.h"       /* 64 bit routines.                        */

/*----------------------------------------------------------------------+
| Private routines
.----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 query_all_objects();
BSA_Int16 get_all_descriptors();
BSA_Int16 get_first_descriptor();
BSA_Int16 get_next_descriptors();
#else /* use ANSI C required prototypes */
BSA_Int16 query_all_objects(QueryDescriptor *queryDescriptorPtr);
BSA_Int16 get_all_descriptors(QueryDescriptor *queryDescriptorPtr,
                              TList           objectDescList);
BSA_Int16 get_first_descriptor(QueryDescriptor *queryDescriptorPtr,
                               TList           objectDescList);
BSA_Int16 get_next_descriptors(TList           objectDescList);
#endif

/*----------------------------------------------------------------------+
| Public routines
.----------------------------------------------------------------------*/

/*----------------------------------------------------------------------+
| Name:    perform_query_object()
|
| Action:  Take the information passed in the dialog to this function,
|          build the proper query, issue it, and display resluts to the
|          screen.  This handles 2 query types:
|          Restore   BSA_Backup_Query and
|          Retrieve  BSA_Archive_Query
|
| Input:   sel_dialog        - Pointer to dialog table to process.
|
| Returns: RC_OK             - Successful
|          RC_NO_MEMORY      - Not enough memory
|          RC_NOT_IN_SESSION - Client is not in session with the server
|          RC_SESSION_FAILED - A X/Open api call failed stopping
|                              execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 perform_query_object(sel_dialog, query_type)
   dialog     *sel_dialog;
   BSA_UInt16 query_type;
#else
BSA_Int16 perform_query_object(dialog     *sel_dialog,
                               BSA_UInt16 query_type)
#endif
{

BSA_Int16       rc;
BSA_Int16        i;
QueryDescriptor *queryDescriptorPtr;
dialog          *dlg;

/*----------------------------------------------------------------------
| Are we signed on?
.----------------------------------------------------------------------*/
rc = check_signon();
if (rc)
{
   rc = RC_NOT_IN_SESSION;
}
else
{
   /*-------------------------------------------------------------------
   | Create a Query Descriptor and initialize the input parameters.
   .-------------------------------------------------------------------*/
   queryDescriptorPtr = create_queryDesc();
   if (queryDescriptorPtr == NULL)
   {
      /* no memory available */
      rc = RC_NO_MEMORY;
   }
   else
   {
      switch (query_type)
      {
         case BSA_Backup_Query:
            queryDescriptorPtr -> copyType = BSACopyType_BACKUP;
            break;
         case BSA_Archive_Query:
            queryDescriptorPtr -> copyType = BSACopyType_ARCHIVE;
            break;
         default:
            printf("\nError in code perform_query_object!\n\n");
            queryDescriptorPtr -> copyType = BSACopyType_BACKUP;
            break;
      } /* switch */

      /*-----------------------------------------------------------------
      |  First pull all values out of the passed dialog for our use.
      .-----------------------------------------------------------------*/
      i = 0;
      dlg = &sel_dialog[i];
      while (dlg->item_type != BSA_API_END)
      {
         switch (dlg->item_type)
         {
            case BSA_API_OBJSPACE :
               strcpy(queryDescriptorPtr->objName.objectSpaceName,
                      dlg->item_buff);
               break;
            case BSA_API_PATH :
               strcpy(queryDescriptorPtr->objName.pathName,
                      dlg->item_buff);
               break;
            case BSA_API_OBJTYPE :
               switch (*dlg->item_buff)
               {
                  case 'A':
                  case 'a':
                     queryDescriptorPtr->objectType =  
                                         BSAObjectType_ANY;
                     break;
                  case 'F':
                  case 'f':
                     queryDescriptorPtr->objectType =  
                                         BSAObjectType_FILE;
                     break;
                  case 'D':
                  case 'd':
                     queryDescriptorPtr->objectType =  
                                         BSAObjectType_DIRECTORY;
                     break;
                  case 'B':
                  case 'b':
                     queryDescriptorPtr->objectType =  
                                         BSAObjectType_DATABASE;
                     break;
                  default:
                     queryDescriptorPtr->objectType =  
                                         BSAObjectType_FILE;
                     break;
               } /* switch */
               break;
            case BSA_API_RESTYPE :
               strcpy(queryDescriptorPtr->resourceType,
                      dlg->item_buff);
               break;
            case BSA_API_BSAOBJOWN :
               strcpy(queryDescriptorPtr->owner.bsaObjectOwner,
                      dlg->item_buff);
               break;
            case BSA_API_APPOBJOWN :
               strcpy(queryDescriptorPtr->owner.appObjectOwner,
                      dlg->item_buff);
               break;
            case BSA_API_DESCRIPTION :
               if (strlen(dlg->item_buff) == 0)
               {
                  /* use wildcard * */
                  strcpy(queryDescriptorPtr->desc, "*");
               }
               else
               {
                  strcpy(queryDescriptorPtr->desc, dlg->item_buff);
               } /* if */
               break;
            case BSA_API_LIFECYCLE :
               strcpy(queryDescriptorPtr->lGName,dlg->item_buff);
               break;
            case BSA_API_COPYGROUP :
               strcpy(queryDescriptorPtr->cGName,dlg->item_buff);
               break;
            case BSA_API_LOWCREATE :
               queryDescriptorPtr->createTimeLB = 
                                   get_time(dlg->item_buff,
                                            ADSM_LOWEST_BOUND);
               break;
            case BSA_API_UPCREATE :
               queryDescriptorPtr->createTimeUB = 
                                   get_time(dlg->item_buff,
                                            ADSM_HIGHEST_BOUND);
               break;
            case BSA_API_LOWEXPI :
               queryDescriptorPtr->expireTimeLB = 
                                   get_time(dlg->item_buff,
                                            ADSM_LOWEST_BOUND);
               break;
            case BSA_API_UPEXPI :
               queryDescriptorPtr->expireTimeUB = 
                                   get_time(dlg->item_buff,
                                            ADSM_HIGHEST_BOUND);
               break;
            case BSA_API_STATUS :
               switch (*dlg->item_buff)
               {
                  case 'A':
                  case 'a':
                     queryDescriptorPtr->status =  
                                         BSAObjectStatus_ACTIVE;
                     break;
                  case 'I':
                  case 'i':
                     queryDescriptorPtr->status =  
                                         BSAObjectStatus_INACTIVE;
                     break;
                  case 'N':
                  case 'n':
                     queryDescriptorPtr->status =  
                                         BSAObjectStatus_ANY;
                     break;
                  default:
                     queryDescriptorPtr->status =  
                                         BSAObjectStatus_ACTIVE;
                     break;
               } /* switch */
               break;
            default :
               ;
         } /* switch */
         i++;
         dlg = &sel_dialog[i];
      } /* while */

      /*----------------------------------------------------------------
      | Receive all the Objects.
      .----------------------------------------------------------------*/
      rc = query_all_objects(queryDescriptorPtr);

      /*----------------------------------------------------------------
      | Do not forget to free the query object descriptor.
      .----------------------------------------------------------------*/
      dispose_queryDesc( queryDescriptorPtr );
   } /* if */
} /* if */

return rc;
} /*  perform_query_object() */

/*----------------------------------------------------------------------+
| Name:    get_objectDesc_list()
|
| Action:  Creates an object descriptor list and starts and ends
|          a transaction with the server
|
| Input:   *queryDescriptorPtr - query descriptor for objects to
|                                restore/retrieve
|
| Output:  *objectDescListPtr  - list of object descriptors
|
| Returns: RC_OK               - Successful
|          RC_NO_MEMORY        - Not enough memory
|          RC_NOT_IN_SESSION   - Client is not in session with the server
|          RC_SESSION_FAILED   - A X/Open api call failed stopping
|                                execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 get_objectDesc_list(queryDescriptorPtr, objectDescListPtr)
   QueryDescriptor *queryDescriptorPtr;
   TList           *objectDescListPtr;
#else
BSA_Int16 get_objectDesc_list(QueryDescriptor *queryDescriptorPtr,
                              TList           *objectDescListPtr)
#endif

{

BSA_Int16       rc;
TList           objectDescList;

rc = RC_OK;
objectDescList = Create_List();
*objectDescListPtr = objectDescList;
if (objectDescList == NULL)
{
   rc = RC_NO_MEMORY;
}
else
{
   rc = ourBeginTxn();
   if (rc)
   {
      return rc;
   }
   rc = get_all_descriptors(queryDescriptorPtr, objectDescList);
   if (rc)
   {
      ourEndTxn(BSAVote_ABORT);
   }
   else
   {
      ourEndTxn(BSAVote_COMMIT);
   } /* if */
} /* if */
*objectDescListPtr = objectDescList;

return rc;
} /* get_objectDesc_list() */

/*----------------------------------------------------------------------+
| Name:    print_descriptor_list() 
|
| Action:  Step through the list and print any object descriptor. 
|
| Input:   objectDescList      - object descriptor list
|
| Output:  None
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void print_descriptor_list(objectDescList)
   TList           objectDescList;
#else
void print_descriptor_list(TList objectDescList)
#endif
{

BSA_UInt32       number = 0;
ObjectDescriptor *objectDescriptorPtr;

GoStart_List(objectDescList);
while (!OffEnd_List(objectDescList))
{ 
   objectDescriptorPtr = Current_List(objectDescList);
   StepNext_List(objectDescList);
   number++;
   printf("Item %u:\n", number);
   print_object_descriptor(objectDescriptorPtr);
} /* while */

return;
} /* print_descriptor_list() */

/*----------------------------------------------------------------------+
| Private routines
.----------------------------------------------------------------------*/

/*----------------------------------------------------------------------+
| Name:    query_all_objects()
|
| Action:  Query and print all objects matching the passed
|          query descriptor.
|
| Input:   *queryDescriptorPtr - query descriptor for objects to
|                                print
|
| Returns: RC_OK             - Successful
|          RC_NOT_IN_SESSION - Client is not in session with the server
|          RC_SESSION_FAILED - A X/Open api call failed stopping
|                              execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 query_all_objects(queryDescriptorPtr)
   QueryDescriptor *queryDescriptorPtr;
#else
BSA_Int16 query_all_objects(QueryDescriptor *queryDescriptorPtr)
#endif
{

BSA_Int16       rc = RC_OK;
TList           objectDescList;

rc = get_objectDesc_list(queryDescriptorPtr, &objectDescList);
if (rc == RC_OK)
{
   if (IsEmpty_List(objectDescList))
   {
      printf("No object matches the query condition.\n");
      rc = RC_OK;
   }
   else
   {
      print_descriptor_list(objectDescList);
   } /* if */
} /* if */
Dispose_List(&objectDescList);
return rc;
} /* query_all_objects() */

/*----------------------------------------------------------------------+
| Name:    get_all_descriptors()
|
| Action:  This function controlls how the first and the remaining
|          object descriptors will be received.
|
| Input:   *queryDescriptorPtr - query descriptor for objects to
|                                 restore/retrieve
|          objectDescList      - an empty list for object descriptors
|
| Output:  objectDescList      - list of object descriptors that
|                                match the query
|
| Returns: RC_OK               - Successful
|          RC_NO_MEMORY        - Not enough memory
|          RC_NOT_IN_SESSION   - Client is not in session with the server
|          RC_SESSION_FAILED   - A X/Open api call failed stopping
|                                execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 get_all_descriptors(queryDescriptorPtr, objectDescList)
   QueryDescriptor *queryDescriptorPtr;
   TList           objectDescList;
#else
BSA_Int16 get_all_descriptors(QueryDescriptor *queryDescriptorPtr,
                              TList           objectDescList)
#endif
{

BSA_Int16       rc;

rc = RC_OK;
rc = get_first_descriptor(queryDescriptorPtr, objectDescList);
if ((rc == BSA_RC_NO_MATCH) || (rc == BSA_RC_NO_MORE_DATA))
{
   rc = RC_OK;
} /* if */
if (rc == BSA_RC_MORE_DATA)
{
   rc = get_next_descriptors(objectDescList);
   if (rc == BSA_RC_NO_MORE_DATA)
   {
      rc = RC_OK;
   } /* if */
} /* if */

return rc;
}  /* get_all_descriptors() */

/*----------------------------------------------------------------------+
| Name:    get_first_descriptor
|
| Action:  Create an object descriptor and receive the first
|          object descriptor that matches the query. This
|          object descriptor will be inserted into the object
|          descriptor list.
|
| Input:   *queryDescriptorPtr - query descriptor for objects to
|                                 restore/retrieve
|          objectDescList      - an empty list for object descriptors
|
| Output:  objectDescList      - list that contains the first
|                                object descriptor or an empty
|                                list if no object matches the query
|
| Returns: RC_OK               - Successful
|          RC_NO_MEMORY        - Not enough memory
|          RC_NOT_IN_SESSION   - Client is not in session with the server
|          RC_SESSION_FAILED   - A X/Open api call failed stopping
|                                execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 get_first_descriptor(queryDescriptorPtr, objectDescList)
   QueryDescriptor *queryDescriptorPtr;
   TList           objectDescList;
#else
BSA_Int16 get_first_descriptor(QueryDescriptor *queryDescriptorPtr,
                               TList           objectDescList)
#endif
{

BSA_Int16        rc;
ObjectDescriptor *objectDescriptorPtr;

rc = RC_OK;
objectDescriptorPtr = create_objDesc();
if (objectDescriptorPtr == NULL)
{
   rc = RC_NO_MEMORY;
}
else
{
   GoStart_List(objectDescList);
   rc = ourQueryObject(queryDescriptorPtr, objectDescriptorPtr);
   if (((rc != BSA_RC_OK) &&
        (rc != BSA_RC_MORE_DATA)) &&
        (rc != BSA_RC_NO_MORE_DATA))
   {
      dispose_objDesc( objectDescriptorPtr );
   }
   else
   {
      Insert_List(objectDescList, objectDescriptorPtr);
   } /* if */
} /* if */

return rc;
} /*  get_first_descriptor() */

/*----------------------------------------------------------------------+
| Name:    get_next_descriptors
|
| Action:  Create object descriptors and receive the remaining
|          object descriptors that matches the query. The
|          object descriptors will be inserted into the object
|          descriptor list.
|
| Input:   objectDescList      - object descriptor list containing
|                                one single entry
|
| Output:  objectDescList      - list that contains all
|                                object descriptors which match
|                                the query
|
| Returns: RC_OK               - Successful
|          RC_NO_MEMORY        - Not enough memory
|          RC_NOT_IN_SESSION   - Client is not in session with the server
|          RC_SESSION_FAILED   - A X/Open api call failed stopping
|                                execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 get_next_descriptors(objectDescList)
   TList            objectDescList;
#else
BSA_Int16 get_next_descriptors(TList           objectDescList)
#endif
{

BSA_Int16        rc;
ObjectDescriptor *objectDescriptorPtr;

rc = BSA_RC_MORE_DATA;

while (rc == BSA_RC_MORE_DATA)
{
   objectDescriptorPtr = create_objDesc();
   if (objectDescriptorPtr == NULL)
   {
      rc = RC_NO_MEMORY;
   }
   else
   {
      rc = ourGetNextQueryObject(objectDescriptorPtr);
      if (((rc != BSA_RC_OK) &&
           (rc != BSA_RC_MORE_DATA)) &&
           (rc != BSA_RC_NO_MORE_DATA))
      {
         dispose_objDesc( objectDescriptorPtr );
      }
      else
      {
         Insert_List(objectDescList, objectDescriptorPtr);
      } /* if */
   } /* if */
} /* while */

return rc;
} /*  get_next_descriptor() */

