/***********************************************************************
* Name:
*        xapirecv.c
*
* Function:
*        Functions invoked via menu selections
*
* Environment:
*        This is a PLATFORM-INDEPENDENT source file. As such it may
*        contain no dependencies on any specific operating system
*        environment or hardware platform.
*
* Description:
*        The file contains the functions that are invoked from the
*        menu processing code and tables.
***********************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
 
#include "xapirecv.h"       /* receive objects                         */
#include "xapiutil.h"       /* Handy utility routines.                 */
#include "xapiour.h"        /* our API functions                       */
#include "xapint64.h"       /* 64 bit routines.                        */
#include "xapilist.h"       /* type definitions and prototypes for the */
                            /* object descriptor list                  */
#include "xapiqry.h"        /* query Backup Services                   */

/*----------------------------------------------------------------------+
| Private routines
.----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 query_and_receive_objects();
BSA_Int16 receive_objects();
BSA_Int16 get_all_objects();
BSA_Int16 get_one_object();
BSA_Int16 get_object_and_check();
BSA_Int16 get_all_data();
#else /* use ANSI C required prototypes */
BSA_Int16 query_and_receive_objects
                            (QueryDescriptor *queryDescriptorPtr);
BSA_Int16 receive_objects(TList objectDescList);
BSA_Int16 get_all_objects(TList objectDescList);
BSA_Int16 get_one_object(ObjectDescriptor *objectDescriptorPtr);
BSA_Int16 get_object_and_check(ObjectDescriptor *objectDescriptorPtr,
                               DataBlock        *dataBlockPtr,
                               char             **seedString,
                               BSA_UInt16       *offsetPtr,
                               BSA_Int16        *check);
BSA_Int16 get_all_data(DataBlock        *dataBlockPtr,
                       char             *seedString,
                       BSA_UInt16        *offsetPtr,
                       BSA_Int16        *check);
#endif 

/*----------------------------------------------------------------------+
| Public routines
.----------------------------------------------------------------------*/
 
/*----------------------------------------------------------------------+
| Name:    perform_receive()
|
| Action:  Take the information passed in the dialog to this function,
|          build the proper query, issue it, and display resluts to the
|          screen.  This handles 2 query types:
|          Restore   BSA_Restore_Get and
|          Retrieve  BSA_Retrieve_Get 
|
| Input:   sel_dialog        - Pointer to dialog table to process.
|
| Returns: RC_OK             - Successful
|          RC_NO_MEMORY      - Not enough memory
|          RC_NOT_IN_SESSION - Client is not in session with the server
|          RC_SESSION_FAILED - A X/Open api call failed stopping
|                              execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 perform_receive(sel_dialog, get_type)
   dialog     *sel_dialog;
   BSA_UInt16 get_type;
#else
BSA_Int16 perform_receive(dialog     *sel_dialog,
                          BSA_UInt16 get_type)
#endif
{
 
BSA_Int16       rc;
BSA_Int16        i;
QueryDescriptor *queryDescriptorPtr;
dialog          *dlg;

/*----------------------------------------------------------------------
| Are we signed on?
.----------------------------------------------------------------------*/
rc = check_signon();
if (rc)
{
   return rc;
}

/*----------------------------------------------------------------------
| Create a Query Descriptor and initialize the input parameters.
.----------------------------------------------------------------------*/
queryDescriptorPtr = create_queryDesc();
if (queryDescriptorPtr == NULL)
   {
   /* no memory available */
   rc = RC_NO_MEMORY;
   return rc;
   }

queryDescriptorPtr -> copyType = get_type;
switch (get_type)
{
   case BSA_Restore_Get:
      queryDescriptorPtr -> copyType = BSACopyType_BACKUP;
      break;
   case BSA_Retrieve_Get: 
      queryDescriptorPtr -> copyType = BSACopyType_ARCHIVE;
      break;
   default:
      printf("\nError in code perform_receive!\n\n");
      queryDescriptorPtr -> copyType = BSACopyType_BACKUP;
      break;
} /* switch */

 
/*-----------------------------------------------------------------------
|  First pull all values out of the passed dialog for our use.
.-----------------------------------------------------------------------*/
i = 0;
dlg = &sel_dialog[i];
while (dlg->item_type != BSA_API_END)
{
   switch (dlg->item_type)
   {
      case BSA_API_OBJSPACE :
         strcpy(queryDescriptorPtr->objName.objectSpaceName,
                dlg->item_buff);
         break;
      case BSA_API_PATH :
         strcpy(queryDescriptorPtr->objName.pathName,
                dlg->item_buff);
         break;
      case BSA_API_OBJTYPE :
         switch (*dlg->item_buff)
         {
            case 'A':
            case 'a':
               queryDescriptorPtr->objectType =  BSAObjectType_ANY;
               break;
            case 'F':
            case 'f':
               queryDescriptorPtr->objectType =  BSAObjectType_FILE;
               break;
            case 'D':
            case 'd':
               queryDescriptorPtr->objectType =  BSAObjectType_DIRECTORY;
               break;
            case 'B':
            case 'b':
               queryDescriptorPtr->objectType =  BSAObjectType_DATABASE;
               break;
            default:
               queryDescriptorPtr->objectType =  BSAObjectType_FILE;
               break;
         }
         break;
      case BSA_API_RESTYPE :
         strcpy(queryDescriptorPtr->resourceType,dlg->item_buff);
         break;
      case BSA_API_BSAOBJOWN :
         strcpy(queryDescriptorPtr->owner.bsaObjectOwner,dlg->item_buff);
         break;
      case BSA_API_APPOBJOWN :
         strcpy(queryDescriptorPtr->owner.appObjectOwner,dlg->item_buff);
         break;
      case BSA_API_DESCRIPTION :
         if (strlen(dlg->item_buff) == 0)
         {
            /* use wildcard * */
            strcpy(queryDescriptorPtr->desc, "*");
         }
         else
         {
            strcpy(queryDescriptorPtr->desc, dlg->item_buff);
         } /* if */
         break;
      case BSA_API_LIFECYCLE :
         strcpy(queryDescriptorPtr->lGName,dlg->item_buff);
         break;
      case BSA_API_COPYGROUP :
         strcpy(queryDescriptorPtr->cGName,dlg->item_buff);
         break;
      case BSA_API_LOWCREATE :
         queryDescriptorPtr->createTimeLB = get_time(dlg->item_buff,
                                                 ADSM_LOWEST_BOUND);
         break;
      case BSA_API_UPCREATE :
         queryDescriptorPtr->createTimeUB = get_time(dlg->item_buff,
                                                ADSM_HIGHEST_BOUND);
         break;
      case BSA_API_LOWEXPI :
         queryDescriptorPtr->expireTimeLB = get_time(dlg->item_buff,
                                                 ADSM_LOWEST_BOUND);
         break;
      case BSA_API_UPEXPI :
         queryDescriptorPtr->expireTimeUB = get_time(dlg->item_buff,
                                                ADSM_HIGHEST_BOUND);
         break;
      case BSA_API_STATUS :
         switch (*dlg->item_buff)
         {
            case 'A':
            case 'a':
               queryDescriptorPtr->status =  BSAObjectStatus_ACTIVE;
               break;
            case 'I':
            case 'i':
               queryDescriptorPtr->status =  BSAObjectStatus_INACTIVE;
               break;
            case 'N':
            case 'n':
               queryDescriptorPtr->status =  BSAObjectStatus_ANY;
               break;
            default:
               queryDescriptorPtr->status =  BSAObjectStatus_ACTIVE;
               break;
         }
         break;         
      default :
         ;
   }
   i++;
   dlg = &sel_dialog[i];
}
 
/*----------------------------------------------------------------------
| Receive all the Objects.
.----------------------------------------------------------------------*/
rc = query_and_receive_objects(queryDescriptorPtr);
if (rc)
{
   return rc;
}

/*----------------------------------------------------------------------
| Do not forget to free the query object descriptor.
.----------------------------------------------------------------------*/
dispose_queryDesc( queryDescriptorPtr );

return RC_OK;
} /*  perform_receive() */

/*----------------------------------------------------------------------+
| Private routines
.----------------------------------------------------------------------*/

/*----------------------------------------------------------------------+
| Name:    query_and_receive_objects()
|
| Action:  Query and restore/retrieve all objects matching the passed
|          query descriptor. 
|
| Input:   *queryDescriptorPtr - query descriptor for objects to
|                                restore/retrieve
|
| Returns: RC_OK             - Successful
|          RC_NOT_IN_SESSION - Client is not in session with the server
|          RC_SESSION_FAILED - A X/Open api call failed stopping
|                              execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 query_and_receive_objects(queryDescriptorPtr)
   QueryDescriptor *queryDescriptorPtr;
#else
BSA_Int16 query_and_receive_objects(QueryDescriptor *queryDescriptorPtr)
#endif
{

BSA_Int16       rc;
TList           objectDescList;
 
rc = get_objectDesc_list(queryDescriptorPtr, &objectDescList);
if (rc == RC_OK)
{
   if (IsEmpty_List(objectDescList))
   {
      printf("No object matches the query condition.\n");
      rc = RC_OK;
   }
   else
   {
      rc = receive_objects(objectDescList);
   } /* if */
} /* if */
Dispose_List(&objectDescList);  
return rc;
} /* query_and_receive_objects() */

/*----------------------------------------------------------------------+
| Name:    receive_objects() 
|
| Action:  Starts a transaction and receives all objects using the
|          object descriptor list. After all objects are received
|          the transaction will be commited. If an error condition
|          occurs the transaction will be aborted. 
|
| Input:   objectDescList      - object descriptor list
|
| Returns: RC_OK               - Successful
|          RC_NO_MEMORY        - Not enough memory
|          RC_NOT_IN_SESSION   - Client is not in session with the server
|          RC_SESSION_FAILED   - A X/Open api call failed stopping
|                                execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 receive_objects(objectDescList)
   TList           objectDescList;
#else
BSA_Int16 receive_objects(TList objectDescList)
#endif
{

BSA_Int16        rc;

rc = ourBeginTxn();
if (!rc)
{
   rc = get_all_objects(objectDescList);
   if (rc)
   {
      ourEndTxn( BSAVote_ABORT );
   }
   else
   {
      ourEndTxn( BSAVote_COMMIT );
   } /* if */
} /* if */

return rc;
} /* receive_objects() */

/*----------------------------------------------------------------------+
| Name:    get_all_objects() 
|
| Action:  Step through the list and receive any object. 
|
| Input:   objectDescList      - object descriptor list
|
| Returns: RC_OK               - Successful
|          RC_NO_MEMORY        - Not enough memory
|          RC_NOT_IN_SESSION   - Client is not in session with the server
|          RC_SESSION_FAILED   - A X/Open api call failed stopping
|                                execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 get_all_objects(objectDescList)
   TList           objectDescList;
#else
BSA_Int16 get_all_objects(TList objectDescList)
#endif
{

BSA_Int16        rc;
BSA_Int16        rc_max;
ObjectDescriptor *objectDescriptorPtr;

rc_max = RC_OK;
GoStart_List(objectDescList);
while (!OffEnd_List(objectDescList))
{ 
   objectDescriptorPtr = Current_List(objectDescList);
   StepNext_List(objectDescList);
   rc = get_one_object(objectDescriptorPtr);
   rc_max = max16(rc, rc_max);
   if (rc)
   {
      printf("Object: %s %s cannot be received from server.\n",
              objectDescriptorPtr->objName.objectSpaceName,
              objectDescriptorPtr->objName.pathName);
   } /* if */
} /* while */

return rc_max;
} /* get_all_objects() */

/*----------------------------------------------------------------------+
| Name:    get_one_object() 
|
| Action:  Receive an object and all its data from the server.
|
| Input:  *objectDescriptorPtr - Object descriptor of the object that
|                                will be received.
|
| Returns: RC_OK               - Successful
|          RC_NO_MEMORY        - Not enough memory
|          RC_NOT_IN_SESSION   - Client is not in session with the server
|          RC_SESSION_FAILED   - A X/Open api call failed stopping
|                                execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 get_one_object()
   ObjectDescriptor *objectDescriptorPtr;
#else
BSA_Int16 get_one_object(ObjectDescriptor *objectDescriptorPtr)
#endif
{

BSA_Int16        rc;
BSA_Int16        rc_check;
BSA_Int16        check;
BSA_Int16        seedLen;
DataBlock        *dataBlockPtr;
char             *seedString;
BSA_UInt32       RecvBufferSize;
BSA_UInt16       offset = 0;

rc_check     = RC_OK;
check        = RC_OK;
RecvBufferSize = get_recv_buff_size();
dataBlockPtr   = create_dataBlock(RecvBufferSize);

rc = get_object_and_check(objectDescriptorPtr,
                          dataBlockPtr,
                          &seedString,
                          &offset,
                          &check);
rc_check = rc_check || check;

if (rc == BSA_RC_MORE_DATA)
{
   rc = get_all_data(dataBlockPtr, seedString, &offset, &check);
}
rc_check = rc_check || check;
dispose_dataBlock(dataBlockPtr);

seedLen      = strlen(seedString);
if (seedLen != 0)
{
   free(seedString);
   seedString = NULL;
   printf("Data validation for object ");
   printf("%s %s\n", 
           objectDescriptorPtr->objName.objectSpaceName,
           objectDescriptorPtr->objName.pathName);
   printf("has been performed: ");
   if (rc_check)
   {
      printf("Validation failure!\n\n");
   }
   else
   {
      printf("Validation O.K.\n\n");
   }
}
else
{
   printf("NO data validation for object: ");
   printf("%s %s\n\n",
           objectDescriptorPtr->objName.objectSpaceName,
           objectDescriptorPtr->objName.pathName);
}

ourEndData();

return rc;
} /*  get_one_object() */

/*----------------------------------------------------------------------+
| Name:    get_object_and_check()
|
| Action:  Receives an object from the server and checks the
|          data against the seed string 
|
| Input:   *objectDescriptorPtr - object descriptor of the object to
|                                 receive
|          *dataBlockPtr        - pointer to the data block
|          *offsetPtr           - pointer to an offset used for data
|                                 validation
|
| Output:  **seedString         - if eye catcher has been found
|                                    seed string is returned
|                                 else
|                                    empty string
|          *offsetPtr           - pointer to an offset used for data
|                                 validation
|          *check               - RC_OK if eye catcher is found in 
|                                 object info field object size,
|                                 copy type and data in buffer is O.K.
|                                 or no eye catcher is found
|                                 RC_CHECK_FAILURE validation failure 
|
| Returns: RC_OK                - Successful
|          RC_CHECK_FAILURE     - validation failure
|          others               - Any other valid X/Open api rc.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 get_object_and_check(objectDescriptorPtr, 
                               dataBlockPtr,
                               seedString,
                               offsetPtr,
                               check)
   ObjectDescriptor *objectDescriptorPtr;
   DataBlock        *dataBlockPtr;
   char             **seedString;
   BSA_UInt16       *offsetPtr;
   BSA_Int16        *check;
#else
BSA_Int16 get_object_and_check(ObjectDescriptor *objectDescriptorPtr, 
                               DataBlock        *dataBlockPtr,
                               char             **seedStringPtr,
                               BSA_UInt16       *offsetPtr,
                               BSA_Int16        *check)
 #endif
{

BSA_Int16       rc;
BSA_Int16       seedlen;
BSA_Int16       rc_check;
BSA_Int16       rc_check_out;
BSA_Boolean     verbose;

*check       = RC_OK;
rc           = RC_OK;
rc_check     = RC_OK;
rc_check_out = RC_OK;
verbose      = get_verbose();

rc = ourGetObject(objectDescriptorPtr, dataBlockPtr);
rc_check = restore_object_info(objectDescriptorPtr,
                               seedStringPtr);
rc_check_out = rc_check_out || rc_check;
if (verbose)
{
   print_object_descriptor(objectDescriptorPtr);
} /* if */
seedlen = strlen(*seedStringPtr);
if (((rc == BSA_RC_OK) 
     || (rc == BSA_RC_MORE_DATA))
     && (seedlen > 0))
{
   rc_check = check_Data(dataBlockPtr, *seedStringPtr, offsetPtr);
} /* if */

rc_check_out = rc_check_out || rc_check;
*check = rc_check_out;
return rc;
} /* get_object_and_check() */

/*----------------------------------------------------------------------+
| Name:    get_all_data()
|
| Action:  Receives all data for an object from the server and checks 
|          data against the seed string
|
| Input:   *dataBlockPtr        - pointer to the data block
|          *seedString          - seed string
|          *offsetPtr           - pointer to an offset used for
|                                 validation
|
| Output:  *check               - RC_OK data in the buffer is O.K.
|                                 RC_CHECK_FAILURE validation failure
|
| Returns: RC_OK                - Successful
|          others               - Any other valid X/Open api rc.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 get_all_data        (dataBlockPtr,
                               seedString,
                               offsetPtr,
                               check)
   DataBlock       *dataBlockPtr;
   char            *seedString;
   BSA_UInt16      *offsetPtr;
   BSA_Int16       *check;
#else
BSA_Int16 get_all_data(DataBlock        *dataBlockPtr,
                       char             *seedString,
                       BSA_UInt16       *offsetPtr,
                       BSA_Int16        *check)
 #endif
{

BSA_Int16       rc;
BSA_Int16       rc_check;
BSA_Int16       seedlen;

*check   = RC_OK;
seedlen  = strlen(seedString);
rc_check = RC_OK;
rc       = BSA_RC_MORE_DATA;

while (rc == BSA_RC_MORE_DATA)
{
   rc = ourGetData(dataBlockPtr);
   if (((rc == BSA_RC_OK)
        || (rc == BSA_RC_MORE_DATA))
        && (seedlen > 0))
   {
      rc_check = rc_check || check_Data(dataBlockPtr, 
                                        seedString,
                                        offsetPtr);
   } /* if */
} /* while */ 

*check = rc_check; 

return rc;
} /* get_all_data() */

