/**********************************************************************
* Name:
*        xapisend.c
*
* Function:
*        Functions invoked via menu selections
*
* Description:
*        The file contains the functions that are invoked from the
*        menu processing code and tables.
***********************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "xapisend.h"       /* send objects                            */
#include "xapiutil.h"       /* Handy utility routines.                 */
#include "xapiour.h"        /* our API functions                       */
#include "policy.h"         /* XOpen API function prototypes           */
#include "xapint64.h"       /* 64 bit routines.                        */
#include "xapipref.h"       /* default values for buffer sizes         */

/*----------------------------------------------------------------------+
| Private routines
.----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static BSA_Int16 create_send_buffer();
static BSA_Int16 send_multiple_objects();
static BSA_Int16 send_object_and_data();
static BSA_Int16 send_all_data();
#else /* use ANSI C required prototypes */
static BSA_Int16 create_send_buffer(char       *seed_string,
                                    char       **send_bufferPtr,
                                    ObjectSize Size64);
static BSA_Int16 send_multiple_objects(char             *send_buffer,
                                       BSA_UInt64        number_of_files64,
                                       ObjectDescriptor *objectDescriptorPtr);
static BSA_Int16 send_object_and_data(char *send_buffer,
                          ObjectDescriptor *objectDescriptorPtr);
static BSA_Int16 send_all_data(char *send_buffer,
                          BSA_UInt64 bytes_to_send);
#endif

/*----------------------------------------------------------------------+
| Public routines
.----------------------------------------------------------------------*/

/*----------------------------------------------------------------------+
| Name:    perform_send()
|
| Action:  Take the information passed in the dialog to this function,
|          build the backup objects dynamically, and send them to the
|          server.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|          send_type   - Indicates whether archive or backup being done
|                        BSA_Backup_Send or BSA_Archive_Send
|
| Returns: RC_OK             - Successful
|          RC_NO_MEMORY      - Not enough memory
|          RC_NOT_IN_SESSION - Client is not in session with the server
|          RC_SESSION_FAILED - A X/Open api call failed stopping
|                              execution
|          RC_NO_SEED_STRING - No seed string is spezified
|          RC_LESS_BUFF_SIZE - Send buffer size is less than seed
|                              string size.
|          others            - Any other valid X/Open api rc.
|
| Side
| Effects: None
|
| Notes:   This routine handles 64bit objects.  Thus
|          the number of files to be sent and the size of the files to be
|          sent are 64bit numbers.  Memory buffers are still 32bit values.
|          Be very careful changing the numbers to insure a type mismatch
|          does not occur.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 perform_send(sel_dialog, send_type)
   dialog  *sel_dialog;
   BSA_UInt16  send_type;
#else /* use ANSI C required prototypes */
BSA_Int16 perform_send(dialog *sel_dialog,BSA_UInt16 send_type)
#endif

{

ObjectDescriptor *objectDescriptorPtr;
BSA_Int16        rc;
BSA_Int16        i;
dialog           *dlg;
BSA_UInt64       NumberOfFiles64;
char             *SeedString;
char             *SendBuffer;

/*----------------------------------------------------------------------
| Are we signed on? 
.----------------------------------------------------------------------*/
rc = check_signon();
if (rc)
{
   return rc;
}

/*----------------------------------------------------------------------
| Create an Object Descriptor and initialize the input parameters.
.----------------------------------------------------------------------*/
objectDescriptorPtr = create_objDesc();
if (objectDescriptorPtr == NULL)
{
   /* no memory available */
   rc = RC_NO_MEMORY;
   return rc;
} /* if */

switch (send_type)
{
   case BSA_Backup_Send:
      objectDescriptorPtr -> copyType = BSACopyType_BACKUP;
      break;
   case BSA_Archive_Send:
      objectDescriptorPtr -> copyType = BSACopyType_ARCHIVE;
      break;
   default:
      printf("\nError in code perform_send!\n\n");
      objectDescriptorPtr -> copyType = BSACopyType_BACKUP;
      break;
} /* switch */

/*----------------------------------------------------------------------
| Now pull all values out of the passed dialog for our use.
.----------------------------------------------------------------------*/
i = 0;
dlg = &sel_dialog[i];
while (dlg->item_type != BSA_API_END)
{
   switch (dlg->item_type)
   {
      case BSA_API_OBJSPACE :
         strcpy(objectDescriptorPtr->objName.objectSpaceName,
                dlg->item_buff);
         break;
      case BSA_API_PATH :
         strcpy(objectDescriptorPtr->objName.pathName,
                dlg->item_buff);
         break;
      case BSA_API_OBJTYPE :
         switch (*dlg->item_buff)
         {
            case 'A':
            case 'a':
               objectDescriptorPtr->objectType =  BSAObjectType_ANY;
               break;
            case 'F':
            case 'f':
               objectDescriptorPtr->objectType =  BSAObjectType_FILE;
               break;
            case 'D':
            case 'd':
               objectDescriptorPtr->objectType =  BSAObjectType_DIRECTORY;
               break;
            case 'B':
            case 'b':
               objectDescriptorPtr->objectType =  BSAObjectType_DATABASE;
               break;
            default:
               objectDescriptorPtr->objectType =  BSAObjectType_FILE;
               break;
         }
         break;
      case BSA_API_RESTYPE :
         strncpy(objectDescriptorPtr->resourceType,
                 dlg->item_buff,
                 BSA_MAX_RESOURCETYPE-1);
         break;
      case BSA_API_BSAOBJOWN :
         strcpy(objectDescriptorPtr->Owner.bsaObjectOwner,dlg->item_buff);
         break;
      case BSA_API_APPOBJOWN :
         strcpy(objectDescriptorPtr->Owner.appObjectOwner,dlg->item_buff);
         break;
      case BSA_API_FILESIZE :
         objectDescriptorPtr->size = ChtoI64(dlg->item_buff,10);
         break;
      case BSA_API_NUM_FILES :
         NumberOfFiles64 = ChtoI64(dlg->item_buff,10);
         break;
      case BSA_API_SEED_STRING :
         SeedString = dlg->item_buff;
         break;
      case BSA_API_DESCRIPTION :
         strcpy(objectDescriptorPtr->desc,dlg->item_buff);
         break;
      case BSA_API_OBJINFO :
         strcpy(objectDescriptorPtr->objectInfo,dlg->item_buff);
         break;
      default :
         ;
   }
   i++;
   dlg = &sel_dialog[i];
}

/*----------------------------------------------------------------------
| Store eyecatcher, object size, copyType, seed string, and object
| info in the object info field for verification after the object has
| been received. 
.----------------------------------------------------------------------*/
rc = build_object_info(objectDescriptorPtr,
                       SeedString);
if (!rc)
{
   /*-------------------------------------------------------------------
   | Now create the send buffer. 
   .-------------------------------------------------------------------*/
   rc = create_send_buffer( SeedString,
                            &SendBuffer,
                            objectDescriptorPtr->size );
   if (!rc)
   {
      /*----------------------------------------------------------------
      | Send all the Objects.
      .----------------------------------------------------------------*/
      rc = send_multiple_objects( SendBuffer,
                                  NumberOfFiles64,
                                  objectDescriptorPtr );
   } /* if */
} /* if */

/*----------------------------------------------------------------------
| Do not forget to free the send buffer and the object descriptor. 
.----------------------------------------------------------------------*/
dispose_buffer( SendBuffer );
dispose_objDesc( objectDescriptorPtr );

return rc;
}


/*----------------------------------------------------------------------+
| Private routines
.----------------------------------------------------------------------*/

/*----------------------------------------------------------------------+
| Name:    create_send_buffer() 
|
| Action:  Allocates memory for a send buffer and fills the buffer  
|          with the seed string. The size of the buffer defined 
|          in the preferences structure will be used to determine
|          the actual size of the buffer. Thereafter the send buffer
|          size in the preference structure will be updated.
|
| Input:   SeedString    - String to fill the buffer 
|          Size64        - Size of the Object(s) to send
|  
|
| Returns: RC_OK             - Successful
|          RC_NO_MEMORY      - Not enough memory to create a send buffer 
|          RC_NO_SEED_STRING - No seed string is spezified
|          RC_LESS_BUFF_SIZE - Send buffer size is less than seed    
|                              string size.
|          others            - Any other valid X/Open api rc.
|          SendBufferPtr     - Pointer to the actual send buffer string
|
| Side
| Effects: None
|
| Notes:   The size of the send buffer in the preference structure
|          will be modified.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 create_send_buffer(SeedString, SendBufferPtr, Size64)
   char       *SeedString;
   char       **SendBuffer;
   ObjectSize Size64;
#else /* use ANSI C required prototypes */
BSA_Int16 create_send_buffer(char       *SeedString, 
                             char       **SendBuffer,
                             ObjectSize Size64)
#endif

{

BSA_Int16  SeedSize;
BSA_UInt32 SendBufferSize;
BSA_UInt32 actBufferSize;

SeedSize       = strlen(SeedString);
SendBufferSize = get_send_buff_size();

if (Size64.left || Size64.right) /* Any data to send */
{
   if (!SeedSize)
   {
      printf("*** Error: Send action terminating since no seed\n");
      printf("string was specified for the object data.\n");
      return RC_NO_SEED_STRING;
   } /* if */
   if (SendBufferSize < SeedSize)
   {
      printf("*** Error: Send action terminating since send buffer size");
      printf("(%lu)\n is less than seed string size (%lu).\n",
           SendBufferSize, SeedSize);
      return RC_LESS_BUFF_SIZE;
   } /* if */

   actBufferSize = (SendBufferSize / SeedSize) * SeedSize;
   /*printf("The actual send buffer size is: %lu\n", actBufferSize);*/
   set_send_buff_size(actBufferSize);
   *SendBuffer = create_buffer( actBufferSize );
   if (*SendBuffer == NULL)
   { 
      return RC_NO_MEMORY;
   } /* if */

   cpy_seed_to_buffer(*SendBuffer, SeedString, actBufferSize);
 
} /* if */

return RC_OK; 
}

/*----------------------------------------------------------------------+
| Name:    send_multiple_objects() 
|
| Action:  Sends all the objects to the server using the buffer. 
|
| Input:   SendBuffer          - Buffer used for BSACreateObject call 
|                                and BSASendData call
|          NumberOfFiles64     - Number of files that will be sent to
|                                the server.
|          objectDescriptorPtr - X/Open API data structure describing
|                                an object 
|
|
| Returns: RC_OK             - Successful
|          others            - Any other valid X/Open api rc.
|
| Side
| Effects: None
|
| Notes:   This function starts a transaction and sends all objects 
|          and their data to the server. If a failure occurs 
|          during the send operation the transaction will be
|          aborted.
|          In case no error occurs the transaction will be ended 
|          normally.
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 send_multiple_objects( SendBuffer,
                                 NumberOfFiles64,
                                 objectDescriptorPtr )
   char             *SendBuffer;
   BSA_UInt64       NumberOfFiles64;
   ObjectDescriptor *objectDescriptorPtr; 
#else /* use ANSI C required prototypes */
BSA_Int16 send_multiple_objects( 
                     char             *SendBuffer,
                     BSA_UInt64       NumberOfFiles64, 
                     ObjectDescriptor *objectDescriptorPtr )
#endif

{

BSA_UInt16 rc;
char       NumberString1[50];
char       NumberString2[50];
char       NumberString3[50];
char       NumberString4[50];
BSA_UInt64 Files64;
char       OldPathname[BSA_MAX_PATHNAME];
BSA_UInt64 MaxObj64;
BSA_UInt64 *MaxObj64Ptr;
BSA_UInt64 Result64;
BSA_UInt64 Rmdr64;
BSA_UInt64 *RmdrPtr;

MaxObj64Ptr = &MaxObj64;
Rmdr64 = Make64(0);
RmdrPtr = &Rmdr64;
strcpy( OldPathname, 
        objectDescriptorPtr->objName.pathName ); 

I64toCh(NumberOfFiles64, NumberString3, 10);  
format_number(NumberString3, NumberString1);
I64toCh(objectDescriptorPtr->size, NumberString3, 10);  
format_number(NumberString3,NumberString2);
printf("Creating %s object(s) called %s %s(nnn) each of size %s.\n",
       NumberString1,
       objectDescriptorPtr->objName.objectSpaceName,
       objectDescriptorPtr->objName.pathName, 
       NumberString2);

/*----------------------------------------------------------------------+
| Now we are set to start sending each object to the server within
| one single transaction. 
.----------------------------------------------------------------------*/
rc = ourBeginTxn();
if (rc)
{
   return rc;
}
/*----------------------------------------------------------------------+
| Get the maximum number of objects in one transaction.
.----------------------------------------------------------------------*/
rc = GetMaxObj(MaxObj64Ptr);
if (rc)
{
   ourEndTxn( BSAVote_ABORT );    /* abort the transaction and  */
   return rc;                     /* return the code of the     */
                                  /* failed call                */
}

/*----------------------------------------------------------------------+
| This is the main loop for each object to be sent.  Since 64bit numbers
| are supported we could be here a long time!
.----------------------------------------------------------------------*/
for (Files64.left=0, Files64.right=1;  /* Init to 1 start       */
                                       /* Still more to process?*/
     !(Cmp64(Files64,NumberOfFiles64) == GREATERTHAN);
     Files64 = Incr64(Files64,1))      /* Bump counter by 1.    */
{
   /*--------------------------------------------------------------------+
   | First create the name of the object to send and init object counters.
   | If we were asked to create more than 1 file in this round then we will
   | add a sequence number onto the end of each pathname qualifier to
   | uniquely identify it to the system.
   .--------------------------------------------------------------------*/
   if ((NumberOfFiles64.right > 1) || (NumberOfFiles64.left > 0))
   {
      strcpy( objectDescriptorPtr->objName.pathName,
              OldPathname );
      I64toCh(Files64,NumberString3,10);
      strcat(objectDescriptorPtr->objName.pathName,NumberString3);
   }

   /*-------------------------------------------------------------------+
   | You can send only MaxObj64 objects within one transaction.
   | Start a new transaction whenever you have more objects.
   .-------------------------------------------------------------------*/
   Result64 = Div64(Files64, MaxObj64, RmdrPtr);
   if ((Cmp64(*RmdrPtr, Make64(1)) == EQUAL) &&
       !(Cmp64(Files64, Make64(1)) == EQUAL))
   {
      rc = ourEndTxn( BSAVote_COMMIT );
      if (rc)
      {
         return rc;
      } /* if */
      rc = ourBeginTxn();
      if (rc)
      {
         return rc;
      } /* if */
      printf("Starting a new transaction.\n");
      rc = GetMaxObj(MaxObj64Ptr);
      if (rc)
      {
         ourEndTxn( BSAVote_ABORT );    /* abort the transaction and  */
         return rc;                     /* return the code of the     */
                                        /* failed call                */
      } /* if */
   } /* if */

   /* Convert 64bit nums to printable strings and format them */
   I64toCh(NumberOfFiles64,NumberString4,10); 
   format_number(NumberString4,NumberString1);
   I64toCh(objectDescriptorPtr->size,NumberString4,10);
   format_number(NumberString4,NumberString2);
   I64toCh(Files64,NumberString4,10);  
   format_number(NumberString4,NumberString3);

   printf("Creating object %s of %s   Size=%s    Name=%s %s\n",
          NumberString3,
          NumberString1,
          NumberString2,
          objectDescriptorPtr->objName.objectSpaceName,
          objectDescriptorPtr->objName.pathName);

   /*-------------------------------------------------------------------+
   | Any application should use the BSAResolveLifecycleGroup before
   | creating an object to determine its lifecycle group and to 
   | determine if the object should be created.
   .-------------------------------------------------------------------*/
   rc = ourResolveLifecycleGroup(objectDescriptorPtr);
   if (rc)
   {
      ourEndTxn( BSAVote_ABORT );    /* abort the transaction and  */
      return rc;                     /* return the code of the     */ 
                                     /* failed call                */
   }
    
   /*----------------------------------------------------------------------+
   | Send the object and all data belonging to it to the server.
   .----------------------------------------------------------------------*/
   rc = send_object_and_data( SendBuffer,
                              objectDescriptorPtr );
   if (rc)
   {
      ourEndTxn( BSAVote_ABORT );    /* abort the transaction and  */
      return rc;                     /* return the code of the     */
                                     /* failed call                */
   }

} /* main loop */

/*----------------------------------------------------------------------+
| End the transaction 
.----------------------------------------------------------------------*/
rc = ourEndTxn( BSAVote_COMMIT );
if (rc)
{
   return rc;
}

return RC_OK;
} /* send_multiple_objects */

/*----------------------------------------------------------------------+
| Name:   send_object_and_data()
|
| Action: Creates an object and sends all data to the server 
|
| Input:  SendBuffer          - Buffer with data to send
|         objectDescriptorPtr - This parameter points to a structure
|                               that contains the attributes of the
|                               object.
|
|
| Returns: RC_OK              - Successful
|          others             - Any other valid X/Open api rc.
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 send_object_and_data()
   char             *SendBuffer;
   ObjectDescriptor *objectDescriptorPtr;
#else /* use ANSI C required prototypes */
BSA_Int16 send_object_and_data( char *SendBuffer,
                    ObjectDescriptor *objectDescriptorPtr )
#endif

{

BSA_Int16     rc;
char          NumberString1[50];
BSA_UInt64    help64;
BSA_UInt64    BufferSize64;
char          *Cursor;
BSA_UInt64    ActSize64;
BSA_UInt64    BytesLeft64;
BSA_UInt32    SendBufferSize;

SendBufferSize = get_send_buff_size();
/*----------------------------------------------------------------------+
| Now we are ready to create an object.
.----------------------------------------------------------------------*/
rc = ourCreateObject( SendBuffer, objectDescriptorPtr );
if (rc)
{
   return rc;
}

/*----------------------------------------------------------------------+
| Send all data of the object to the server. 
.----------------------------------------------------------------------*/
BufferSize64 = Make64( SendBufferSize );
ActSize64    = objectDescriptorPtr->size;
if ( Cmp64(ActSize64, BufferSize64) == GREATERTHAN)
{
   BytesLeft64 = ActSize64;
   /* fist data has been sent with the BSACreateObject() call */
   BytesLeft64 = Sub64( BytesLeft64, BufferSize64 );
   rc = send_all_data( SendBuffer, BytesLeft64 );
   if (rc)
   {
      return rc;
   }
}

/*----------------------------------------------------------------------+
| Indicate the end of data for our object. 
.----------------------------------------------------------------------*/
rc = ourEndData();
if (rc)
{
   return rc;
}

return RC_OK;
}

/*----------------------------------------------------------------------+
| Name:   send_all_data()
|
| Action: Sends all the data of an object to the server
|
| Input:  SendBuffer          - Buffer with data to send
|         BytesToSend         - spezifies the number of bytes sent to
|                               the server
|
|
| Returns: RC_OK              - Successful
|          others             - Any other valid X/Open api rc.
|
| Side
| Effects: None
|
| Notes:   If you need to send more bytes than the actual buffer
|          length is then this function will send the buffer as
|          many times as needed. 
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 send_all_data() 
   char       *SendBuffer;
   BSA_UInt64 BytesToSend; 
#else /* use ANSI C required prototypes */
BSA_Int16 send_all_data( char *SendBuffer,
                   BSA_UInt64 BytesToSend ) 
#endif

{

BSA_Int16     rc;
BSA_UInt64    BufferSize64;
BSA_UInt64    BytesLeft64;
BSA_UInt32    SendBufferSize;

SendBufferSize = get_send_buff_size();
BufferSize64   = Make64( SendBufferSize );
BytesLeft64    = BytesToSend;

while (BytesLeft64.left || BytesLeft64.right)
{
   if ( Cmp64(BytesLeft64, BufferSize64) == LESSTHAN)
   {   
      
      rc = ourSendData(SendBuffer, BytesLeft64.right);
                                   /* Only need low 4 bytes here. */
      BytesLeft64 =Sub64(BytesLeft64, BytesLeft64);
   }
   else
   {
      rc = ourSendData(SendBuffer, SendBufferSize);
      BytesLeft64 =Sub64(BytesLeft64, BufferSize64);
   }
   if (rc)
   {
      return rc;
   }
}

return RC_OK;
}

