/***********************************************************************
* Name:
*        xapisess.c
*
* Function:
*        Perform a signon to the XBSA server via the XOpen
*        API functions.
*
* Environment:
*        This is a PLATFORM-INDEPENDENT source file. As such it may
*        contain no dependencies on any specific operating system
*        environment or hardware platform.
*
* Description:
*        Call the BSAInit function to perform a signon to the
*        XBSA server.
***********************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "custom.h"         /* Definitions for platform specific types */
#include "xapisess.h"       /* session control                         */
#include "xapiour.h"        /* our API functions                       */
#include "policy.h"         /* XOpen API function prototypes           */
#include "xapidef.h"        /* XOpen Sample API type definitions       */
#include "xapint64.h"       /* 64 bit routines.                        */

/*----------------------------------------------------------------------+
| Public routines
.----------------------------------------------------------------------*/

/*----------------------------------------------------------------------+
| Name:    perform_signon()
|
| Action:  Initializes the environment and sets up a session. 
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|          RC_NO_MEMORY     - There is not enough memory available.
|          others           - Any other valid X/Open api rc.
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 perform_signon(sel_dialog, parm1)
   dialog *sel_dialog;
   BSA_UInt16 parm1;
#else /* use ANSI C required prototypes */
BSA_Int16 perform_signon(dialog *sel_dialog,BSA_UInt16 parm1)
#endif

{
ObjectOwner    *objectOwnerPtr;
SecurityToken  *tokenPtr;
dialog         *dlg;
BSA_UInt32     i;
BSA_Int16      rc = 0;
char           t[100];
BSA_Int16      len;
BSA_Int16      j;

/* Allocate some memory. */
objectOwnerPtr = (ObjectOwner*)malloc( sizeof( ObjectOwner));
if (objectOwnerPtr == NULL)
  {
  /* no memory available */
  rc = RC_NO_MEMORY;
  return rc;
  }
 
/* First pull all values out of the passed dialog for our use.  */
i = 0;
dlg = &sel_dialog[i];
while (dlg->item_type != BSA_API_END)
  {
  switch (dlg->item_type)
    {
    case BSA_API_NODENAME :
      strcpy(objectOwnerPtr->bsaObjectOwner, dlg->item_buff);
      break;
    case BSA_API_OWNER :
      strcpy(objectOwnerPtr->appObjectOwner, dlg->item_buff);
      break;
    case BSA_API_PASSWORD :
      tokenPtr = (SecurityToken*)dlg->item_buff;
      break;
    default :
      printf("*** Signon dialog does not match code in perform_signon! ***\n");
    }
  i++;
  dlg = &sel_dialog[i];
  }

printf("Doing signon for node ");
if (strlen(objectOwnerPtr->bsaObjectOwner) == 0)
{
   printf("<NULL>,");
}
else
{
   printf("%s,", objectOwnerPtr->bsaObjectOwner);
} /* if */
if (strlen(objectOwnerPtr->appObjectOwner) == 0)
{
   printf(" owner <NULL>,");
}
else
{
   printf(" owner %s,", objectOwnerPtr->appObjectOwner);
} /* if */
if (strlen((char *)tokenPtr) == 0)
{
   printf(" password <NULL>\n");
}
else
{
   printf(" password %s\n", tokenPtr);
} /* if */

rc = ourInit(tokenPtr,        /* password                               */
             objectOwnerPtr); /* Node name and Owner name               */

/* Free the memory. */
free(objectOwnerPtr);

return RC_OK;
}

/*----------------------------------------------------------------------+
| Name:    perform_chgpw()
|
| Action:  Changes the signon password.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK            - Successful
|          RC_NO_MEMORY     - There is not enough memory available.
|          others           - Any other valid X/Open api rc.
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 perform_chgpw(sel_dialog, parm1)
   dialog *sel_dialog;
   BSA_UInt16 parm1;
#else /* use ANSI C required prototypes */
BSA_Int16 perform_chgpw(dialog *sel_dialog,BSA_UInt16 parm1)
#endif

{
SecurityToken  *oldTokenPtr;
SecurityToken  *newTokenPtr1;
SecurityToken  *newTokenPtr2;
dialog         *dlg;
BSA_UInt32     i;
BSA_Int16      rc = RC_OK;

/* Allocate some memory. */
oldTokenPtr  = (SecurityToken*)malloc(BSA_MAX_TOKEN_SIZE);
newTokenPtr1 = (SecurityToken*)malloc(BSA_MAX_TOKEN_SIZE);
newTokenPtr2 = (SecurityToken*)malloc(BSA_MAX_TOKEN_SIZE);
if (((oldTokenPtr  == NULL)  ||
     (newTokenPtr1 == NULL)) ||
     (newTokenPtr2 == NULL))
{
   /* no memory available */
   rc = RC_NO_MEMORY;
}
else
{ 
   /* First pull all values out of the passed dialog for our use.  */
   i = 0;
   dlg = &sel_dialog[i];
   while (dlg->item_type != BSA_API_END)
     {
     switch (dlg->item_type)
       {
       case BSA_API_CURRENTPW :
         strncpy((char *)oldTokenPtr,
                 dlg->item_buff,
                 BSA_MAX_TOKEN_SIZE - 1);
         break;
       case BSA_API_NEWPW1 :
         strncpy((char *)newTokenPtr1, 
                 dlg->item_buff,
                 BSA_MAX_TOKEN_SIZE - 1);
         break;
       case BSA_API_NEWPW2 :
         strncpy((char *)newTokenPtr2,
                 dlg->item_buff,
                 BSA_MAX_TOKEN_SIZE - 1);
         break;
       default :
         printf("*** Change password dialog does not match code in perform_chgpw! ***\n");
       } /* switch */
     i++;
     dlg = &sel_dialog[i];
   } /* while */
   if ((strcmp((char *)newTokenPtr1, (char *)newTokenPtr2)))
   {
      printf("Your new passwords do not match.\n");
      rc = RC_OK;
   }
   else
   {
      rc = ourChangeToken(oldTokenPtr, newTokenPtr1);
   } /* if */
} /* if */

/* Free the memory. */
free(oldTokenPtr);
free(newTokenPtr1);
free(newTokenPtr2);

return rc;
} /* perform_chgpw() */

/*----------------------------------------------------------------------+
| Name:    perform_exit()
|
| Action:  Exits from the sample application.
|
| Input:   None
|
| Returns: RC_OK            - Successful
|
| Output:  None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 perform_exit()
#else /* use ANSI C required prototypes */
BSA_Int16 perform_exit()
#endif

{

if (check_signon() == RC_OK)
{
   ourTerminate();
} /* if */

return RC_OK;
} /* perform_exit() */

/*----------------------------------------------------------------------+
| Name:    perform_qry_version()
|
| Action:  Prints the installed API verion.
|
| Input:   None
|
| Output:  None
|
| Returns: RC_OK            - Successful
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 perform_qry_verion()
#else /* use ANSI C required prototypes */
BSA_Int16 perform_qry_version()
#endif

{

ApiVersion apiLibVer;
ourQueryApiVersion(&apiLibVer);
printf("\nAPI Version = %d.%d.%d\n\n",apiLibVer.version,
                                      apiLibVer.release,
                                      apiLibVer.level);

return RC_OK;
} /* perform_qry_version() */

/*----------------------------------------------------------------------+
| Name:    perform_qry_sess()
|
| Action:  Prints the current session information.
|
| Input:   None
|
| Output:  None
|
| Returns: RC_OK             - Successful
|          RC_NO_MEMORY      - Not enough memory
|          others            - Any other valid X/Open api rc.
|
| Side
| Effects: None
|
| Notes:   This call identifies the Backup Services instance and
|          other implementation-dependent variables such as
|          communication ports, etc.  Each variable is a
|          (keyword, value) pair separated by a space.
|          If a value contains spaces, it must be enclosed in
|          single or double quotes.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 perform_qry_sess()
#else /* use ANSI C required prototypes */
BSA_Int16 perform_qry_sess()
#endif

{

BSA_Int16      rc = RC_OK;
ObjectOwner    *objectOwnerPtr;
char           *envPtr[ADSM_ENV_STRS+1];
char           envStrs[ADSM_ENV_STRS] [BSA_MAX_DESC];
BSA_Int16      i;

for (i=0; i<ADSM_ENV_STRS; i++)
{
   envPtr[i] = envStrs[i];
} /* for */
envPtr[ADSM_ENV_STRS] = NULL;
objectOwnerPtr = (void *)create_objOwner();
if (objectOwnerPtr != NULL)
{
   rc = ourGetEnvironment(objectOwnerPtr, envPtr);

   printf("ADSM node/bsaObjectOwner = %s\n",objectOwnerPtr->bsaObjectOwner);
   printf("ADSM owner/appObjectOwner = %s\n",objectOwnerPtr->appObjectOwner);
   printf("\nBSA Environment:\n");
   for (i=0; i<ADSM_ENV_STRS; i++)
      printf(" %s\n", envPtr[i]);

   printf("\n");   
   dispose_objOwner(objectOwnerPtr);
}
else
{
   rc = RC_NO_MEMORY;
} /* if */

return rc;
} /* perform_qry_sess() */

/*----------------------------------------------------------------------+
| Name:    perform_pref_query()
|
| Action:  Prints the current preference settings.
|
| Input:   None
|
| Output:  None
|
| Returns: RC_OK            - Successful
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 perform_pref_query()
#else /* use ANSI C required prototypes */
BSA_Int16 perform_pref_query()
#endif

{

BSA_UInt32  send_buff;
BSA_UInt32  recv_buff;
BSA_Boolean verbose;
BSA_Boolean qry_details;

send_buff   = get_send_buff_size();
recv_buff   = get_recv_buff_size();
verbose     = get_verbose();
qry_details = get_qry_detail();
printf("        Send buffer size: %lu\n",send_buff);
printf("     Receive buffer size: %lu\n",recv_buff);
printf("         Verbose flag is: ");
if (verbose == TRUE)
{
   printf("On\n");
}
else
{
   printf("Off\n");
} /* if */
printf("  Display query details?: ");
if (qry_details == TRUE)
{
   printf("Yes\n");
}
else
{
   printf("No\n");
} /* if */

return RC_OK;
} /* perform_pref_query() */

/*----------------------------------------------------------------------+
| Name:    perform_pref_sendbuff()
|
| Action:  Changes the size of the send buffer.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Output:  None
|
| Returns: RC_OK            - Successful
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 perform_pref_sendbuff(sel_dialog, parm1)
   dialog *sel_dialog;
   BSA_UInt16 parm1;
#else /* use ANSI C required prototypes */
BSA_Int16 perform_pref_sendbuff(dialog *sel_dialog,BSA_UInt16 parm1)
#endif

{

dialog         *dlg;
BSA_UInt32     i;
BSA_UInt32     send_buffer_size = 1;

/* First pull all values out of the passed dialog for our use.  */
i = 0;
dlg = &sel_dialog[i];
while (dlg->item_type != BSA_API_END)
{
   switch (dlg->item_type)
   {
      case BSA_API_SENDBUFFSIZE:
         send_buffer_size = atol(dlg->item_buff);
         break;
       default :
         printf("*** Preference send buffer dialog does not match ");
         printf("code in perform_pref_sendbuff! ***\n");
   } /* switch */
   i++;
   dlg = &sel_dialog[i];
} /* while */
if (send_buffer_size >= 1)
{
   set_send_buff_size(send_buffer_size);
} /* if */

return RC_OK;
} /* perform_pref_sendbuff() */

/*----------------------------------------------------------------------+
| Name:    perform_pref_recvbuff()
|
| Action:  Changes the size of the receive buffer.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Output:  None
|
| Returns: RC_OK            - Successful
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 perform_pref_recvbuff(sel_dialog, parm1)
   dialog *sel_dialog;
   BSA_UInt16 parm1;
#else /* use ANSI C required prototypes */
BSA_Int16 perform_pref_recvbuff(dialog *sel_dialog,BSA_UInt16 parm1)
#endif

{

dialog         *dlg;
BSA_UInt32     i;
BSA_UInt32     recv_buffer_size = 1;

/* First pull all values out of the passed dialog for our use.  */
i = 0;
dlg = &sel_dialog[i];
while (dlg->item_type != BSA_API_END)
{
   switch (dlg->item_type)
   {
      case BSA_API_RECVBUFFSIZE:
         recv_buffer_size = atol(dlg->item_buff);
         break;
       default :
         printf("*** Preference receive buffer dialog does not match ");
         printf("code in perform_pref_recvbuff! ***\n");
   } /* switch */
   i++;
   dlg = &sel_dialog[i];
} /* while */
if (recv_buffer_size >= 1)
{
   set_recv_buff_size(recv_buffer_size);
} /* if */

return RC_OK;
} /* perform_pref_recvbuff() */

/*----------------------------------------------------------------------+
| Name:    perform_pref_verbose()
|
| Action:  Changes the status of the verbose flag.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Output:  None
|
| Returns: RC_OK            - Successful
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 perform_pref_verbose(sel_dialog, parm1)
   dialog *sel_dialog;
   BSA_UInt16 parm1;
#else /* use ANSI C required prototypes */
BSA_Int16 perform_pref_verbose(dialog *sel_dialog,BSA_UInt16 parm1)
#endif

{

dialog         *dlg;
BSA_UInt32     i;
BSA_Boolean    verbose = TRUE;

/* First pull all values out of the passed dialog for our use.  */
i = 0;
dlg = &sel_dialog[i];
while (dlg->item_type != BSA_API_END)
{
   switch (dlg->item_type)
   {
      case BSA_API_VERBOSEFLAG:
         switch (*dlg->item_buff)
         {
            case 'V':
            case 'v':
               verbose = TRUE;
               break;
            case 'Q':
            case 'q':
               verbose = FALSE;
               break;
            default:
               verbose = TRUE;
               break;
         } /* switch */
         break;
       default :
         printf("*** Preference verbose dialog does not match ");
         printf("code in perform_pref_verbose! ***\n");
   } /* switch */
   i++;
   dlg = &sel_dialog[i];
} /* while */
set_verbose(verbose);

return RC_OK;
} /* perform_pref_verbose() */

/*----------------------------------------------------------------------+
| Name:    perform_pref_details()
|
| Action:  Changes the status of the details flag.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Output:  None
|
| Returns: RC_OK            - Successful
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 perform_pref_details(sel_dialog, parm1)
   dialog *sel_dialog;
   BSA_UInt16 parm1;
#else /* use ANSI C required prototypes */
BSA_Int16 perform_pref_details(dialog *sel_dialog,BSA_UInt16 parm1)
#endif

{

dialog         *dlg;
BSA_UInt32     i;
BSA_Boolean    details = TRUE;

/* First pull all values out of the passed dialog for our use.  */
i = 0;
dlg = &sel_dialog[i];
while (dlg->item_type != BSA_API_END)
{
   switch (dlg->item_type)
   {
      case BSA_API_DETAILSFLAG:
         switch (*dlg->item_buff)
         {
            case 'Y':
            case 'y':
               details = TRUE;
               break;
            case 'N':
            case 'n':
               details = FALSE;
               break;
            default:
               details = TRUE;
               break;
         } /* switch */
         break;
       default :
         printf("*** Preference details dialog does not match ");
         printf("code in perform_pref_details! ***\n");
   } /* switch */
   i++;
   dlg = &sel_dialog[i];
} /* while */
set_qry_detail(details);

return RC_OK;
} /* perform_pref_details() */

/*----------------------------------------------------------------------+
| Name:    GetMaxObj
|
| Action:  Gets the maximum number of objects in one transaction
|          from the environment string.
|
| Input:   None
|
| Output:  *MaxObj64        - Maximum number of objects  
|
| Returns: RC_OK            - Successful
|          RC_NO_MEMORY     - Not enough memory
|          others           - Any other valid X/Open api rc.
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 GetMaxObj(MaxObj64Ptr)
   BSA_UInt64 *MaxObj64Ptr;
#else /* use ANSI C required prototypes */
BSA_Int16 GetMaxObj(BSA_UInt64 *MaxObj64Ptr)
#endif

{

BSA_Int16      rc = RC_OK;
ObjectOwner    *objectOwnerPtr;
char           *MaxString;
char           *NumberString;
int            End;
BSA_UInt64     Result;
char           *envPtr[ADSM_ENV_STRS+1];
char           envStrs[ADSM_ENV_STRS] [BSA_MAX_DESC];
BSA_Int16      i, notequal, maxLen;

for (i=0; i<ADSM_ENV_STRS; i++)
{
   memset(envStrs[i], 0x00, BSA_MAX_DESC);
} /* for */
for (i=0; i<ADSM_ENV_STRS; i++)
{
   envPtr[i] = envStrs[i];
} /* for */
envPtr[ADSM_ENV_STRS] = NULL;
MaxString = "MAXOBJ=";
maxLen = strlen(MaxString);
objectOwnerPtr = (void *)create_objOwner();
if (objectOwnerPtr != NULL)
{
   rc = ourGetEnvironment(objectOwnerPtr, envPtr);
   i = 0;
   notequal = memcmp(envPtr[i], MaxString, maxLen);
   while ((i<ADSM_ENV_STRS) && notequal)
   {
      i++;
      notequal = memcmp(envPtr[i], MaxString, maxLen);
   } /* while */
   NumberString = envPtr[i];
   if (notequal)
   {
      printf("Environment string does not contain MAXOBJ parameter.\n");
      Result = Make64(0);
   }
   else
   {
      if (maxLen == strlen(NumberString))
      {
         printf("MAXOBJ parameter in environment string does not contain a value.\n");
         Result = Make64(0);
      }
      else
      {
         NumberString = &NumberString[maxLen];
         End          = strspn(NumberString, "0123456789");
         NumberString[End] = 0x00;
         Result = ChtoI64(NumberString, 10);
      } /* if */
   } /* if */
   dispose_objOwner(objectOwnerPtr);
}
else
{
   rc = RC_NO_MEMORY;
} /* if */
*MaxObj64Ptr = Result;

return rc;
} /* GetMaxObj() */

