/***********************************************************************
* Name:
*        xapismp.c
*
* Function:
*        Sample XOpen API application.  Main entry point and top level
*        functions.
*
* Status:
*        Version 1, Release 0
*
* Environment:
*        This is a PLATFORM-INDEPENDENT source file. As such it may
*        contain no dependencies on any specific operating system
*        environment or hardware platform.
*
* Description:
*        The file contains the functions that are invoked from the
*        menu processing code and tables.
***********************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "xapiour.h"       /* our API functions                        */
#include "xapipref.h"      /* default values for buffer sizes          */
#include "xapidisp.h"      /* Display menus and accept user input      */

/*----------------------------------------------------------------------+
| Name:    main()
|
| Action:  Main entry point for the XOpen sample API application.
|          Put out welcome banner, get API version, and start menu
|          processing for the application.
|
| Input:   None
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int
main(argc, argv)
   int  argc;
   char **argv;
#else /* use ANSI C required prototypes */ 
int
main(int argc,char **argv)
#endif

{
int           rc;
ApiVersion    apiLibVer;    /* version of API Library at run time    */

set_preferences();        /* Set default values that user can modify */

ourQueryApiVersion(&apiLibVer);

/* check for compatibility problems */
if (apiLibVer.version < BSA_API_VERSION)
{ printf("The XOpen API library is at a lower version than the application.\n");
  printf("Install the current version.\n");
  return;
}
if ((apiLibVer.version == BSA_API_VERSION) &&
    (apiLibVer.release < BSA_API_RELEASE))
{ printf("\n***************************************************************\n");
  printf("The XOpen API library is at a lower release than the application.\n");
  printf("Proceed with caution.\n");
  /* Make sure you know what changes you have incorporated from the  */
  /* later release to ensure compatibility.                          */
}

printf("\n*************************************************************\n");
printf(  "*   Welcome to the sample application for the XOpen API.    *\n");
printf(  "*   The XOpen API uses ADSM services.                       *\n");
printf("*   API Library Version = %d.%d.%d                             *\n",
     apiLibVer.version,
     apiLibVer.release,
     apiLibVer.level);
printf("*************************************************************\n");

Process_Main_Menu();   /* Go process the menu and functions. */
}

